@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2011 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2024 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -20,11 +21,13 @@
 
 GBLREF spdesc stringpool;
 
-void cg_var(mvar *v, var_tabent **p)
+void cg_var(mtreenode *node, void *var_tabent_arg)
 {	/* Copy mident with variable name to variable table entry */
-	assert(stringpool.base <= (unsigned char *)v->mvname.addr && (unsigned char *)v->mvname.addr < stringpool.top);
-	(*p)[v->mvidx].var_name = v->mvname;
-	COMPUTE_HASH_MNAME(&((*p)[v->mvidx]));
-	(*p)[v->mvidx].var_name.addr = (char *)(v->mvname.addr - (char *)stringpool.base);
-	(*p)[v->mvidx].marked = FALSE;
+	var_tabent **p = var_tabent_arg;
+
+	assert(stringpool.base <= (unsigned char *)node->var.mvname.addr && (unsigned char *)node->var.mvname.addr < stringpool.top);
+	(*p)[node->var.mvidx].var_name = node->var.mvname;
+	COMPUTE_HASH_MNAME(&((*p)[node->var.mvidx]));
+	(*p)[node->var.mvidx].var_name.addr = (char *)(node->var.mvname.addr - (char *)stringpool.base);
+	(*p)[node->var.mvidx].marked = FALSE;
 }
