@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2022 Fidelity National Information	*
+ * Copyright (c) 2001-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -11,7 +11,6 @@
  ****************************************************************/
 
 #include "mdef.h"
-
 #include "gtm_stdio.h"
 #include "io.h"
 #include <rtnhdr.h>
@@ -35,7 +34,7 @@
  */
 
 GBLREF boolean_t		is_tracing_on;				/* M profiling */
-GBLREF boolean_t		tp_timeout_set_xfer;
+GBLREF boolean_t		async_dm_read, tp_timeout_set_xfer;
 GBLREF intrpt_state_t		intrpt_ok_state;
 GBLREF io_pair			io_std_device;
 GBLREF size_t			gtmMallocErrorSize;			/* Size of malloc limit violation */
@@ -50,6 +49,7 @@ GBLREF xfer_entry_t		xfer_table[];				/* transfer table */
 error_def(ERR_CTRAP);
 error_def(ERR_CTRLC);
 error_def(ERR_MALLOCCRIT);
+error_def(ERR_SOCKHANGUP);
 error_def(ERR_TERMHANGUP);
 error_def(ERR_TERMWRITE);
 error_def(ERR_JOBINTRRQST);
@@ -79,7 +79,7 @@ boolean_t xfer_set_handlers(int4  event_type, int4 param_val, boolean_t popped_e
 {	/* Keep track of what event types have come in and deal with them appropriately */
 	boolean_t	already_ev_handling;
 	int4		e_type, pv;
-	intrpt_state_t	prev_intrpt_state;
+	intrpt_state_t	prev_intrpt_state = INTRPT_OK_TO_INTERRUPT;
 	save_xfer_entry	*entry;
 	DCL_THREADGBL_ACCESS;
 
@@ -91,8 +91,8 @@ boolean_t xfer_set_handlers(int4  event_type, int4 param_val, boolean_t popped_e
 	{	/* events already in flux - stash this "as is" in the record for this event */
 		if (not_in_play == entry->event_state)
 			entry->event_state = signaled;
-		DBGDFRDEVNT((stderr, "%d %s: xfer_set_handlers - stashed: %d with state: %d\n", __LINE__, __FILE__, e_type,
-			TAREF1(save_xfer_root, e_type).event_state));
+		DBGDFRDEVNT((stderr, "%d %s: xfer_set_handlers - stashed: %d with state: %d\n", __LINE__, __FILE__, event_type,
+			TAREF1(save_xfer_root, event_type).event_state));
 		return TRUE;						/* return value currently only used by tp_timeout.c */
 	}
 	/* WARNING! AIO sets multi_thread_in_use which disables DEFER_INTERRUPTS, treat it like an active event */
@@ -125,9 +125,8 @@ boolean_t xfer_set_handlers(int4  event_type, int4 param_val, boolean_t popped_e
 			}
 		}
 	}
-	if (!already_ev_handling)
+	if ((!already_ev_handling) && (no_event == outofband))
 	{
-		assert(no_event == outofband || (event_type == outofband));
 		assert(!dollar_zininterrupt || (jobinterrupt != event_type));
 		if (entry != (TREF(save_xfer_root_ptr))->ev_que.fl)
 		{	/* no event in play so pend this one by jiggeriing the xfer_table */
@@ -139,8 +138,7 @@ boolean_t xfer_set_handlers(int4  event_type, int4 param_val, boolean_t popped_e
 		} else
 			DBGDFRDEVNT((stderr, "%d %s: xfer_set_handlers - skipping [re]queued event type %d\n", __LINE__, __FILE__,
 				event_type));
-		if (!already_ev_handling)
-			ENABLE_EVENT_INTERRUPTS(prev_intrpt_state);
+		ENABLE_EVENT_INTERRUPTS(prev_intrpt_state);
 		return TRUE;
 	}
 	if (not_in_play != entry->event_state)
@@ -151,50 +149,8 @@ boolean_t xfer_set_handlers(int4  event_type, int4 param_val, boolean_t popped_e
 			ENABLE_EVENT_INTERRUPTS(prev_intrpt_state);
 		return TRUE;
 	}
-	if (!already_ev_handling && (no_event == outofband) && (!have_crit(CRIT_HAVE_ANY_REG | CRIT_IN_COMMIT)))
-	{	/* no competion or blocking interrupt: collect $200 and go straight to pending */
-		/* -------------------------------------------------------
-		 * If table changed, it was not synchronized.
-		 * (Assumes these entries are all that would be changed)
-		 * --------------------------------------------------------
-		 */
-		assert((xfer_table[xf_linefetch] == op_linefetch) ||
-		       (xfer_table[xf_linefetch] == op_zstepfetch) ||
-		       (xfer_table[xf_linefetch] == op_zst_fet_over) ||
-		       (xfer_table[xf_linefetch] == op_mproflinefetch));
-		assert((xfer_table[xf_linestart] == op_linestart) ||
-		       (xfer_table[xf_linestart] == op_zstepstart) ||
-		       (xfer_table[xf_linestart] == op_zst_st_over) ||
-		       (xfer_table[xf_linestart] == op_mproflinestart));
-		assert((xfer_table[xf_zbfetch] == op_zbfetch) ||
-		       (xfer_table[xf_zbfetch] == op_zstzb_fet_over) ||
-		       (xfer_table[xf_zbfetch] == op_zstzbfetch));
-		assert((xfer_table[xf_zbstart] == op_zbstart) ||
-		       (xfer_table[xf_zbstart] == op_zstzb_st_over) ||
-		       (xfer_table[xf_zbstart] == op_zstzbstart));
-		assert((xfer_table[xf_forchk1] == op_forchk1) ||
-		       (xfer_table[xf_forchk1] == op_mprofforchk1));
-		assert((xfer_table[xf_forloop] == op_forloop));
-		assert(xfer_table[xf_ret] == opp_ret ||
-		       xfer_table[xf_ret] == opp_zst_over_ret ||
-		       xfer_table[xf_ret] == opp_zstepret);
-		assert(xfer_table[xf_retarg] == op_retarg ||
-		       xfer_table[xf_retarg] == opp_zst_over_retarg ||
-		       xfer_table[xf_retarg] == opp_zstepretarg);
-		/* -----------------------------------------------
-		 * Now call the specified set function to swap in
-		 * the desired handlers (and set flags or whatever).
-		 * -----------------------------------------------
-		 */
-		assert(entry != (TREF(save_xfer_root_ptr))->ev_que.fl);
-		assert(!dollar_zininterrupt || (jobinterrupt != event_type));
-		entry->event_state = pending;				/* jiggering the transfer table for this event */
-		outofband = event_type;
-		entry->set_fn(param_val);
-		DBGDFRDEVNT((stderr, "%d %s: xfer_set_handlers - set xfer_table for event type %d\n" ,__LINE__, __FILE__,
-			     event_type));
-	} else if (queued != entry->event_state)
-	{	/* queue it */
+	if (queued != entry->event_state)
+	{	/* If event is not in play, queue it. */
 		entry->event_state = queued;
 		SAVE_XFER_QUEUE_ENTRY(event_type, param_val);
 		if (outofband == event_type)
@@ -210,8 +166,8 @@ boolean_t xfer_set_handlers(int4  event_type, int4 param_val, boolean_t popped_e
 boolean_t xfer_reset_if_setter(int4 event_type)
 {	/* if the transfer table has been changed to activate event_type, return it to "normal," otherwise leve it alone */
 	boolean_t	already_ev_handling, res;
-	int4		dummy;
-	intrpt_state_t	prev_intrpt_state;
+	int4		dummy = 0;
+	intrpt_state_t	prev_intrpt_state = INTRPT_OK_TO_INTERRUPT;
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
@@ -241,7 +197,7 @@ boolean_t xfer_reset_if_setter(int4 event_type)
 	if (res)
 	{	/* it's worth a try */
 		DBGDFRDEVNT((stderr, "%d %s: xfer_reset_if_setter: event_type %d is first\n", __LINE__, __FILE__, event_type));
-		if (res = (active == TAREF1(save_xfer_root, event_type).event_state))	/* WARNING: assignment */
+		if ((res = (active == TAREF1(save_xfer_root, event_type).event_state)))	/* WARNING: assignment */
 			res = (real_xfer_reset(event_type));
 		DBGDFRDEVNT((stderr, "%d %s: xfer_reset_if_setter: xfer_reset_handlers returned %d\n", __LINE__, __FILE__, res));
 		if (res && (not_in_play != TAREF1(save_xfer_root, tptimeout).event_state))
@@ -278,7 +234,7 @@ boolean_t real_xfer_reset(int4 event_type)
 	boolean_t	already_ev_handling, cur_outofband;
 	int 		e_type;
 	int4		param_val, status;
-	intrpt_state_t	prev_intrpt_state;
+	intrpt_state_t	prev_intrpt_state = INTRPT_OK_TO_INTERRUPT;
 	save_xfer_entry	*entry;
 	DCL_THREADGBL_ACCESS;
 
@@ -341,12 +297,14 @@ boolean_t real_xfer_reset(int4 event_type)
  */
 void async_action(bool lnfetch_or_start)
 {
-	boolean_t	ours;
+	boolean_t	ours, save_async_dm_read;
 	intrpt_state_t	prev_intrpt_state;
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
 	assert(INTRPT_IN_EVENT_HANDLING != intrpt_ok_state);
+	save_async_dm_read = async_dm_read;
+	async_dm_read = FALSE;
 	DEFER_INTERRUPTS(INTRPT_IN_EVENT_HANDLING, prev_intrpt_state);
 	if (jobinterrupt == outofband)
 	{
@@ -382,6 +340,8 @@ void async_action(bool lnfetch_or_start)
 			break;
 		case ctrlc:							/* these go from pending to active here */
 			TAREF1(save_xfer_root, outofband).event_state = active;
+			if (save_async_dm_read && (MVST_MVAL == mv_chain->mv_st_type))
+				POP_MV_STENT();		/* In Direct Mode, so pop mval from top of stack to avoid memory leak */
 			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_CTRLC);
 			break;
 		case ctrap:
@@ -390,7 +350,10 @@ void async_action(bool lnfetch_or_start)
 			break;
 		case sighup:
 			TAREF1(save_xfer_root, sighup).event_state = pending;
-			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_TERMHANGUP);
+			if (tt == io_std_device.in->type)
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_TERMHANGUP);
+			else if (gtmsocket == io_std_device.in->type)
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_SOCKHANGUP);
 			break;
 		case (defer_error):
 			TAREF1(save_xfer_root, outofband).event_state = active;
