@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2021 Fidelity National Information	*
+ * Copyright (c) 2001-2023 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -29,6 +29,7 @@
 #include "copy.h"
 #include "util.h"
 #include "dse.h"
+#include "filestruct.h"
 
 /* Include prototypes*/
 #include "t_qread.h"
@@ -40,6 +41,7 @@ GBLREF gd_region	*gv_cur_region;
 GBLDEF global_root_list	*global_roots_head, *global_roots_tail;
 GBLDEF int4		patch_offset[MAX_BT_DEPTH + 1], patch_offset1[MAX_BT_DEPTH + 1];
 GBLREF sgmnt_addrs	*cs_addrs;
+GBLREF sgmnt_data_ptr_t	cs_data;
 GBLDEF short int	patch_dir_path_count, patch_path_count;
 
 static boolean_t	was_crit, was_hold_onto_crit, nocrit_present;
@@ -62,6 +64,7 @@ void dse_f_blk(void)
 	short int		count, rsize, size;
 	sm_uc_ptr_t		blk_id, bp, b_top, key_top, rp, r_top, sp, srp, s_top;
 
+	DSE_DB_IS_TOO_OLD(cs_addrs, cs_data, gv_cur_region);
 	if (BADDSEBLK == (patch_find_blk = dse_getblk("BLOCK", DSENOBML, DSEBLKCUR)))		/* WARNING: assignment */
 		return;
 	patch_find_sibs = (CLI_PRESENT == cli_present("SIBLINGS"));
@@ -137,7 +140,7 @@ void dse_f_blk(void)
 	patch_find_root_search = TRUE;
 	if ((exhaust = (CLI_PRESENT == cli_present("EXHAUSTIVE"))) || (0 >= size))		/* NOTE assignment */
 	{
-		if (patch_exh_found = (patch_find_blk == patch_path[0]))			/* NOTE assignment */
+		if ((patch_exh_found = (patch_find_blk == patch_path[0])))			/* NOTE assignment */
 		{
 			if (patch_find_sibs)
 			{
@@ -161,7 +164,7 @@ void dse_f_blk(void)
 				patch_path[0] = global_roots_head->link->root;
 				patch_path_count = 1;
 				patch_left_sib = patch_right_sib = 0;
-				if (patch_exh_found = (patch_find_blk == patch_path[0]))	/* NOTE assignment */
+				if ((patch_exh_found = (patch_find_blk == patch_path[0])))	/* NOTE assignment */
 				{
 					if (patch_find_sibs)
 					{
