@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2010 Fidelity Information Services, Inc	*
+ *	Copyright 2001, 2014 Fidelity Information Services, Inc	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -34,10 +34,10 @@ GBLREF	boolean_t		run_time;	/* needed by IS_MCODE_RUNNING macro */
 #define IS_MCODE_RUNNING	(run_time)
 
 #define IS_DSE_IMAGE			(DSE_IMAGE == image_type)
+#define IS_GTCM_SERVER_IMAGE		(GTCM_SERVER_IMAGE == image_type)
 #define IS_GTCM_GNP_SERVER_IMAGE	(GTCM_GNP_SERVER_IMAGE == image_type)
 #define IS_GTMSECSHR_IMAGE		(GTMSECSHR_IMAGE == image_type)
 #define IS_GTM_IMAGE			IS_MUMPS_IMAGE
-#define IS_GTM_SVC_DAL_IMAGE		(GTM_SVC_DAL_IMAGE == image_type)
 #define IS_LKE_IMAGE			(LKE_IMAGE == image_type)
 #define IS_MUMPS_IMAGE			(GTM_IMAGE == image_type)
 #define IS_MUPIP_IMAGE			(MUPIP_IMAGE == image_type)
