@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2009 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -20,5 +21,3 @@ IMAGE_TABLE_ENTRY (LKE_IMAGE,             "LKE")
 IMAGE_TABLE_ENTRY (GTMSECSHR_IMAGE,       "GTMSECSHR")
 IMAGE_TABLE_ENTRY (GTCM_SERVER_IMAGE,     "GTCM_SERVER")
 IMAGE_TABLE_ENTRY (GTCM_GNP_SERVER_IMAGE, "GTCM_GNP_SERVER")
-IMAGE_TABLE_ENTRY (DBCERTIFY_IMAGE,       "DBCERTIFY")
-IMAGE_TABLE_ENTRY (GTM_SVC_DAL_IMAGE,     "GTM_SVC_DAL")
