@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2021 Fidelity National Information	*
+ * Copyright (c) 2001-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -116,9 +116,10 @@ void gv_rundown(void)
 				if (csa->jnlpool && (jnlpool != csa->jnlpool))
 					jnlpool = csa->jnlpool;
 			        tp_change_reg();
-				rundown_status |= gds_rundown(CLEANUP_UDI_TRUE);
+				rundown_status |= gds_rundown(CLEANUP_UDI_TRUE, FALSE);
 			}
 			r_local->open = r_local->was_open = FALSE;
+			r_local->file_initialized = r_local->did_file_initialization = FALSE;
 		}
 	}
 	rc_close_section();
