@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2011 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2024 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -20,12 +21,14 @@
 
 GBLREF spdesc indr_stringpool;
 
-void ind_cg_var(mvar *v, var_tabent **p)
+void ind_cg_var(mtreenode *node, void *var_tabent_arg)
 {	/* Copy mident with variable name to variable table entry */
-	assert((char *)indr_stringpool.base <= v->mvname.addr && v->mvname.addr < (char *)indr_stringpool.top);
-	(*p)[v->mvidx].var_name = v->mvname;
-	COMPUTE_HASH_MNAME(&((*p)[v->mvidx]));
-	(*p)[v->mvidx].var_name.addr = (char *)((v->mvname.addr - (char *)indr_stringpool.base) +
+	var_tabent **p = var_tabent_arg;
+
+	assert((char *)indr_stringpool.base <= node->var.mvname.addr && node->var.mvname.addr < (char *)indr_stringpool.top);
+	(*p)[node->var.mvidx].var_name = node->var.mvname;
+	COMPUTE_HASH_MNAME(&((*p)[node->var.mvidx]));
+	(*p)[node->var.mvidx].var_name.addr = (char *)((node->var.mvname.addr - (char *)indr_stringpool.base) +
 						ROUND_UP2(SIZEOF(ihdtyp), NATIVE_WSIZE));
-	(*p)[v->mvidx].marked = FALSE;
+	(*p)[node->var.mvidx].marked = FALSE;
 }
