@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2021 Fidelity National Information	*
+ * Copyright (c) 2001-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -15,6 +15,9 @@
 
 #ifdef DEBUG
 #include <stddef.h>		/* for offsetof macro (see OFFSETOF usage in assert below) */
+#include "wcs_clean_dbsync.h"
+#include "timers.h"
+#include "have_crit.h"
 #include "wbox_test_init.h"
 #endif
 
@@ -96,7 +99,7 @@ error_def(ERR_JNLENDIANLITTLE);
 
 #define JNL_EXTEND_MIN		0
 #define JNL_EXTEND_DEF		2048
-#define JNL_EXTEND_MAX		1073741823
+#define JNL_EXTEND_MAX		8388607		/* should actually be JNL_ALLOC_MAX - jnl_alq but that's complex with no benefit */
 #define JNL_MIN_WRITE		32768
 #define JNL_MAX_WRITE		65536
 /* FE was changed to EB because, the bit pattern there seems to vary more than the one for "FE".
@@ -208,12 +211,13 @@ error_def(ERR_JNLENDIANLITTLE);
 #define	JNL_WRITE_APPROPRIATE(CSA, JPC, RECTYPE, JREC, JFB)								\
 MBSTART {														\
 	GBLREF	boolean_t		is_replicator;									\
+	GBLREF	jnl_gbls_t		jgbl;										\
 															\
 	assert(JNL_ENABLED(CSA) || REPL_WAS_ENABLED(CSA));								\
 	assert((NULL == JFB) || (RECTYPE == ((jnl_record *)(((jnl_format_buffer *)JFB)->buff))->prefix.jrec_type));	\
 	if (JNL_ENABLED(CSA))												\
 		jnl_write(JPC, RECTYPE, JREC, JFB); 	/* write to jnlbuffer */					\
-	if (REPL_ALLOWED(CSA) && is_replicator)										\
+	if (REPL_ALLOWED(CSA) && is_replicator && !jgbl.skip_jplwrites)							\
 	{														\
 		assert(REPL_ENABLED(CSA) || REPL_WAS_ENABLED(CSA));							\
 		jnl_pool_write(CSA, RECTYPE, JREC, JFB);	/* write to jnlpool */					\
@@ -526,6 +530,59 @@ typedef struct
 	unsigned char		buff[1];		/* Actually buff[size] */
 } jnl_buffer;
 
+#ifdef DEBUG
+#define CHAOS_IN_JNL_CLEANUP()													\
+MBSTART {															\
+	GBLREF uint4 		process_id;											\
+	GBLREF volatile int4	gtmMallocDepth;											\
+	GBLREF intrpt_state_t	intrpt_ok_state;										\
+	GBLREF boolean_t	gtm_white_box_test_case_enabled;								\
+	int			start_jnl_cleanup_wbox = gtm_white_box_test_case_count;						\
+																\
+	assert(!OK_TO_INTERRUPT);												\
+	if (WBTEST_ENABLED(WBTEST_JNL_CLEANUP))											\
+	{															\
+		if (!gtm_wbox_input_test_case_count && (JNL_CLEANUP_MUPIP_STOP <= gtm_white_box_test_case_count))		\
+		{														\
+			gtm_wbox_input_test_case_count = JNL_CLEANUP_MUPIP_STOP;						\
+		} else if (!gtm_wbox_input_test_case_count && (JNL_CLEANUP_KILL9 <= gtm_white_box_test_case_count))		\
+		{														\
+			gtm_wbox_input_test_case_count = JNL_CLEANUP_KILL9;							\
+		}														\
+		if ((JNL_CLEANUP_MUPIP_STOP <= gtm_white_box_test_case_count)							\
+				&& (gtm_wbox_input_test_case_count++ >= (gtm_white_box_test_case_count + 10)) 			\
+				&& !have_crit(CRIT_IN_COMMIT | CRIT_IN_WTSTART)) 						\
+		{														\
+			gtm_white_box_test_case_enabled = FALSE;								\
+			for (int mustp = 0; mustp < 5; mustp++)									\
+			{													\
+				kill(process_id, SIGTERM);									\
+				SHORT_SLEEP(100);										\
+			}													\
+			assert(false);												\
+		} else if ((JNL_CLEANUP_KILL9 <= gtm_white_box_test_case_count)							\
+				&& (gtm_wbox_input_test_case_count++ >= (gtm_white_box_test_case_count + 10)) 			\
+				&& !have_crit(CRIT_IN_COMMIT | CRIT_IN_WTSTART))						\
+		{														\
+			gtm_white_box_test_case_enabled = FALSE;								\
+			kill(process_id, SIGKILL);										\
+			LONG_SLEEP(10);												\
+			assert(false);												\
+		}														\
+	}															\
+} MBEND
+#else
+#define CHAOS_IN_JNL_CLEANUP()
+#endif
+
+#define FIXUP_JBP_FREE_IF_NEEDED(JBP)										\
+{														\
+	assert(JBP->phase2_commit_latch.u.parts.latch_pid == process_id);					\
+	if (JBP->free != (JBP->freeaddr % JBP->size))								\
+	{													\
+		JBP->free = JBP->freeaddr % JBP->size;								\
+	}													\
+}
 /* Sets jbp->freeaddr & jbp->free. They need to be kept in sync at all times */
 #define	SET_JBP_FREEADDR(JBP, FREEADDR)										\
 {														\
@@ -545,6 +602,10 @@ typedef struct
 	 * cache changes. It is up to readers to also specify a read memory barrier if necessary to receive	\
 	 * this broadcast.											\
 	 */													\
+	if (WBTEST_ENABLED(WBTEST_JNL_CLEANUP))									\
+	{													\
+		CHAOS_IN_JNL_CLEANUP();										\
+	}													\
 	SHM_WRITE_MEMORY_BARRIER;										\
 	JBP->free = (FREEADDR) % JBP->size;									\
 	DBG_CHECK_JNL_BUFF_FREEADDR(JBP);									\
@@ -580,8 +641,10 @@ typedef struct jnl_private_control_struct
 {
 	jnl_buffer_ptr_t	jnl_buff;		/* pointer to shared memory */
 	gd_region		*region;		/* backpointer to region head */
-	fd_type			channel,		/* output channel, aka fd in UNIX */
-				old_channel;		/* VMS only - for dealing with deferred deassign */
+	fd_type			channel;		/* output channel, aka fd in UNIX */
+	uint4			next_align_addr;	/* Private copy of the jnlbuff next_align_addr taken after acquiring crit.
+							 * Makes it possible to defer writing updates to shared memory.
+							 */
 	gd_id			fileid;			/* currently initialized and used only by source-server */
 	uint4			pini_addr,		/* virtual on-disk address for JRT_PINI record, if journaling */
 				new_freeaddr;
@@ -1176,7 +1239,7 @@ typedef struct
 	token_num			mur_jrec_strm_seqno;	/* This is the strm_seqno of the current record that backward
 								 * recovery/rollback is playing in its forward phase.
 								 */
-	unsigned short			filler_short;
+	unsigned short			skip_jplwrites;
 	unsigned short			mur_jrec_participants;
 	jnl_tm_t			gbl_jrec_time;
 	jnl_tm_t			mur_tp_resolve_time;	/* tp resolve time as determined by journal recovery.
@@ -1286,6 +1349,16 @@ MBSTART {													\
 #define DECR_INDEX2_AND_KILL(IDX2, IDX1)	/* No-OP */
 #endif	/* #ifdef DEBUG */
 
+#ifdef DEBUG
+#define SET_CUR_CMT_STEP_IF(CONDITION, TARGET, STEP)								\
+MBSTART {													\
+	if (CONDITION)												\
+		TARGET = STEP;											\
+} MBEND
+#else
+#define SET_CUR_CMT_STEP_IF(CONDITION, TARGET, STEP)
+#endif
+
 #define	UPDATE_JBP_RSRV_FREEADDR(CSA, JPC, JBP, JPL, RLEN, INDEX, IN_PHASE2, JNL_SEQNO, STRM_SEQNO, REPLICATION)	\
 MBSTART {														\
 	uint4			rsrv_freeaddr;										\
@@ -1395,17 +1468,20 @@ MBSTART {											\
 #define	FINISH_JNL_PHASE2_IN_JNLBUFF(CSA, JRS)										\
 MBSTART {														\
 	assert(NEED_TO_FINISH_JNL_PHASE2(JRS));										\
-	jnl_write_phase2(CSA, JRS);			/* Mark jnl record writing into jnlbuffer as complete */	\
+	jnl_write_phase2(CSA, JRS);		/* Mark jnl record writing into jnlbuffer as complete */		\
 	assert(!NEED_TO_FINISH_JNL_PHASE2(JRS));									\
 } MBEND
 
 #define	FINISH_JNL_PHASE2_IN_JNLPOOL_IF_NEEDED(REPLICATION, JNLPOOL)							\
 MBSTART {														\
+	GBLREF jnl_gbls_t jgbl;												\
+															\
+	assert(!REPLICATION || !jgbl.skip_jplwrites || !JNLPOOL->jrs.tot_jrec_len);					\
 	if (REPLICATION && JNLPOOL->jrs.tot_jrec_len)									\
 	{														\
-		JPL_PHASE2_WRITE_COMPLETE(JNLPOOL);	/* Mark jnl record writing into jnlpool   as complete */	\
-		assert(0 == JNLPOOL->jrs.tot_jrec_len);									\
+		JPL_PHASE2_WRITE_COMPLETE(JNLPOOL);	/* Mark jnl record writing into jnlpool as complete */		\
 	}														\
+	assert(!REPLICATION || !JNLPOOL->jrs.tot_jrec_len);								\
 } MBEND
 
 #define	NONTP_FINISH_JNL_PHASE2_IN_JNLBUFF_AND_JNLPOOL(CSA, JRS, REPLICATION, JNLPOOL)					\
@@ -1427,10 +1503,14 @@ MBSTART {														\
 		jrs = si->jbuf_rsrv_ptr;										\
 		assert(JNL_ALLOWED(csa));										\
 		assert(!REPLICATION || (!csa->jnlpool || (csa->jnlpool == JNLPOOL)));					\
+		SET_CUR_CMT_STEP_IF((TREF(cur_cmt_step) < DECL_CMT16), TREF(cur_cmt_step), DECL_CMT16); 		\
 		if (NEED_TO_FINISH_JNL_PHASE2(jrs))									\
 			FINISH_JNL_PHASE2_IN_JNLBUFF(csa, jrs);		/* Step CMT16 (if BG) */			\
+		SET_CUR_CMT_STEP_IF(JNL_FENCE_LIST_END == csa->next_fenced, TREF(cur_cmt_step), CMT16); 		\
 	}														\
-	FINISH_JNL_PHASE2_IN_JNLPOOL_IF_NEEDED(replication, JNLPOOL);	/* Step CMT17 (if BG) */			\
+	SET_CUR_CMT_STEP_IF(TRUE, TREF(cur_cmt_step), DECL_CMT17);							\
+	FINISH_JNL_PHASE2_IN_JNLPOOL_IF_NEEDED(REPLICATION, JNLPOOL);	/* Step CMT17 (if BG) */			\
+	SET_CUR_CMT_STEP_IF(TRUE, TREF(cur_cmt_step), CMT17);								\
 } MBEND
 
 /* BEGIN : Structures used by "jnl_write_reserve" */
@@ -1472,8 +1552,15 @@ MBSTART {									\
 	}									\
 } MBEND
 
+#define SET_JNLBUF_NEXT_ALIGN_ADDR(JPC, JBP)					\
+MBSTART {									\
+	assert(JPC->next_align_addr >= JBP->next_align_addr);			\
+	JBP->next_align_addr = JPC->next_align_addr;				\
+} MBEND
+
 #define	UPDATE_JRS_RSRV_FREEADDR(CSA, JPC, JBP, JRS, JPL, JNL_FENCE_CTL, REPLICATION)			\
 MBSTART {												\
+	SET_JNLBUF_NEXT_ALIGN_ADDR(JPC, JBP);								\
 	SET_JNLBUFF_PREV_JREC_TIME(JBP, jgbl.gbl_jrec_time, DO_GBL_JREC_TIME_CHECK_TRUE);		\
 		/* Keep jb->prev_jrec_time up to date */						\
 	assert(JNL_ENABLED(CSA));									\
@@ -1485,7 +1572,8 @@ MBSTART {												\
 
 #define	FREE_JBUF_RSRV_STRUCT(JRS)		\
 MBSTART {					\
-	assert(NULL != JRS);			\
+	assert((NULL != JRS) 			\
+		|| (WBTEST_ENABLED(WBTEST_SKIP_CORE_FOR_MEMORY_ERROR)));	\
 	if (NULL != JRS)			\
 	{					\
 		if (NULL != JRS->jrs_array)	\
