@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2022 Fidelity National Information	*
+ * Copyright (c) 2001-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -14,6 +14,82 @@
 #ifndef MDEF_included
 #define MDEF_included
 
+#ifdef __linux__
+#	define LINUX_ONLY(X) X
+#	define NON_LINUX_ONLY(X)
+#else
+#	define LINUX_ONLY(X)
+#	define NON_LINUX_ONLY(X) X
+#endif
+
+#ifdef __MVS__
+#	define ZOS_ONLY(X) X
+#else
+#	define ZOS_ONLY(X)
+#endif
+
+#ifdef Linux390
+#	define Linux390_ONLY(X) X
+#else
+#	define Linux390_ONLY(X)
+#endif
+
+#if !defined(__alpha) && !defined(__sparc) && !defined(__hpux) && !defined(mips) && !defined(__ia64)
+#	define UNALIGNED_ACCESS_SUPPORTED
+#endif
+
+#if defined(__i386) || defined(__x86_64__) || defined(_AIX) || defined (__sun)
+#	define GTM_PTHREAD
+#	define GTM_PTHREAD_ONLY(X) X
+#	define NON_GTM_PTHREAD_ONLY(X)
+#else
+#	define GTM_PTHREAD_ONLY(X)
+#	define NON_GTM_PTHREAD_ONLY(X)	X
+#endif
+
+#if defined(__ia64)
+#	define IA64_ONLY(X)	X
+#	define NON_IA64_ONLY(X)
+#  ifdef DEBUG
+#	define	IA64_DEBUG_ONLY(X)	X
+#  else
+#	define	IA64_DEBUG_ONLY(X)
+#  endif /* DEBUG */
+#else
+#	define	IA64_ONLY(X)
+#	define	NON_IA64_ONLY(X)	X
+#	define	IA64_DEBUG_ONLY(X)
+#endif/* __ia64 */
+
+#ifdef __x86_64__
+#define X86_64_ONLY(x)		x
+#define NON_X86_64_ONLY(x)
+#else
+#define X86_64_ONLY(x)
+#define NON_X86_64_ONLY(x)    x
+#endif /* __x86_64__ */
+
+#if defined(__i386) || defined(__x86_64__) || defined(__ia64) || defined(__MVS__) || defined(Linux390)
+#define NON_RISC_ONLY(x)	x
+#define RISC_ONLY(x)
+#elif defined(__sparc) || defined(_AIX) || defined(__alpha)
+#define RISC_ONLY(x)	x
+#define NON_RISC_ONLY(x)
+#endif
+
+#ifdef _AIX
+#       define  AIX_ONLY(X) X
+#else
+#       define  AIX_ONLY(X)
+#endif
+
+#ifdef __sparc
+#	define  SPARC_ONLY(X) X
+#else
+#define SPARC_ONLY(X)
+#endif
+#define BITS_PER_UCHAR	8 /* note, C does not require this to be 8, see <limits.h> for definitions of CHAR_BIT and UCHAR_MAX */
+
 #ifdef __clang__
 #define CLANG_SCA_ANALYZER_NORETURN __attribute__((analyzer_noreturn))
 #else
@@ -87,6 +163,20 @@ typedef unsigned int 	uint4;		/* 4-byte unsigned integer */
 #define INVALID_SEMID			-1
 #define INVALID_SHMID 			-1L
 
+/* For use in the SHMHUGETLB syslog warning message */
+enum shmget_caller
+{
+	LOCK_FILE,
+	SNAPSHOT_FILE,
+	RELINK,
+	JOURNAL_POOL,
+	DATABASE_FILE,
+	RC_CPT,
+	GTM_MULTI_PROC_FREEZE,
+	GTM_MULTI_PROC_RECOVER,
+	N_SHMGET_CALLERS
+};
+
 /* constant needed for FIFO - OS390 redefines in mdefsp.h */
 #define FIFO_PERMISSION		010666 /* fifo with RW permissions for owner, group, other */
 
@@ -97,6 +187,7 @@ typedef unsigned int 	uint4;		/* 4-byte unsigned integer */
 #include "gtm_common_defs.h"
 #include <mdefsp.h>
 #include "gtm_sizeof.h"
+#include "gtmxc_types.h"
 #include "gtm_threadgbl.h"
 
 /* Anchor for thread-global structure rather than individual global vars */
@@ -104,12 +195,10 @@ GBLREF void	*gtm_threadgbl;		/* Accessed through TREF macro in gtm_threadgbl.h *
 
 #if defined(DEBUG)
 # ifndef STATIC_ANALYSIS
-error_def(ERR_ASSERT);
-#  define assert(x) ((x) ? 1 : rts_error_csa(CSA_ARG(NULL) VARLSTCNT(7) ERR_ASSERT, 5, LEN_AND_LIT(__FILE__), __LINE__,		\
-						(SIZEOF(#x) - 1), (#x)))
+#  define assert(x) ((x) ? 1 : (assertfail(SIZEOF(#x) - 1, (#x), LEN_AND_LIT(__FILE__), __LINE__), 0))
 # else
 #  include <stdlib.h>
-static inline int gtm_abrt() { abort(); return 0;}
+   static inline int gtm_abrt() { abort(); return 0;}
 #  define assert(x) ((x) ? 1 : gtm_abrt())
 # endif
 # ifdef UNIX
@@ -159,16 +248,6 @@ static inline int gtm_abrt() { abort(); return 0;}
 #define	FD_INVALID		-1	/* fd of -1 is invalid in UNIX posix calls */
 #define	FD_INVALID_NONPOSIX	FD_INVALID
 
-#if defined(UNIX)
-#	define	USE_POLL
-#	define	POLL_ONLY(X)	X
-#	define	SELECT_ONLY(X)
-#else
-#	define	USE_SELECT
-#	define	POLL_ONLY(X)
-#	define	SELECT_ONLY(X)	X
-#endif
-
 /* INTPTR_T is an integer that has the same length as a pointer on each platform.  Its basic use is for arithmetic
  * or generic parameters.
  */
@@ -207,53 +286,6 @@ typedef UINTPTR_T uintszofptr_t;
 #	define CYGWIN_ONLY(X)
 #endif
 
-#ifdef __linux__
-#	define LINUX_ONLY(X) X
-#	define NON_LINUX_ONLY(X)
-#else
-#	define LINUX_ONLY(X)
-#	define NON_LINUX_ONLY(X) X
-#endif
-
-#ifdef __MVS__
-#	define ZOS_ONLY(X) X
-#else
-#	define ZOS_ONLY(X)
-#endif
-
-#ifdef Linux390
-#	define Linux390_ONLY(X) X
-#else
-#	define Linux390_ONLY(X)
-#endif
-
-#if !defined(__alpha) && !defined(__sparc) && !defined(__hpux) && !defined(mips) && !defined(__ia64)
-#	define UNALIGNED_ACCESS_SUPPORTED
-#endif
-
-#if defined(__i386) || defined(__x86_64__) || defined(_AIX) || defined (__sun)
-#	define GTM_PTHREAD
-#	define GTM_PTHREAD_ONLY(X) X
-#	define NON_GTM_PTHREAD_ONLY(X)
-#else
-#	define GTM_PTHREAD_ONLY(X)
-#	define NON_GTM_PTHREAD_ONLY(X)	X
-#endif
-
-#if defined(__ia64)
-#	define IA64_ONLY(X)	X
-#	define NON_IA64_ONLY(X)
-#  ifdef DEBUG
-#	define	IA64_DEBUG_ONLY(X)	X
-#  else
-#	define	IA64_DEBUG_ONLY(X)
-#  endif /* DEBUG */
-#else
-#	define	IA64_ONLY(X)
-#	define	NON_IA64_ONLY(X)	X
-#	define	IA64_DEBUG_ONLY(X)
-#endif/* __ia64 */
-
 /* macro to check that the OFFSET & SIZE of TYPE1.MEMBER1 is identical to that of TYPE2.MEMBER2 */
 #define	IS_OFFSET_AND_SIZE_MATCH(TYPE1, MEMBER1, TYPE2, MEMBER2)		\
 	(SIZEOF(((TYPE1 *)NULL)->MEMBER1) == SIZEOF(((TYPE2 *)NULL)->MEMBER2))	\
@@ -261,36 +293,6 @@ typedef UINTPTR_T uintszofptr_t;
 
 #define	IS_OFFSET_MATCH(TYPE1, MEMBER1, TYPE2, MEMBER2)	(OFFSETOF(TYPE1, MEMBER1) == OFFSETOF(TYPE2, MEMBER2))
 
-#ifdef __x86_64__
-#define X86_64_ONLY(x)		x
-#define NON_X86_64_ONLY(x)
-#else
-#define X86_64_ONLY(x)
-#define NON_X86_64_ONLY(x)    x
-#endif /* __x86_64__ */
-
-#if defined(__i386) || defined(__x86_64__) || defined(__ia64) || defined(__MVS__) || defined(Linux390)
-#define NON_RISC_ONLY(x)	x
-#define RISC_ONLY(x)
-#elif defined(__sparc) || defined(_AIX) || defined(__alpha)
-#define RISC_ONLY(x)	x
-#define NON_RISC_ONLY(x)
-#endif
-
-
-#ifdef _AIX
-#       define  AIX_ONLY(X) X
-#else
-#       define  AIX_ONLY(X)
-#endif
-
-#ifdef __sparc
-#	define  SPARC_ONLY(X) X
-#else
-#define SPARC_ONLY(X)
-#endif
-#define BITS_PER_UCHAR	8 /* note, C does not require this to be 8, see <limits.h> for definitions of CHAR_BIT and UCHAR_MAX */
-
 #define MAXPOSINT4		((int4)0x7fffffff)
 #define MINNEGINT4		((int4)0x80000000)
 #define MAX_DIGITS_IN_INT	10	/* maximum number of decimal digits in a  4-byte integer */
@@ -512,31 +514,6 @@ mval *underr_overwrite(mval *start, ...);
 /* Note MV_FORCE_CANONICAL currently only used in op_add() when vars are known to be defined so no MV_FORCE_DEFINED()
    macro has been added. If uses are added, this needs to be revisited. 01/2008 se
 */
-#define ZTIMEOUTSTR "ZTIMEOUT"
-#define MV_FORCE_MSTIMEOUT(TMV, TMS, NOTACID)	/* requires a flock of include files especially for TP */		\
-MBSTART {					/* also requires threaddef DCL and SETUP*/				\
-	GBLREF uint4		dollar_tlevel;										\
-	GBLREF uint4		dollar_trestart;									\
-															\
-															\
-	MV_FORCE_NUM(TMV);												\
-	if (NO_M_TIMEOUT == TMV->m[1])											\
-		TMS = NO_M_TIMEOUT;											\
-	else														\
-	{														\
-		assert(MV_BIAS >= 1000);	/* if formats change scale may need attention */			\
-		/* negative becomes 0 larger than MAXPOSINT4 caps to MAXPOSINT4 */					\
-		if (!(TMV->mvtype & MV_INT))										\
-			TMV->e += 3;	/* for non-ints, bump exponent to get millisecs from MV_FORCE_INT */		\
-		TMS = ((TMV->mvtype & MV_INT) ? TMV->m[1] : MIN(MAXPOSINT4, MV_FORCE_INT(TMV)));			\
-		if (!(TMV->mvtype & MV_INT))										\
-			TMV->e -= 3;	/* if we messed with the exponent, restore it to its original value */		\
-		if (0 > TMS)												\
-			TMS = 0;											\
-	}														\
-	if ((STRNCMP_LIT(NOTACID, "ZTIMEOUTSTR")) && ((TREF(tpnotacidtime)).m[1] < TMS))				\
-		TPNOTACID_CHECK(NOTACID);										\
-} MBEND
 #define MV_FORCE_CANONICAL(X)	((((X)->mvtype & MV_NM) == 0 ? s2n(X) : 0 ) \
 				 ,((X)->mvtype & MV_NUM_APPROX ? (X)->mvtype &= MV_NUM_MASK : 0 ))
 #define MV_IS_NUMERIC(X)	(((X)->mvtype & MV_NM) != 0)
@@ -600,9 +577,7 @@ MBSTART {					/* also requires threaddef DCL and SETUP*/				\
 #define	IS_ASCII(X)		((uint4)(X) <= ASCII_MAX)	/* X can be greater than 255 hence the typecast to uint4 */
 
 #ifdef UTF8_SUPPORTED
-#	define	MV_FORCE_LEN(X)	    ((!((X)->mvtype & MV_UTF_LEN)) 							\
-				     ? (utf8_len(&(X)->str), ((X)->mvtype |= MV_UTF_LEN), (X)->str.char_len)		\
-				     : (X)->str.char_len)
+	/* MV_FORCE_LEN() inlined and moved to gtm_utf8.h */
 #	define	MV_FORCE_LEN_DEC(X) ((!((X)->mvtype & MV_UTF_LEN)) 							\
 				     ? (utf8_len_dec(&(X)->str), ((X)->mvtype |= MV_UTF_LEN), (X)->str.char_len)	\
 				     : (X)->str.char_len)
@@ -659,9 +634,12 @@ MBSTART {					/* also requires threaddef DCL and SETUP*/				\
 
 #define CALLFROM	LEN_AND_LIT(__FILE__), __LINE__
 void gtm_assert(int file_name_len, char file_name[], int line_no)				CLANG_SCA_ANALYZER_NORETURN;
-int gtm_assert2(int condlen, char *condtext, int file_name_len, char file_name[], int line_no)	CLANG_SCA_ANALYZER_NORETURN;
+void gtm_assert2(int condlen, char *condtext, int file_name_len, char file_name[], int line_no)	__attribute__((noreturn));
 #define GTMASSERT	(gtm_assert(CALLFROM))
-#define assertpro(x) ((x) ? 1 : gtm_assert2((SIZEOF(#x) - 1), (#x), CALLFROM))
+#define assertpro(x) ((x) ? 1 : (gtm_assert2((SIZEOF(#x) - 1), (#x), CALLFROM), 0))
+#ifndef static_assert
+#define static_assert _Static_assert
+#endif
 #ifdef UNIX
 #ifdef DEBUG
 /* The below debug only macros are always used in pairs to indicate a window where the code doesn't expect rts_errors to happen.
@@ -699,12 +677,14 @@ int4 timeout2msec(int4 timeout);
 MBSTART {					\
 	rts_error_csa(CSA, __VA_ARGS__);	\
 	assert(FALSE);				\
+	GTM_UNREACHABLE();			\
 } MBEND
 
 #define RTS_ERROR_ABT(...)			\
 MBSTART {					\
 	rts_error(__VA_ARGS__);			\
 	assert(FALSE);				\
+	GTM_UNREACHABLE();			\
 } MBEND
 
 /* the RTS_ERROR_TEXT macro will stay till all existing references to it have been renamed to RTS_ERROR_{LITERAL,STRING} */
@@ -715,6 +695,22 @@ MBSTART {					\
 #define RTS_ERROR_MSTR(MSTR)		(MSTR)->len, (MSTR)->addr
 #define RTS_ERROR_MVAL(MVAL)		RTS_ERROR_MSTR(&(MVAL)->str)
 
+#ifdef __GNUC__
+#define GTM_UNREACHABLE()	__builtin_unreachable()
+#else
+#define GTM_UNREACHABLE()
+#endif
+
+inline void assertfail(size_t testlen, char *teststr, int flen, char *fstr, int line) __attribute__ ((noreturn));
+
+error_def(ERR_ASSERT);
+
+inline void assertfail(size_t testlen, char *teststr, int flen, char *fstr, int line)
+{
+	rts_error_csa(NULL, VARLSTCNT(7) ERR_ASSERT, 5, flen, fstr, line, testlen, teststr);
+	GTM_UNREACHABLE();
+}
+
 #define	SET_PROCESS_EXITING_TRUE						\
 {										\
 	GBLREF	int		process_exiting;				\
@@ -1387,6 +1383,7 @@ void mval_lex(mval *v, mstr *output);
 #define ZTRAP_POP	0x00000004
 #define ZTRAP_ADAPTIVE	(ZTRAP_CODE | ZTRAP_ENTRYREF)
 
+#define BYTEMASK	0xFF
 #define GTM_BYTESWAP_16(S)		\
 	(  (((S) & 0xff00) >> 8)	\
 	 | (((S) & 0x00ff) << 8)	\
@@ -1614,7 +1611,7 @@ typedef struct repl_histinfo_struct
 	int4		prev_histinfo_num;			/* = 'n' if the previous history record corresponding to this
 								 * stream is the n'th history record in the instance file.
 								 */
-	char		strm_index;				/* = 0 by default.
+	unsigned char	strm_index;				/* = 0 by default.
 								 * = anywhere from 1 to 15 if this history record corresponds to a
 								 *	non-supplementary stream of updates.
 								 */
@@ -1896,4 +1893,9 @@ enum
 # define DBG_VERIFY_ACCESS(PTR)
 #endif
 
+/* Defines used with in_rlbk */
+#define RLBKNOTACTIVE	0	/* Not actively performing a rollback */
+#define RLBKACTIVE	1	/* Performing a rollback */
+#define RLBKEXITING	2	/* Performing a rollback and about to finish */
+
 #endif /* MDEF_included */
