@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2021 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -13,6 +13,7 @@
 #ifndef MLK_SHRBLK_CREATE_INCLUDED
 #define MLK_SHRBLK_CREATE_INCLUDED
 
-mlk_shrblk_ptr_t mlk_shrblk_create(mlk_pvtblk *p, unsigned char *val, int len, mlk_shrblk_ptr_t par, ptroff_t *ptr, int nshrs);
+mlk_shrblk_ptr_t mlk_shrblk_create(mlk_pvtblk *p, unsigned char *val, unsigned char len,
+					mlk_shrblk_ptr_t par, ptroff_t *ptr, int nshrs);
 
 #endif /* MLK_SHRBLK_CREATE_INCLUDED */
