@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2003-2017 Fidelity National Information	*
+ * Copyright (c) 2003-2023 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -69,7 +69,9 @@ uint4 mur_process_intrpt_recov()
 	jnl_record			*jnlrec;
 	jnl_file_header			*jfh;
 	jnl_tm_t			now;
+	DCL_THREADGBL_ACCESS;
 
+	SETUP_THREADGBL_ACCESS;
 	for (rctl = mur_ctl, rctl_top = mur_ctl + murgbl.reg_total; rctl < rctl_top; rctl++)
 	{
 		TP_CHANGE_REG(rctl->gd);
@@ -285,7 +287,7 @@ uint4 mur_process_intrpt_recov()
 		jfh->turn_around_offset = jctl->turn_around_offset;	/* save progress in file header for 	*/
 		jfh->turn_around_time = jctl->turn_around_time;		/* possible re-issue of recover 	*/
 		for (idx = 0; idx < MAX_SUPPL_STRMS; idx++)
-			jfh->strm_end_seqno[idx] = csd->strm_reg_seqno[idx];
+			jfh->strm_end_seqno[idx] = rctl->csd->strm_reg_seqno[idx];
 		jfh_changed = TRUE;
 		/* We are about to update the journal file header of the turnaround-point journal file to store the
 		 * non-zero jfh->turn_around_offset. Ensure corresponding database is considered updated.
