@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2003-2020 Fidelity National Information	*
+ * Copyright (c) 2003-2023 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -66,7 +66,7 @@ boolean_t	mur_select_rec(jnl_ctl_list *jctl)
 {
 	boolean_t		exc_item_seen, inc_item_seen, inc_seen, wildcard_match, exact_match;
 	char			key_buff[MAX_KEY_SZ + 1 + SIZEOF(uint4) * 2], asc_key_buff[MAX_ZWR_KEY_SZ], *ptr, *val_ptr;
-	int			i, key_len, pat_pos, subs_pos;
+	int			i, key_len = 0;
 	uint4			pini_addr;
 	gv_key			*key;
 	jnl_record		*rec;
@@ -136,7 +136,7 @@ boolean_t	mur_select_rec(jnl_ctl_list *jctl)
 				wildcard_match = mur_do_wildcard(pv->jpv_user, sl_ptr->buff, pv_len, sl_ptr->len);
 			if (!wildcard_match)
 				sl_len = MIN(sl_ptr->len, JPV_LEN_USER);
-			if (wildcard_match || (pv_len == sl_len) && (0 == memcmp(pv->jpv_user, sl_ptr->buff, sl_len)))
+			if (wildcard_match || ((pv_len == sl_len) && (0 == memcmp(pv->jpv_user, sl_ptr->buff, sl_len))))
 				SET_INCFLAG_OR_RETURN(sl_ptr);
 		}
 		RETURN_IF_ITEM_DOESNT_MATCH_ANY_INCLUDE_PATTERN;
@@ -149,14 +149,17 @@ boolean_t	mur_select_rec(jnl_ctl_list *jctl)
 			wildcard_match = FALSE;
 			SET_FLAG_IF_INCLUDE_PATTERN_SEEN(sl_ptr);
 			if (sl_ptr->has_wildcard)
+			{
+				assert(key_len);
 				wildcard_match = mur_do_wildcard(asc_key_buff, sl_ptr->buff, key_len, sl_ptr->len);
+			}
 			i = sl_ptr->len;
 			if (sl_ptr->buff[i - 1] == ')')
 				--i;
 			if (wildcard_match
-				|| (key_len == i) && (0 == memcmp(asc_key_buff, sl_ptr->buff, i))
-				|| (key_len >  i) && (0 == memcmp(asc_key_buff, sl_ptr->buff, i))
-					&& (('(' == asc_key_buff[i]) || (')' == asc_key_buff[i]) || (',' == asc_key_buff[i])))
+				|| ((key_len == i) && (0 == memcmp(asc_key_buff, sl_ptr->buff, i)))
+				|| ((key_len >  i) && (0 == memcmp(asc_key_buff, sl_ptr->buff, i))
+					&& (('(' == asc_key_buff[i]) || (')' == asc_key_buff[i]) || (',' == asc_key_buff[i]))))
 			{
 					SET_INCFLAG_OR_RETURN(sl_ptr);
 			}
@@ -195,7 +198,7 @@ boolean_t	mur_select_rec(jnl_ctl_list *jctl)
 				pv_len = real_len(JPV_LEN_PRCNAM, (uchar_ptr_t)pv->jpv_prcnam);
 				sl_len = MIN(sl_ptr->len, JPV_LEN_PRCNAM);
 			}
-			if (wildcard_match || (pv_len == sl_len) && (0 == memcmp(pv->jpv_prcnam, sl_ptr->buff, sl_len)))
+			if (wildcard_match || ((pv_len == sl_len) && (0 == memcmp(pv->jpv_prcnam, sl_ptr->buff, sl_len))))
 				SET_INCFLAG_OR_RETURN(sl_ptr);
 		}
 		RETURN_IF_ITEM_DOESNT_MATCH_ANY_INCLUDE_PATTERN;
