@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2021 Fidelity National Information	*
+ * Copyright (c) 2001-2023 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -30,7 +30,7 @@
 # define DBGTRIGR_ONLY(x)
 #endif
 
-#define	INFO_MSK(error)	(error & ~SEV_MSK | INFO)
+#define	INFO_MSK(error)	((error & ~SEV_MSK) | INFO)
 
 GBLREF mident_fixed	zlink_mname;
 GBLREF volatile int4	outofband;
