@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001 Sanchez Computer Associates, Inc.	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -13,25 +14,24 @@
 #include "compiler.h"
 #include "opcode.h"
 
-GBLDEF triple *curr_fetch_trip, *curr_fetch_opr;
-GBLDEF int4 curr_fetch_count;
-
 oprtype put_mvar(mident *x)
 {
 	triple *ref,*fetch;
 	mvar *var;
+	DCL_THREADGBL_ACCESS;
 
+	SETUP_THREADGBL_ACCESS;
 	ref = newtriple(OC_VAR);
 	ref->operand[0].oprclass = MVAR_REF;
 	ref->operand[0].oprval.vref = var = get_mvaddr(x);
-	if (var->last_fetch != curr_fetch_trip)
-	{
+	if ((var->last_fetch != (TREF(fetch_control)).curr_fetch_trip) && !TREF(discard))
+	{	/* This block is identical to one in m_new */
 		fetch = newtriple(OC_PARAMETER);
-		curr_fetch_opr->operand[1] = put_tref(fetch);
+		(TREF(fetch_control)).curr_fetch_opr->operand[1] = put_tref(fetch);
 		fetch->operand[0] = put_ilit(var->mvidx);
-		curr_fetch_count++;
-		curr_fetch_opr = fetch;
-		var->last_fetch = curr_fetch_trip;
+		((TREF(fetch_control)).curr_fetch_count)++;
+		(TREF(fetch_control)).curr_fetch_opr = fetch;
+		var->last_fetch = (TREF(fetch_control)).curr_fetch_trip;
 	}
 	ref->destination.oprclass = TVAR_REF;
 	ref->destination.oprval.temp = var->mvidx;
