@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2022 Fidelity National Information	*
+ * Copyright (c) 2001-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -241,7 +241,7 @@ MBSTART {														\
 #endif
 
 #define PROCESS_CACHE_ENTRY(cp)							\
-        if (cp->src.str.len)	/* entry is used */				\
+	if (cp->src.str.len)	/* entry is used */				\
 		MSTR_STPG_ADD(&cp->src.str);					\
 	/* Run list of mvals for each code stream that exists */		\
 	if (cp->obj.len)							\
@@ -459,12 +459,12 @@ void stp_gcol(size_t space_asked)	/* BYPASSOK */
 	unsigned char		*strpool_base, *straddr, *tmpaddr, *begaddr, *endaddr;
 	int			index, fixup_cnt;
 	ssize_t			space_before_compact, space_after_compact, blklen, delta, space_reclaim, padlen;
-	ssize_t			totspace, space_needed, tmplen, tmplen2;
+	ssize_t			prev_stp_size = 0, reserved_bytes, space_needed, tmplen, tmplen2, totspace;
 	io_log_name		*l;		/* logical name pointer		*/
 	lv_blk			*lv_blk_ptr;
 	lv_val			*lvp, *lvlimit;
 	lvTreeNode		*node, *node_limit;
-	mstr			**cstr, *x;
+	mstr			**cstr = NULL, *x;
 	mv_stent		*mvs;
 	mval			*m, **mm, **mmtop, *mtop;
 	intszofptr_t		lv_subs;
@@ -472,9 +472,7 @@ void stp_gcol(size_t space_asked)	/* BYPASSOK */
 	tp_frame		*tf;
 	zwr_sub_lst		*zwr_sub;
 	ihdtyp			*ihdr;
-	int			*low_reclaim_passes;
-	int			*incr_factor;
-	int			killcnt;
+	int			*incr_factor, killcnt, *low_reclaim_passes, save_errno;
 	ssize_t			stp_incr;
 	ht_ent_objcode 		*tabent_objcode, *topent;
 	ht_ent_mname		*tabent_mname, *topent_mname;
@@ -543,13 +541,14 @@ void stp_gcol(size_t space_asked)	/* BYPASSOK */
 	assert(stringpool.free >= stringpool.base);
 	assert(stringpool.free <= stringpool.top);
 	assert(stringpool.invokestpgcollevel <= stringpool.top);
-        assert(CHK_BOUNDARY_ALIGNMENT(stringpool.top) == 0);
+	assert(CHK_BOUNDARY_ALIGNMENT(stringpool.top) == 0);
 	/* stp_vfy_mval(); / * uncomment to debug lv corruption issues.. */
 #	ifdef STP_MOVE
 	assert(stp_move_from < stp_move_to); /* why did we call with zero length range, or a bad range? */
-	/* Assert that range to be moved does not intersect with stringpool range. */
-	assert(((stp_move_from < (char *)stringpool.base) && (stp_move_to < (char *)stringpool.base))
-		|| ((stp_move_from >= (char *)stringpool.top) && (stp_move_to >= (char *)stringpool.top)));
+	/* Assert that range to be moved does not intersect with stringpool range.
+	 * Since stp_move_from<->stp_move_to is a range, it can hit the edges of the string pool.
+	 */
+	assert((stp_move_to <= (char *)stringpool.base) || (stp_move_from >= (char *)stringpool.top));
 #	endif
 	space_needed = ROUND_UP2(space_asked, NATIVE_WSIZE);
 	assert(0 == (INTPTR_T)stringpool.base % NATIVE_WSIZE);
@@ -562,9 +561,14 @@ void stp_gcol(size_t space_asked)	/* BYPASSOK */
 		low_reclaim_passes = &indr_stp_low_reclaim_passes;
 		incr_factor = &indr_stp_incr_factor;
 	} else
+	{
 		assertpro(FALSE && stringpool.base);	/* neither rts_stringpool, nor indr_stringpool */
+	}
 	if (NULL == stp_array)
-		stp_array = (mstr **)malloc((stp_array_size = STP_MAXITEMS) * SIZEOF(mstr *));
+	{
+		stp_array_size = STP_MAXITEMS;
+		stp_array = (mstr **)stp_mmap(stp_array_size * SIZEOF(mstr *));
+	}
 	topstr = array = stp_array;
 	arraytop = topstr + stp_array_size;
 	/* If dqloop == 0 then we got here from mcompile. If literal_chain.que.fl == 0 then put_lit was never
@@ -659,7 +663,7 @@ void stp_gcol(size_t space_asked)	/* BYPASSOK */
 			for (xnewvar = symtab->xnew_var_list; xnewvar; xnewvar = xnewvar->next)
 				MSTR_STPG_ADD(&xnewvar->key.var_name);
 		}
-		if (x = comline_base)
+		if ((x = comline_base))
 			for (index = MAX_RECALL; index > 0 && x->len; index--, x++)
 			{	/* These strings are guaranteed to be in the stringpool so use MSTR_STPG_PUT macro directly
 				 * instead of going through MSTR_STPG_ADD. But assert accordingly to be sure.
@@ -994,6 +998,7 @@ void stp_gcol(size_t space_asked)	/* BYPASSOK */
 						(size_t)(stringpool.top - stringpool.base)));
 			assert((ssize_t)(stringpool.top - stringpool.base)
 					< ((ssize_t)(stp_incr + stringpool.top - stringpool.base)));
+			prev_stp_size = stringpool.lastallocbytes;
 			expand_stp((ssize_t)(stp_incr + stringpool.top - stringpool.base));
 #			ifdef DEBUG
 			/* If expansion failed and stp_gcol_ch did an UNWIND and we were already in exit handling code,
@@ -1031,12 +1036,14 @@ void stp_gcol(size_t space_asked)	/* BYPASSOK */
 			 * stringpool is the run-time or indirection stringpool.
 		 	 */
 			(strpool_base == rts_stringpool.base) ? (rts_stringpool = stringpool) : (indr_stringpool = stringpool);
-			free(strpool_base);
+			assert(0 < prev_stp_size);
+			stp_fini(strpool_base, prev_stp_size);
 			/* Adjust incr_factor */
 			if (*incr_factor < STP_NUM_INCRS) *incr_factor = *incr_factor + 1;
 		} else
 		{	/* Could not expand during forced expansion */
 			assert(non_mandatory_expansion && stop_non_mandatory_expansion);
+			assert(cstr);
 			if (space_after_compact < space_needed)
 			{	/* Restore stringpool.free since no garbage collection happened. */
 				stringpool.free = old_free;
@@ -1095,9 +1102,24 @@ void stp_gcol(size_t space_asked)	/* BYPASSOK */
 	assert(stringpool.free <= stringpool.top);
 	stringpool.invokestpgcollevel = (STP_SPACE_USED_MULTIPLIER * (space_asked + stringpool.free - stringpool.base))
 		+ stringpool.base;
-	stringpool.invokestpgcollevel = (((stringpool.invokestpgcollevel - stringpool.base) < STP_GCOL_TRIGGER_FLOOR)
+	reserved_bytes = (ssize_t)(stringpool.top - stringpool.invokestpgcollevel);
+	if (((stringpool.invokestpgcollevel - stringpool.base) < STP_GCOL_TRIGGER_FLOOR)
 		|| (stringpool.invokestpgcollevel > stringpool.top))
-		? stringpool.top : stringpool.invokestpgcollevel;
+	{
+		stringpool.invokestpgcollevel = stringpool.top;
+	} else if (( STP_MIN_CONTRACTION < reserved_bytes) && (array < topstr))
+	{	/* Contract the stringpool */
+		assert(stringpool.top > stringpool.invokestpgcollevel);
+		strpool_base = stringpool.base;
+		prev_stp_size = stringpool.lastallocbytes;
+		stp_init((ssize_t)(stringpool.invokestpgcollevel - stringpool.base));
+		assert(strpool_base != stringpool.base); /* Initiate a smaller space successfully */
+		cstr = array;
+		COPY2STPOOL(cstr, topstr);
+		(strpool_base == rts_stringpool.base) ? (rts_stringpool = stringpool) : (indr_stringpool = stringpool);
+		assert(0 < prev_stp_size);
+		stp_fini(strpool_base, prev_stp_size);
+	}
 	assert(stringpool.invokestpgcollevel <= stringpool.top);
 #	ifndef STP_MOVE
 	assert(stringpool.top - stringpool.free >= space_asked);
