@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2021 Fidelity National Information	*
+ * Copyright (c) 2001-2023 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -32,12 +32,12 @@ GBLREF	gd_region		*gv_cur_region;
 GBLREF	jnl_format_buffer	*non_tp_jfb_ptr;
 GBLREF	sgmnt_addrs		*cs_addrs;
 GBLREF	trans_num		start_tn;
-GBLREF	uint4			t_err;
+GBLREF	uint4			mu_upgrade_in_prog, t_err;
 GBLREF	unsigned char		cw_set_depth;
 GBLREF	unsigned char		t_fail_hist;
 GBLREF	unsigned int		t_tries;
 GBLREF	uint4			update_trans;
-GBLREF	boolean_t		mu_reorg_upgrd_dwngrd_in_prog, write_after_image;
+GBLREF	boolean_t		write_after_image;
 GBLREF	volatile int4		fast_lock_count;
 
 void	t_begin_crit(uint4 err)	/* err - error code for current gvcst_routine */
@@ -64,7 +64,7 @@ void	t_begin_crit(uint4 err)	/* err - error code for current gvcst_routine */
 	/* the only currently known callers of this routine are DSE and MUPIP RECOVER (mur_put_aimg_rec.c).
 	 * they set "write_after_image" to TRUE. but mu_upgrade_bmm does not, hence the assert below.
 	 */
-	assert(write_after_image || mu_reorg_upgrd_dwngrd_in_prog);
+	assert(write_after_image || mu_upgrade_in_prog);
 	update_trans = UPDTRNS_DB_UPDATED_MASK;
 	was_crit = cs_addrs->now_crit;
 	assert(!was_crit || cs_addrs->hold_onto_crit);
