@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2021 Fidelity National Information	*
+ * Copyright (c) 2001-2023 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -85,7 +85,7 @@ enum cdb_sc tp_hist(srch_hist *hist1)
 {
 	int			hist_index;
 	srch_hist		*hist;
-	off_chain		chain;
+	block_ref		chain;
 	srch_blk_status		*t1, *t2;
 	block_id		blk;
 	srch_blk_status		*local_hash_entry;
@@ -127,8 +127,9 @@ enum cdb_sc tp_hist(srch_hist *hist1)
 	 * we should have the same blk_target for both the histories and hence the same value for
 	 * !blk_target->noisolation and hence the same value for store_history. We assert this is the case below.
 	 */
-	assert(ERR_GVPUTFAIL == t_err && (NULL == hist1 || hist1 == &cs_addrs->dir_tree->hist && !cs_addrs->dir_tree->noisolation)
-			|| !hist1 || hist1->h[0].blk_target == gvt);
+	assert(((ERR_GVPUTFAIL == t_err) && ((NULL == hist1) || ((hist1 == &cs_addrs->dir_tree->hist)
+									&& !cs_addrs->dir_tree->noisolation)))
+			|| !hist1 || (hist1->h[0].blk_target == gvt));
 	/* We are going to reference fields from shared memory including cr->in_tend and t1->buffaddr->tn in that order (as part
 	 * of the TP_IS_CDB_SC_BLKMOD macro below). To ensure we read an uptodate value of cr->in_tend, we do a read memory
 	 * barrier here. Note that to avoid out-of-order reads, the read memory barrier needs to be done AFTER the reference
@@ -166,8 +167,8 @@ enum cdb_sc tp_hist(srch_hist *hist1)
 			 * after the block contents get modified, then we may incorrectly decide (looking at the
 			 * tn) that the block hasn't changed when actually its contents have.
 			 */
-			chain = *(off_chain *)&blk;
-			if (!chain.flag)
+			chain.id = blk;
+			if (!chain.chain.flag)
 			{	/* We need to ensure the shared copy hasn't changed since the beginning of the
 				 * transaction since not checking that can cause at least false UNDEFs.
 				 * e.g. Say earlier in this TP transaction we had gone down
@@ -324,8 +325,8 @@ enum cdb_sc tp_hist(srch_hist *hist1)
 			 * For an existing block, we copy its search history into si->last_tp_hist.
 			 * For a created block, we dont since it doesn't need to be validated at commit.
 			 */
-			assert(!chain.flag || !t1->first_tp_srch_status);
-			if (store_history && !chain.flag)
+			assert(!chain.chain.flag || !t1->first_tp_srch_status);
+			if (store_history && !chain.chain.flag)
 			{
 				assert(si->cw_set_depth || !t1->cse);
 				local_hash_entry = t1->first_tp_srch_status;
@@ -402,8 +403,8 @@ enum cdb_sc tp_hist(srch_hist *hist1)
 					/* Ensure that "first_tp_srch_status" reflects what is in the hash-table.
 					 * The only exception is when two histories are passed (explained above)
 					 */
-					assert(local_hash_entry && lookup_tabent && (local_hash_entry == lookup_tabent->value)
-						|| !local_hash_entry && (hist == hist1));
+					assert((local_hash_entry && lookup_tabent && (local_hash_entry == lookup_tabent->value))
+						|| (!local_hash_entry && (hist == hist1)));
 					/* Ensure we don't miss updating "cse" */
 					assert((NULL != local_hash_entry) || ((srch_blk_status *)(tabent->value))->cse|| !t1->cse);
 					assert(!local_hash_entry || !local_hash_entry->cse || !t1->cse
