@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2022 Fidelity National Information	*
+ * Copyright (c) 2001-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -87,7 +87,7 @@ void tptimeout_set(int4 dummy_param)
 	SETUP_THREADGBL_ACCESS;
 	assert(INTRPT_IN_EVENT_HANDLING == intrpt_ok_state);
 	assert(tptimeout == outofband);
-	if (dollar_zininterrupt || ((0 < dollar_ecode.index) && ETRAP_IN_EFFECT) || have_crit(CRIT_HAVE_ANY_REG | CRIT_IN_COMMIT))
+	if (dollar_zininterrupt || ((0 < dollar_ecode.index) && ETRAP_IN_EFFECT))
 	{	/* Error handling or job interrupt is in effect - defer tp timeout until $ECODE is cleared and/or we have unrolled
 		* the job interrupt frame.
 		*/
@@ -119,13 +119,13 @@ void tptimeout_set(int4 dummy_param)
  * how it's called from timer_handler(), but other invokers use 0 and null.
  * ------------------------------------------------------------------
  */
-void tp_start_timer(int4 timeout_seconds)
+void tp_start_timer(int4 timeout_milliseconds)
 {
 	assert(!in_timed_tn);
 	SHOWTIME(asccurtime);
 	DBGDFRDEVNT((stderr, "%d %s %s: tp_start_timer - tptimeout: %d\n", __LINE__, __FILE__, asccurtime, timeout_seconds));
 	in_timed_tn = TRUE;
-	start_timer(TP_TIMER_ID, (1000 * timeout_seconds), &tp_expire_now, 0, NULL);
+	start_timer(TP_TIMER_ID, timeout_milliseconds, &tp_expire_now, 0, NULL);
 }
 
 /* ------------------------------------------------------------------
