@@ -1,7 +1,6 @@
-
 /****************************************************************
  *								*
- * Copyright (c) 2005-2022 Fidelity National Information	*
+ * Copyright (c) 2005-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -126,7 +125,7 @@ typedef enum {
 	WBTEST_ANTIFREEZE_OUTOFSPACE,		/* 81 :  */
 	/* End ANTIFREEZE related white box test cases */
 	WBTEST_SIGTSTP_IN_JNL_OUTPUT_SP,	/* 82 : Send SIGTSTP to self if wcs_timers is 0 */
-	WBTEST_CONCBKUP_RUNNING,		/* 83 : Sleep in mupip_backup to test concurrent BACKUPs */
+	WBTEST_FORCE_SEMGETERROR,		/* 83 : Send SEMGETERROR to gtmsecshr to validate it handles it correctly */
 	WBTEST_LONGSLEEP_IN_REPL_SHUTDOWN,	/* 84 : Sleep in Source/Receiver shutdown logic to ensure sem/shm is not removed */
 	WBTEST_FORCE_WCS_GET_SPACE,		/* 85 : Simulate state in which nearly all global buffers are dirty, forcing
 						 *      wcs_get_space to be called before committing an update */
@@ -224,7 +223,20 @@ typedef enum {
 	WBTEST_JOBFAIL_FILE_LIM,		/* 174 : starve the process of file descriptors to produce a JOBFAIL */
 	WBTEST_BACKUP_FORCE_MV_RV,		/* 175 : Force error during MUPIP BACKUP mv phase */
 	WBTEST_RTNOBJ_INTEG,			/* 176 : Inject a loop into the relinkctl linked list to trigger rtnobj_integ() */
-	WBTEST_INDUCE_TLSIOERR			/* 177 : Source Server does not close the network socket in error shutdown */
+	WBTEST_INDUCE_TLSIOERR,			/* 177 : Source Server does not close the network socket in error shutdown */
+	WBTEST_MU_STOP_THREAD,			/* 178 : Will sleep in gtm_multi_thread to let us verify that first two MUPIP STOPs
+                                                 *       are indeed deferred in the interrupt-deferred zone, but the third isn't */
+	WBTEST_FLUSH_FH,			/* 179 : Sleep in repl_inst_flush_gtmsrc_lcl to validate concurrency controls. */
+	WBTEST_PHS1_PRESLEEP,			/* 180 : Sleep right after releasing instance crit. */
+	WBTEST_PHS1_POSTSLEEP,			/* 181 : Sleep right after finishing phase1. */
+	WBTEST_SEQNO_PREKILL,			/* 182 : Kill right after incrementing the sequence number. */
+	WBTEST_REL_LOCK_PREKILL,		/* 183 : Kill right before releaseing instance crit. */
+	WBTEST_REL_LOCK_POSTKILL,		/* 184 : Kill after releasing instance crit and sleeping for a second. */
+	WBTEST_PHS1_NOSPACE,			/* 185 : Set fake enospc right before releasing instance crit. */
+	WBTEST_MUNMAP_FREE,			/* 186 : Verify that munmap() returns memory to the OS */
+	WBTEST_JNL_CLEANUP,			/* 187 : Various asynchronous events in a jnl cleanup */
+	WBTEST_ZTTEST,				/* 188 : Inject garbage collection at awkward place in sv_put.c SV_ZTRAP code */
+	WBTEST_JNL_PREAD			/* 189 : Benchmark pread() against lseek()+read() */
 
 	/* Note 1: when adding new white box test cases, please make use of WBTEST_ENABLED and WBTEST_ASSIGN_ONLY (defined below)
 	 * whenever applicable
@@ -232,6 +244,11 @@ typedef enum {
 	 */
 } wbtest_code_t;
 
+enum jnl_cleanup_wbox {
+	JNL_CLEANUP_KILL9 = 1,
+	JNL_CLEANUP_MUPIP_STOP = 300
+};
+
 #if defined (DEBUG) && !defined (STATIC_ANALYSIS)
 /* Make sure to setenv gtm_white_box_test_case_count if you are going to use GTM_WHITE_BOX_TEST */
 #define GTM_WHITE_BOX_TEST(input_test_case_num, lhs, rhs)						\
