@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2023 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -13,9 +13,9 @@
 /* maintained in conjunction with table in deviceparameters */
 
 typedef struct {
-	char offset;
-	char letter;
-}zshow_index;
+	unsigned char	offset;
+	unsigned char	letter;
+} zshow_index;
 
 enum zshow_params
 {
