@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2017 Fidelity National Information	*
+ * Copyright (c) 2001-2021 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -63,7 +63,7 @@ bool gtcmtr_put(void)
 	CM_GET_GVCURRKEY(ptr, len);
 	gtcm_bind_name(reg_ref->reghead, TRUE);
 	if (gv_cur_region->read_only)
-		rts_error_csa(CSA_ARG(cs_addrs) VARLSTCNT(4) ERR_DBPRIVERR, 2, DB_LEN_STR(gv_cur_region));
+		RTS_ERROR_CSA_ABT(cs_addrs, VARLSTCNT(4) ERR_DBPRIVERR, 2, DB_LEN_STR(gv_cur_region));
 	if (JNL_ALLOWED(cs_addrs))
 	{	/* we need to copy client's specific prc_vec into the global variable in order that the gvcst* routines
 		 *	do the right job. actually we need to do this only if JNL_ENABLED(cs_addrs), but since it is not
@@ -89,14 +89,14 @@ bool gtcmtr_put(void)
 	{
 		if ((end = format_targ_key(&buff[0], MAX_ZWR_KEY_SZ, gv_currkey, TRUE)) == 0)
 			end = &buff[MAX_ZWR_KEY_SZ - 1];
-		rts_error_csa(CSA_ARG(cs_addrs) VARLSTCNT(11) ERR_KEY2BIG, 4, n, (int4)gv_cur_region->max_key_size,
+		RTS_ERROR_CSA_ABT(cs_addrs, VARLSTCNT(11) ERR_KEY2BIG, 4, n, (int4)gv_cur_region->max_key_size,
 			REG_LEN_STR(gv_cur_region), 0, ERR_GVIS, 2, end - buff, buff);
 	}
 	if (n + v.str.len + SIZEOF(rec_hdr) > gv_cur_region->max_rec_size)
 	{
 		if ((end = format_targ_key(&buff[0], MAX_ZWR_KEY_SZ, gv_currkey, TRUE)) == 0)
 			end = &buff[MAX_ZWR_KEY_SZ - 1];
-		rts_error_csa(CSA_ARG(cs_addrs) VARLSTCNT(10) ERR_REC2BIG, 4, n + v.str.len + SIZEOF(rec_hdr),
+		RTS_ERROR_CSA_ABT(cs_addrs, VARLSTCNT(10) ERR_REC2BIG, 4, n + v.str.len + SIZEOF(rec_hdr),
 			(int4)gv_cur_region->max_rec_size, REG_LEN_STR(gv_cur_region),
 			ERR_GVIS, 2, end - buff, buff);
 	}
