@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2007 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2021 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -12,8 +13,8 @@
 #ifndef GVCMZ_INCLUDED
 #define GVCMZ_INCLUDED
 
+#include "gvcmz_neterr.h"			/* needs its own home for linking reasons */
 void gvcmz_error(char code, uint4 status);
-void gvcmz_neterr(INTPTR_T *error);
 void gvcmz_bunch(mval *v);
 void gvcmz_clrlkreq(void);
 void gvcmz_doop(unsigned char query_code, unsigned char reply_code, mval *v);
