@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2011 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2017 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -13,9 +14,7 @@
 #include "compiler.h"
 #include "comp_esc.h"
 
-GBLREF unsigned char *source_buffer;
-GBLREF short int source_column;
-GBLREF char *lexical_ptr;
+GBLREF int   source_column;
 GBLREF struct ce_sentinel_desc	*ce_def_list;
 
 void ce_substitute(struct ce_sentinel_desc *shp, int4 source_col, int4 *skip_ct)
