@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2022 Fidelity National Information	*
+ * Copyright (c) 2001-2023 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -16,7 +16,7 @@
 #include "gtm_signal.h"
 #include "gtm_unistd.h"
 #include "gtm_stdlib.h"
-#include "gtm_select.h"
+#include "gtm_poll.h"
 #include "gtm_string.h"
 
 #include <errno.h>
@@ -46,7 +46,7 @@
 #include "gtm_utf8.h"
 #endif
 
-GBLREF boolean_t		dmterm_default, gtm_utf8_mode, prin_in_dev_failure,
+GBLREF boolean_t		async_dm_read, dmterm_default, gtm_utf8_mode, prin_in_dev_failure,
 				prin_out_dev_failure;
 GBLREF char			*CLR_EOL, *CURSOR_DOWN, *CURSOR_LEFT, *CURSOR_RIGHT, *CURSOR_UP;
 GBLREF char			*KEY_BACKSPACE, *KEY_DC, *KEY_DOWN, *KEY_INSERT, *KEY_LEFT, *KEY_RIGHT, *KEY_UP;
@@ -149,7 +149,7 @@ if (0 < (TREF(gtmprompt)).len)													\
 	}												\
 }
 
-void	dm_read (mval *v)
+mval 	*dm_read (mval *v)
 {
 	boolean_t	buffer_moved, insert_mode, terminator_seen, utf8_active, zint_restart;
 #	ifdef UTF8_SUPPORTED
@@ -161,8 +161,7 @@ void	dm_read (mval *v)
 	char		*strtokptr;
 	const char	delimiter_string[] = " \t";
 	d_tt_struct 	*tt_ptr;
-	enum RECALL_ERR_CODE	err_recall = NO_ERROR;
-	fd_set		input_fd;
+	enum RECALL_ERR_CODE	err_recall;
 	int		backspace, cl, cur_cl, cur_value, delete, down, hist, histidx, index, insert_key, keypad_len, left;
 	int		delchar_width;		/* display width of deleted char */
 	int		delta_width;		/* display width change for replaced char */
@@ -173,22 +172,24 @@ void	dm_read (mval *v)
 	int		instr;			/* insert point in input string */
 	int		ioptr_width;		/* display width of the IO device */
 	int		outlen;			/* total characters in line so far */
-	int		match_length, msk_in, msk_num, num_chars_left, num_lines_above, right, selstat, status, up, utf8_more;
+	int		match_length, msk_in, msk_num, num_chars_left, num_lines_above, right, selstat, status, up, utf8_more = 0;
 	io_desc 	*io_ptr;
 	io_termmask	mask_term;
 	mv_stent	*mvc, *mv_zintdev;
-	struct timeval	input_timeval;
+	mval		dummy;
 	tt_interrupt	*tt_state;
 	uint4		mask;
 	unsigned int	exp_length, len, length;
-	unsigned char	*argv1;
 	unsigned char	*buffer_start;		/* beginning of non UTF8 buffer */
 	unsigned char	*current_ptr;		/* insert next character into buffer here */
 	unsigned char	escape_sequence[ESC_LEN];
 	unsigned char	inbyte, *outptr, *outtop, *ptr, *ptrnext, *ptrtop;
-	unsigned char	more_buf[GTM_MB_LEN_MAX + 1], *more_ptr;	/* to build up multi byte for character */
+	unsigned char	more_buf[GTM_MB_LEN_MAX + 1], *more_ptr = NULL;	/* to build up multi byte for character */
 	unsigned short	escape_length = 0;
-	wint_t		*buffer_32_start, codepoint, *current_32_ptr, inchar, *ptr32;
+	wint_t		*buffer_32_start = NULL, codepoint, *current_32_ptr, inchar, *ptr32;
+	int		poll_timeout;
+	nfds_t		poll_nfds;
+	struct pollfd	poll_fdlist[1];
 #	ifdef __MVS__
 	wint_t		asc_inchar;
 #	endif
@@ -234,6 +235,14 @@ void	dm_read (mval *v)
 			mv_zintdev->mv_st_cont.mvs_zintdev.io_ptr = NULL;
 			if (mv_chain == mv_zintdev)
 				POP_MV_STENT();		/* pop if top of stack */
+			else if ((mv_stent *)((char *) mv_chain + mv_chain->mv_st_next) == mv_zintdev)
+			{	/* ZINTDEV buried under input line on stack, so pop both and replace mval */
+				POP_MV_STENT();
+				assert(mv_chain == mv_zintdev);
+				POP_MV_STENT();
+				dummy.mvtype = dummy.str.len = 0;
+				v = push_mval(&dummy);
+			}
 			buffer_moved = (buffer_start != tt_state->buffer_start);
 			if (utf8_active)
 			{	/* need to properly align U32 buffer */
@@ -247,7 +256,7 @@ void	dm_read (mval *v)
 				current_32_ptr = buffer_32_start;
 				utf8_more = tt_state->utf8_more;
 				more_ptr = tt_state->more_ptr;
-				memcpy(more_buf, tt_state->more_buf, SIZEOF(more_buf));
+				memcpy((void*)more_buf, tt_state->more_buf, SIZEOF(more_buf));
 			}
 			instr = tt_state->instr;
 			outlen = tt_state->outlen;
@@ -319,6 +328,9 @@ void	dm_read (mval *v)
 				tt_state->who_saved = dmread;
 				tt_state->length = length;
 				tt_state->buffer_start = buffer_start;
+				/* since op_oldvar is not called in case of SIGUSR1, pop mval from top of stack */
+				assert(MVST_MVAL == mv_chain->mv_st_type);
+				POP_MV_STENT();
 				PUSH_MV_STENT(MVST_ZINTDEV);
 				mv_chain->mv_st_cont.mvs_zintdev.curr_sp_buffer.addr = (char *)buffer_start;
 				mv_chain->mv_st_cont.mvs_zintdev.curr_sp_buffer.len = exp_length;
@@ -326,6 +338,7 @@ void	dm_read (mval *v)
 				mv_chain->mv_st_cont.mvs_zintdev.io_ptr = io_ptr;
 				if (utf8_active)
 				{
+					assert(buffer_32_start);
 					tt_state->exp_length = exp_length;
 					tt_state->buffer_32_start = buffer_32_start;
 					tt_state->utf8_more = utf8_more;
@@ -351,22 +364,16 @@ void	dm_read (mval *v)
 			if (sighup == outofband)
 				exi_condition = -ERR_TERMHANGUP;
 			TAREF1(save_xfer_root, outofband).event_state = pending;
+			async_dm_read = TRUE;
 			async_action(FALSE);
 			break;
 		}
-		assertpro(FD_SETSIZE > tt_ptr->fildes);
-		FD_ZERO(&input_fd);
-		FD_SET(tt_ptr->fildes, &input_fd);
-		assert(0 != FD_ISSET(tt_ptr->fildes, &input_fd));
+		poll_fdlist[0].fd = tt_ptr->fildes;
+		poll_fdlist[0].events = POLLIN;
+		poll_nfds = 1;
 		/* Arbitrarily-chosen timeout value to prevent consumption of resources in tight loop when no input is available. */
-		input_timeval.tv_sec = 100;
-		input_timeval.tv_usec = 0;
-		/* N.B. On some Unix systems, the documentation for select() is ambiguous with respect to the first argument.
-		 * It specifies the number of contiguously-numbered file descriptors to be tested, starting with descriptor zero.
-		 * Thus, it should be equal to the highest-numbered file descriptor to test plus one.
-		 * (See _UNIX_Network_Programming_ by W. Richard Stevens, Section 6.13, pp. 328-331)
-		 */
-		selstat = select(tt_ptr->fildes + 1, (void *)&input_fd, (void *)NULL, (void *)NULL, &input_timeval);
+		poll_timeout = 100 * MILLISECS_IN_SEC;
+		selstat = poll(&poll_fdlist[0], poll_nfds, poll_timeout);
 		if (0 > selstat)
 			if (EINTR != errno)
 				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) errno);
@@ -385,14 +392,13 @@ void	dm_read (mval *v)
 			} else
 				continue;
 		} else if (0 == status)
-		{	/* select() says there's something to read, but read() found zero characters; assume connection dropped. */
+		{	/* poll() says there's something to read, but read() found zero characters; assume connection dropped. */
 			ISSUE_NOPRINCIO_IF_NEEDED(io_ptr, FALSE, FALSE);		/* FALSE, FALSE: READ tt not socket */
 			tt_ptr->discard_lf = FALSE;
 			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_IOEOF);
 		}
 		else if (0 < status)
-		{	/* select() says it's ready to read and read() found some data */
-			assert(0 != FD_ISSET(tt_ptr->fildes, &input_fd));
+		{
 			/* set prin_in_dev_failure to FALSE in case it was set to TRUE in the previous read which may have failed */
 			prin_in_dev_failure = FALSE;
 #			ifdef UTF8_SUPPORTED
@@ -406,6 +412,7 @@ void	dm_read (mval *v)
 				}
 				if (utf8_more)
 				{	/* needed extra bytes */
+					assert(more_ptr);
 					*more_ptr++ = inbyte;
 					if (--utf8_more)
 						continue;	/* get next byte */
@@ -487,7 +494,9 @@ void	dm_read (mval *v)
 			}
 			if (terminator_seen)
 			{	/* carriage return or other line terminator has been typed */
+				assert(!utf8_active || buffer_32_start);
 				STORE_OFF(0, outlen);
+				err_recall = NO_ERROR;
 				if (utf8_active && (ASCII_CR == INPUT_CHAR))
 					tt_ptr->discard_lf = TRUE;
 					/* exceeding the maximum length exits the while loop, so it must fit here . */
@@ -510,11 +519,13 @@ void	dm_read (mval *v)
 						&& ((' ' == GET_OFF(match_length)) || ('\t' == GET_OFF(match_length))))
 							matched = TRUE;		/* REC or RECALL then space or tab */
 					else if (matched)
+					{
 						if (((SIZEOF(RECALL) - 1) != match_length) && ((SIZEOF(REC) - 1) != match_length))
 							matched = FALSE;	/* wrong size */
 						else if ((outlen > match_length)
 							&& (' ' != GET_OFF(match_length) && ('\t' != GET_OFF(match_length))))
 								matched = FALSE;	/* or RECALL then not eol, space, or tab */
+					}
 					if (!matched)
 						break;		/* not RECALL so end of line */
 					match_length++;		/* get past space or tab */
@@ -537,12 +548,10 @@ void	dm_read (mval *v)
 					match_length = (uint4)strcspn((const char *)buffer_start, delimiter_string);
 					/* only "rec" and "recall" should be accepted */
 					if (((strlen(REC) == match_length) || (strlen(RECALL) == match_length))
-						&& (0 == strncmp((const char *)buffer_start, RECALL, match_length)))
-					{
-						for (argv1 = buffer_start + match_length; !(IS_AT_END_OF_STRINGPOOL(argv1, 0))
-								&& (' ' != argv1[0]) && ('\t' != argv1[1]); argv1++);
-						argv[1] = (char *)((IS_AT_END_OF_STRINGPOOL(argv1, 0)) ? NULL : argv1);
-					} else
+					     && (0 == strncmp((const char *)buffer_start, RECALL, match_length))
+					     && ((' ' == (inbyte = GET_OFF(match_length)) || !inbyte)))		/* WARNING assign */
+						argv[1] = inbyte ? (char *)(buffer_start + match_length + 1) : NULL;
+					else
 						break;		/* not RECALL so end of line */
 #				ifdef UTF8_SUPPORTED
 				}
@@ -651,6 +660,7 @@ void	dm_read (mval *v)
 			{
 				if (0 < instr)
 				{
+					assert(!utf8_active || buffer_32_start);
 					MOVE_CURSOR_LEFT_ONE_CHAR(dx, instr, dx_instr, dx_start, ioptr_width);
 					DEL_ONE_CHAR_AT_CURSOR(outlen, dx_outlen, dx, dx_instr, dx_start, ioptr_width);
 				}
@@ -676,6 +686,7 @@ void	dm_read (mval *v)
 				iott_escape(&escape_sequence[escape_length - 1], &escape_sequence[escape_length], io_ptr);
 			} else
 			{
+				assert(!utf8_active || buffer_32_start);
 				switch (inchar)
 				{
 					case EDIT_SOL:
@@ -845,7 +856,7 @@ void	dm_read (mval *v)
 				instr = dx_instr = outlen = dx_outlen = 0;
 				if (0 == up)
 				{
-					if (((MAX_RECALL + 1 != index) && (0 != (*(comline_base + cl)).len) || (0 == index)))
+					if (((MAX_RECALL + 1 != index) && ((0 != (*(comline_base + cl)).len) || (0 == index))))
 						index++;
 				} else
 				{
@@ -961,5 +972,5 @@ void	dm_read (mval *v)
 		}
 	}
 	active_device = 0;
-	return;
+	return v;
 }
