@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2022 Fidelity National Information	*
+ * Copyright (c) 2001-2023 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -111,8 +111,10 @@ static readonly CLI_ENTRY never_always_allowexisting[] = {
 };
 
 static readonly CLI_ENTRY db_vers[] = {
-{ "V4", 0, 0, 0, 0, 0, 0, VAL_DISALLOWED, 0, NON_NEG, VAL_N_A, 0 },
-{ "V6", 0, 0, 0, 0, 0, 0, VAL_DISALLOWED, 0, NON_NEG, VAL_N_A, 0 },
+{ "V6",  0, 0, 0, 0, 0, 0, VAL_DISALLOWED, 0, NON_NEG, VAL_N_A, 0 },
+{ "V6P", 0, 0, 0, 0, 0, 0, VAL_DISALLOWED, 0, NON_NEG, VAL_N_A, 0 },
+{ "V7",  0, 0, 0, 0, 0, 0, VAL_DISALLOWED, 0, NON_NEG, VAL_N_A, 0 },
+{ "V7M", 0, 0, 0, 0, 0, 0, VAL_DISALLOWED, 0, NON_NEG, VAL_N_A, 0 },
 { "" }
 };
 
@@ -132,6 +134,7 @@ static readonly CLI_ENTRY dse_cfhead_qual[] = {
 { "CORRUPT_FILE",              0, 0, 0,                     true_false_nochange,        0, 0, VAL_REQ,     0, NON_NEG, VAL_STR,  0       },
 { "CRIT",                      0, 0, 0,                     0,                          0, 0, VAL_N_A,     0, NEG,     0,        0       },
 { "CURRENT_TN",                0, 0, 0,                     0,                          0, 0, VAL_REQ,     0, NON_NEG, VAL_NUM,  VAL_HEX },
+{ "DATA_RESERVED_BYTES",       0, 0, 0,                     0,                          0, 0, VAL_REQ,     0, NON_NEG, VAL_NUM,  VAL_DCM },
 { "DB_WRITE_FMT",              0, 0, 0,                     db_vers,                    0, 0, VAL_REQ,     0, NON_NEG, VAL_STR,  0       },
 { "DECLOCATION",               0, 0, 0,                     0,                          0, 0, VAL_REQ,     0, NON_NEG, VAL_NUM,  VAL_DCM },
 { "DECVALUE",                  0, 0, 0,                     0,                          0, 0, VAL_REQ,     0, NON_NEG, VAL_NUM,  VAL_DCM },
@@ -147,6 +150,7 @@ static readonly CLI_ENTRY dse_cfhead_qual[] = {
 { "HARD_SPIN_COUNT",           0, 0, 0,                     0,                          0, 0, VAL_REQ,     0, NON_NEG, VAL_NUM,  VAL_DCM },
 { "HEXLOCATION",               0, 0, 0,                     0,                          0, 0, VAL_REQ,     0, NON_NEG, VAL_NUM,  VAL_HEX },
 { "HEXVALUE",                  0, 0, 0,                     0,                          0, 0, VAL_REQ,     0, NON_NEG, VAL_NUM,  VAL_HEX },
+{ "INDEX_RESERVED_BYTES",      0, 0, 0,                     0,                          0, 0, VAL_REQ,     0, NON_NEG, VAL_NUM,  VAL_DCM },
 { "INHIBIT_KILLS",             0, 0, 0,                     0,                          0, 0, VAL_REQ,     0, NON_NEG, VAL_NUM,  VAL_DCM },
 { "INTERRUPTED_RECOV",         0, 0, 0,                     true_false_nochange,        0, 0, VAL_REQ,     0, NON_NEG, VAL_STR,  0       },
 { "JNL_SYNCIO",                0, 0, 0,                     0,                          0, 0, VAL_REQ,     0, NON_NEG, VAL_STR,  0       },
