@@ -1,6 +1,6 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;								;
-; Copyright (c) 2006-2020 Fidelity National Information		;
+; Copyright (c) 2006-2024 Fidelity National Information		;
 ; Services, Inc. and/or its subsidiaries. All rights reserved.	;
 ;								;
 ;	This source code contains the intellectual property	;
@@ -33,6 +33,7 @@ LOAD
 	i gldfmt>6 s reghasv550fields=TRUE
 	s reghasv600fields=FALSE
 	i gldfmt>7 s reghasv600fields=TRUE
+	if (label="GTCGBDUNX015")!(label="GTCGBDUNX115") set label=hdrlab,update=1  ;autoconvert
 	s v6312=0
 	if (label="GTCGBDUNX014")!(label="GTCGBDUNX114") set label=hdrlab,v6312=1,update=1  ;autoconvert
 	if (v6312=1) new SIZEOF do v6312init
@@ -178,7 +179,11 @@ LOAD
 	d cretmps
 	i (reghasv550fields=TRUE) d tmpreg("ALIGNSIZE")
 	s s="ALLOCATION" d tmpreg(s)
-	if (gldfmt>10) d tmpreg("AUTODB") s tmpreg("STATS")='(tmpreg("AUTODB")\2#2),tmpreg("AUTODB")=tmpreg("AUTODB")#2
+	if (gldfmt>10) d
+	. d tmpreg("AUTODB")
+	. s tmpreg("STATS")='(tmpreg("AUTODB")\2#2)
+	. s tmpreg("AUTODB")=tmpreg("AUTODB")#2
+	. s:gldfmt>15 tmpreg("AUTODELETE")=(tmpreg("AUTODB")\8)#2
 	i (reghasv550fields=TRUE) d tmpreg("AUTOSWITCHLIMIT")
 	f s="BEFORE_IMAGE","BUFFER_SIZE" d tmpreg(s)
 	i tmpreg("BUFFER_SIZE")<minreg("BUFFER_SIZE")  s tmpreg("BUFFER_SIZE")=minreg("BUFFER_SIZE"),update=1
@@ -368,11 +373,13 @@ region:
 	i (gldfmt>8) s rel=rel+4 ; "gd_region.is_spanned" not needed by GDE
 	i (gldfmt>10) s rel=rel+4 ; "gd_region.statsDB_reg_index" never used by GDE (generated by gdeput if needed)
 	s regs(s,"EPOCHTAPER")=1
-	s regs(s,"AUTODB")=0,regs(s,"STATS")=1,regs(s,"LOCK_CRIT")=0
+	s regs(s,"AUTODB")=0,regs(s,"STATS")=1,regs(s,"AUTODELETE")=0,regs(s,"LOCK_CRIT")=0
 	i (gldfmt>9) do
 	. s regs(s,"EPOCHTAPER")=$$bin2num($ze(rec,rel)),rel=rel+1
 	. i (gldfmt>10) do
-	. . s x=$$bin2num($ze(rec,rel)),regs(s,"AUTODB")=x#2,regs(s,"STATS")='(x\2#2),rel=rel+1
+	. . s x=$$bin2num($ze(rec,rel)),regs(s,"AUTODB")=x#2,regs(s,"STATS")='(x\2#2)
+	. . s:gldfmt>15 regs(s,"AUTODELETE")=(x\8#2)
+	. . s rel=rel+1
 	. . s regs(s,"LOCK_CRIT")='$$bin2num($ze(rec,rel)),rel=rel+1
 	. . s rel=rel+34	; reserved for runtime fillers
 	. s rel=rel+11		; reserved for runtime fillers
@@ -512,6 +519,7 @@ cretmps:
 	s tmpreg("ALLOCATION")=2048
 	s tmpreg("AUTOSWITCHLIMIT")=8386560
 	s tmpreg("AUTODB")=0
+	s tmpreg("AUTODELETE")=0
 	s tmpreg("BEFORE_IMAGE")=1
 	s tmpreg("BUFFER_SIZE")=2312
 	s tmpreg("COLLATION_DEFAULT")=0
