@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2014 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -14,6 +15,7 @@
 #include "gtm_string.h"
 #include "gtm_stdlib.h"
 #include "gtm_unistd.h"
+#include "gtm_limits.h"
 
 #include "eintr_wrappers.h"
 #include "geteditor.h"
@@ -24,7 +26,7 @@ GBLDEF mstr editor;
 void geteditor(void)
 {
 	char		*edt, **pedt;
-	short		len;
+	size_t		len;
 	int		iter;
 	char		*editor_list[] =
 			{
@@ -37,18 +39,21 @@ void geteditor(void)
 	edt = GETENV("EDITOR");
 	pedt = &editor_list[0];
 	do {
-		if (0 == ACCESS(edt, (F_OK|X_OK))) /* if the file exists and is executable we are good */
+		if (0 == ACCESS(edt, (F_OK|X_OK)))	/* File exists and is executable, use it */
 			break;
 		edt = *pedt++;
 	} while (edt);
 	WBTEST_ASSIGN_ONLY(WBTEST_BADEDITOR_GETEDITOR, edt, 0);
 	if (edt)
 	{
-		len = strlen(edt) + 1;	/* for zero */
-		editor.len = len - 1;	/* but not for mstr */
-		editor.addr = (char*) malloc(len);	/* must be zero term */
+		len = strlen(edt);
+		if (PATH_MAX < len)			/* 4SCA: access() ensures path is < PATH_MAX */
+			len = PATH_MAX;
+		editor.len = (mstr_len_t)len;
+		if (PATH_MAX >= len)			/* 4SCA: Increment behind a guard */
+			len++;
+		editor.addr = (char*) malloc(len);	/* must be null terminated */
 		memcpy(editor.addr, edt, len);
-	}
-	else
+	} else
 		editor.len = 0;
 }
