@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2012 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2015 Fidelity National Information 	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -15,6 +16,7 @@
 #include "getjobnum.h"
 
 GBLREF	uint4	process_id;
+GBLREF	uid_t	user_id;
 
 void getjobnum(void)
 {
@@ -22,5 +24,5 @@ void getjobnum(void)
 
 	SETUP_THREADGBL_ACCESS;
 	process_id = getpid();
-	TREF(user_id) = (uint4)getuid();
+	user_id = (uint4)getuid();
 }
