@@ -1078,7 +1078,7 @@ DEFINE_ATOMIC_XCHG(OBJ_TYPE, ORD, PRE_BARRIER, POST_BARRIER)
 	DEFINE_ATOMIC_COMPARE_EXCHANGE(TYPE, memory_order_seq_cst, memory_order_seq_cst, "\tsync\n", "\tisync\n", "\tisync\n")
 #define DEFINE_REGULAR_ATOMIC_OP(TYPE, OPERATION, MEM_ORDER) DEFINE_##MEM_ORDER##_##OPERATION(TYPE)
 #define DEFINE_CE_ATOMIC_OP(TYPE, OPERATION, MEM_ORDER_1, MEM_ORDER_2) \
-	DEFINE_##MEM_ORDER_1##MEM_ORDER_2##_##ATOMIC_COMPARE_EXCHANGE(TYPE)
+	DEFINE_##MEM_ORDER_1##_##MEM_ORDER_2##_##ATOMIC_COMPARE_EXCHANGE(TYPE)
 #define GET_OPDEF_TYPE(_1, _2, _3, _4, OPNAME, ...) DEFINE_##OPNAME##_ATOMIC_OP
 #define DEFINE_ATOMIC_OP(...) GET_OPDEF_TYPE(__VA_ARGS__, CE, REGULAR)(__VA_ARGS__)
 #define DEFINE_ALL_ATOMIC_OPS(TYPE)		\
