@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2010-2021 Fidelity National Information	*
+ * Copyright (c) 2010-2023 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -79,7 +79,7 @@ GBLREF	uint4			t_err;
 GBLREF	int			tprestart_state;
 GBLREF	unsigned int		t_tries;
 GBLREF	unsigned char		t_fail_hist[CDB_MAX_TRIES];
-GBLREF	pid_t			process_id;
+GBLREF	uint4			process_id;
 GBLREF	trans_num		local_tn;
 #ifdef DEBUG
 GBLREF	uint4			dollar_trestart;
@@ -314,6 +314,19 @@ LITREF	mval	literal_null;
 			LEN_AND_LIT(SUBSCRIPT), ERR_TEXT, 2, RTS_ERROR_TEXT(MOREINFO));	\
 }
 
+/* This macro is used to give SCA info on size of the buf used in the memcpy().
+ * (More memory may be actually be allocated than is used by the "len" in the memcpy(), which is OK
+ * Hinting to SCA is done with dynamic typedefs which are only in scope in the macro context */
+#define ASSIGN_GVTR_SUBS_RANGE_KEYS(LEN,KEY_FIELD)										\
+{																\
+	typedef char new_elem_block[DIVIDE_ROUND_UP(LEN, GVTR_LIST_ELE_SIZE)];							\
+	typedef new_elem_block *new_elem_block_p;										\
+	new_elem_block_p dststart;												\
+	dststart = (new_elem_block_p) get_new_element(gvt_trigger->gv_trig_list, DIVIDE_ROUND_UP(LEN, GVTR_LIST_ELE_SIZE));	\
+	memcpy((char *) dststart, out_key->base, LEN);										\
+	subsdsc->KEY_FIELD = (char *) dststart;											\
+}
+
 /* This code is modeled around "updproc_ch" in updproc.c */
 CONDITION_HANDLER(gvtr_tpwrap_ch)
 {
@@ -421,7 +434,7 @@ STATICFNDEF uint4	gvtr_process_range(gv_namehead *gvt, gvtr_subs_t *subsdsc, int
 	char		keybuff[SIZEOF(gv_key) + DBKEYSIZE(MAX_KEY_SZ)];
 	gv_key		*out_key;
 	uint4		len, len1, min, nelems;
-	gvt_trigger_t	*gvt_trigger;
+	gvt_trigger_t	*gvt_trigger = NULL;
 	char		*dststart, *dstptr, *srcptr, ch;
 	boolean_t	ret;
 	int		cmpres;
@@ -493,11 +506,10 @@ STATICFNDEF uint4	gvtr_process_range(gv_namehead *gvt, gvtr_subs_t *subsdsc, int
 	{
 		case GVTR_PARSE_POINT:
 			assert(&subsdsc->gvtr_subs_type == &subsdsc->gvtr_subs_point.gvtr_subs_type);
+			assert(gvt_trigger);
 			len = out_key->end;	/* keep trailing 0 */
 			subsdsc->gvtr_subs_point.len = len;
-			dststart = (char *)get_new_element(gvt_trigger->gv_trig_list, DIVIDE_ROUND_UP(len, GVTR_LIST_ELE_SIZE));
-			memcpy(dststart, out_key->base, len);
-			subsdsc->gvtr_subs_point.subs_key = dststart;
+			ASSIGN_GVTR_SUBS_RANGE_KEYS(len,gvtr_subs_point.subs_key);
 			subsdsc->gvtr_subs_point.next_range = NULL;
 			subsdsc->gvtr_subs_type = GVTR_SUBS_POINT;
 			break;
@@ -505,12 +517,10 @@ STATICFNDEF uint4	gvtr_process_range(gv_namehead *gvt, gvtr_subs_t *subsdsc, int
 			assert(&subsdsc->gvtr_subs_type == &subsdsc->gvtr_subs_range.gvtr_subs_type);
 			if (len)
 			{
+				assert(gvt_trigger);
 				len = out_key->end;	/* keep trailing 0 */
 				assert(len);
-				dststart = (char *)get_new_element(gvt_trigger->gv_trig_list,
-									DIVIDE_ROUND_UP(len, GVTR_LIST_ELE_SIZE));
-				memcpy(dststart, out_key->base, len);
-				subsdsc->gvtr_subs_range.subs_key1 = dststart;
+				ASSIGN_GVTR_SUBS_RANGE_KEYS(len,gvtr_subs_range.subs_key1);
 				subsdsc->gvtr_subs_range.len1 = len;
 			} else
 				subsdsc->gvtr_subs_range.len1 = GVTR_RANGE_OPEN_LEN;
@@ -523,10 +533,8 @@ STATICFNDEF uint4	gvtr_process_range(gv_namehead *gvt, gvtr_subs_t *subsdsc, int
 			{
 				len = out_key->end;	/* keep trailing 0 */
 				assert(len);
-				dststart = (char *)get_new_element(gvt_trigger->gv_trig_list,
-									DIVIDE_ROUND_UP(len, GVTR_LIST_ELE_SIZE));
-				memcpy(dststart, out_key->base, len);
-				subsdsc->gvtr_subs_range.subs_key2 = dststart;
+				assert(gvt_trigger);
+				ASSIGN_GVTR_SUBS_RANGE_KEYS(len,gvtr_subs_range.subs_key2);
 				subsdsc->gvtr_subs_range.len2 = len;
 				len1 = subsdsc->gvtr_subs_range.len1;
 				if (GVTR_RANGE_OPEN_LEN != len1)
@@ -534,7 +542,7 @@ STATICFNDEF uint4	gvtr_process_range(gv_namehead *gvt, gvtr_subs_t *subsdsc, int
 					min = MIN(len1, len);
 					cmpres = memcmp(subsdsc->gvtr_subs_range.subs_key1,
 								subsdsc->gvtr_subs_range.subs_key2, min);
-					if ((0 < cmpres) || (0 == cmpres) && (len1 > len))
+					if ((0 < cmpres) || ((0 == cmpres) && (len1 > len)))
 					{	/* Invalid range. Issue error */
 						DBG_MARK_STRINGPOOL_USABLE;
 						return ERR_TRIGSUBSCRANGE;
@@ -577,7 +585,7 @@ STATICFNDEF uint4	gvtr_process_pattern(char *ptr, uint4 len, gvtr_subs_t *subsds
 	memcpy(source_buffer, ptr, len);
 	instr.addr = source_buffer;
 	instr.len = len;
-	if (status = patstr(&instr, &pat_ptstr, NULL))
+	if ((status = patstr(&instr, &pat_ptstr, NULL)))
 		return status;
 	assert(pat_ptstr.len <= MAX_PATOBJ_LENGTH);
 	len = pat_ptstr.len * SIZEOF(uint4);
@@ -599,7 +607,7 @@ STATICFNDEF uint4 gvtr_process_gvsubs(char *start, char *end, gvtr_subs_t *subsd
 	if ('?' == start[0])
 	{
 		assert(!colon_imbalance);
-		if (status = gvtr_process_pattern(start, UINTCAST(end - start), subsdsc, gvt->gvt_trigger))
+		if ((status = gvtr_process_pattern(start, UINTCAST(end - start), subsdsc, gvt->gvt_trigger)))
 			return status;
 	} else if ('*' == start[0])
 	{	/* subscript is a "*" */
@@ -609,7 +617,7 @@ STATICFNDEF uint4 gvtr_process_gvsubs(char *start, char *end, gvtr_subs_t *subsd
 		subsdsc->gvtr_subs_star.next_range = NULL;
 	} else
 	{
-		if (status = gvtr_process_range(gvt, subsdsc, colon_imbalance ? GVTR_PARSE_RIGHT : GVTR_PARSE_POINT, start, end))
+		if ((status = gvtr_process_range(gvt, subsdsc, colon_imbalance ? GVTR_PARSE_RIGHT : GVTR_PARSE_POINT, start, end)))
 		{
 			/* As of now we expect only ERR_TRIGSUBSCRANGE (which is triggered only for GVTR_PARSE_RIGHT) */
 			assert((ERR_TRIGSUBSCRANGE == status) && colon_imbalance);
@@ -633,13 +641,13 @@ void	gvtr_db_read_hasht(sgmnt_addrs *csa)
 	gvt_trigger_t		*gvt_trigger;
 	uint4			trigidx, num_gv_triggers, num_pieces, len, cmdtype, index, minpiece, maxpiece;
 	gv_trigger_t		*gv_trig_array, *trigdsc, *trigtop;
-	uint4			currkey_end, cycle, numsubs, cursub, numlvsubs, curlvsub;
+	uint4			currkey_end, cycle, numsubs, cursub, numlvsubs, curlvsub = -1;
 	char			ch, *ptr, *ptr_top, *ptr_start;
 	boolean_t		quote_imbalance, colon_imbalance, end_of_subscript;
 	uint4			paren_imbalance;
 	gvtr_subs_t		*subsdsc;
-	mname_entry		*lvnamedsc;
-	uint4			*lvindexdsc, status;
+	mname_entry		*lvnamedsc = NULL;
+	uint4			*lvindexdsc = NULL, status;
 	int			ctype;
 	char			save_rtn_name[MAX_TRIGNAME_LEN], save_var_name[MAX_MIDENT_LEN];
 	uint4			save_rtn_name_len, save_var_name_len;
@@ -656,11 +664,11 @@ void	gvtr_db_read_hasht(sgmnt_addrs *csa)
 	/* Save gv_currkey and gv_altkey */
 	assert(NULL != gv_currkey);
 	assert((SIZEOF(gv_key) + gv_currkey->end + 1) <= SIZEOF(save_currkey));
-	save_gv_currkey = (gv_key *)&save_currkey.key;
+	save_gv_currkey = &save_currkey.key;
 	MEMCPY_KEY(save_gv_currkey, gv_currkey);
 	assert(NULL != gv_altkey);
 	assert((SIZEOF(gv_key) + gv_altkey->end + 1) <= SIZEOF(save_altkey));
-	save_gv_altkey = (gv_key *)&save_altkey.key;
+	save_gv_altkey = &save_altkey.key;
 	MEMCPY_KEY(save_gv_altkey, gv_altkey);
 	save_gv_last_subsc_null = TREF(gv_last_subsc_null);
 	save_gv_some_subsc_null = TREF(gv_some_subsc_null);
@@ -959,17 +967,19 @@ void	gvtr_db_read_hasht(sgmnt_addrs *csa)
 					end_of_subscript = TRUE;
 				else if ('=' == ch)
 				{	/* a local variable name has been specified for a subscript */
+					assert(0 <= curlvsub);
 					assert(curlvsub < numlvsubs);
+					assert(lvindexdsc);
 					*lvindexdsc++ = cursub;
 					len = UINTCAST(ptr - ptr_start);
 					assert(len);
 #					ifdef DEBUG
 					/* Check validity of lvname */
-					ctype = ctypetab[ptr_start[0]];
+					ctype = ctypetab[(unsigned char)ptr_start[0]];
 					assert((TK_PERCENT == ctype) || (TK_LOWER == ctype) || (TK_UPPER == ctype));
 					for (index = 1; index < len; index++)
 					{
-						ctype = ctypetab[ptr_start[index]];
+						ctype = ctypetab[(unsigned char)ptr_start[index]];
 						assert((TK_LOWER == ctype) || (TK_UPPER == ctype) || (TK_DIGIT == ctype));
 					}
 #					endif
@@ -1250,7 +1260,7 @@ STATICFNDEF	boolean_t	gvtr_is_key_a_match(char *keysub_start[], gv_trigger_t *tr
 					{
 						min = MIN(len1, thissublen);
 						cmpres = memcmp(subsdsc1->gvtr_subs_range.subs_key1, thissub, min);
-						if ((0 < cmpres) || (0 == cmpres) && (thissublen < len1))
+						if ((0 < cmpres) || ((0 == cmpres) && (thissublen < len1)))
 							left_side_matched = FALSE;
 					}
 					/* Check right side of range */
@@ -1262,7 +1272,7 @@ STATICFNDEF	boolean_t	gvtr_is_key_a_match(char *keysub_start[], gv_trigger_t *tr
 						{
 							min = MIN(len2, thissublen);
 							cmpres = memcmp(subsdsc1->gvtr_subs_range.subs_key2, thissub, min);
-							if ((0 > cmpres) || (0 == cmpres) && (thissublen > len2))
+							if ((0 > cmpres) || ((0 == cmpres) && (thissublen > len2)))
 								right_side_matched = FALSE;
 						}
 						if (right_side_matched)
@@ -1474,7 +1484,7 @@ int	gvtr_match_n_invoke(gtm_trigger_parms *trigparms, gvtr_invoke_parms_t *gvtr_
 	gv_key_buf		save_currkey;
 	gv_trigger_t		*trigdsc, *trigstop, *trigstart;
 	int			gtm_trig_status, tfxb_status, num_triggers_invoked, trigmax, trig_list_offset;
-	mstr			*ztupd_mstr;
+	mstr			*ztupd_mstr = NULL;
 	mval			*keysub_mval;
 	mval			*lvvalarray[MAX_GVSUBSCRIPTS + 1];
 	mval			*ztupd_mval, dummy_mval;
@@ -1531,7 +1541,7 @@ int	gvtr_match_n_invoke(gtm_trigger_parms *trigparms, gvtr_invoke_parms_t *gvtr_
 	 * gets opened) inside any of the "gtm_trigger" invocations. So we should maintain pointers only
 	 * to the local copy (not the global gv_currkey) for use inside all iterations of the for loop.
 	 */
-	save_gv_currkey = (gv_key *)&save_currkey.key;
+	save_gv_currkey = &save_currkey.key;
 	assert(((char *)save_gv_currkey + SIZEOF(gv_key) + gv_currkey->end + 1) < (char *)(&save_currkey + 1));
 	MEMCPY_KEY(save_gv_currkey, gv_currkey);
 	key_ptr = (char *)save_gv_currkey->base;
@@ -1630,6 +1640,7 @@ int	gvtr_match_n_invoke(gtm_trigger_parms *trigparms, gvtr_invoke_parms_t *gvtr_
 			if (is_set_trigger)
 			{
 				assert(0 == ztupd_mval->mvtype);
+				assert(ztupd_mstr);
 				if (trigdsc->delimiter.str.len)
 				{
 					trigparms->ztdelim_new = (mval *)&trigdsc->delimiter;
