@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2021 Fidelity National Information	*
+ * Copyright (c) 2001-2023 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -155,6 +155,7 @@ boolean_t incr_link(int *file_desc, zro_ent *zro_entry, uint4 fname_len, char *f
 	rhdtyp			*old_rhead;
 	rtn_tabent		*tabent_ptr;
 	int			sect_ro_rel_size, sect_rw_rel_size, name_buf_len, alloc_len, order, zerofd;
+	bool			order_valid;
 	uint4			lcl_compiler_qlf;
 	boolean_t		dynlits;
 	ssize_t	 		status, sect_rw_nonrel_size, sect_ro_rel_offset;
@@ -487,7 +488,7 @@ boolean_t incr_link(int *file_desc, zro_ent *zro_entry, uint4 fname_len, char *f
 		zhent = (LINK_SHROBJ == linktyp) ? recent_zhist->end - 1 : NULL;
 		if ((old_rhead->objhash == hdr->objhash)
 		    && (((NULL == zhent) && (NULL == old_rhead->relinkctl_bkptr))			/* Both non-relink */
-			|| (NULL != zhent) && (zhent->relinkctl_bkptr == old_rhead->relinkctl_bkptr)))  /* Both same relink */
+			|| ((NULL != zhent) && (zhent->relinkctl_bkptr == old_rhead->relinkctl_bkptr))))  /* Both same relink */
 #		else
 		if (old_rhead->objhash == ((rhdtyp *)hdr)->objhash)
 #		endif
@@ -674,8 +675,7 @@ boolean_t incr_link(int *file_desc, zro_ent *zro_entry, uint4 fname_len, char *f
 	 */
 	alloc_len = hdr->linkage_len * SIZEOF(lnk_tabent);
 	hdr->linkage_adr = (lnk_tabent *)malloc((0 != alloc_len) ? alloc_len : SIZEOF(lnk_tabent));
-	assert(PADLEN(hdr->linkage_adr, SIZEOF(lnk_tabent) == 0));
-	assert(((UINTPTR_T)hdr->linkage_adr % SIZEOF(lnk_tabent)) == 0);
+	assert((0 == (PADLEN(hdr->linkage_adr, SIZEOF(lnk_tabent)))) && (0 == ((UINTPTR_T)hdr->linkage_adr % SIZEOF(lnk_tabent))));
 	memset((char *)hdr->linkage_adr, 0, (hdr->linkage_len * SIZEOF(lnk_tabent)));
 	/* Relocations for read-write releasable section. Perform relocation on literal mval table and
 	 * variable table entries since they both point to the offsets from the beginning of the
@@ -784,14 +784,18 @@ boolean_t incr_link(int *file_desc, zro_ent *zro_entry, uint4 fname_len, char *f
 		lbt_ent = lbt_bot;
 		olbt_bot = old_rhead->labtab_adr;
 		olbt_top = olbt_bot + old_rhead->labtab_len;
+		order = 0;
+		order_valid = FALSE;
 		for (olbt_ent = olbt_bot;  olbt_ent < olbt_top;  olbt_ent++)
 		{	/* Match new label entries with old label entries */
 			for (; lbt_ent < lbt_top; lbt_ent++)
 			{
 				MIDENT_CMP(&olbt_ent->lab_name, &lbt_ent->lab_name, order);
+				order_valid = TRUE;
 				if (0 >= order)
 					break;
 			}
+			assert((lbt_ent >= lbt_top) || order_valid);
 			if ((lbt_ent < lbt_top) && !order)
 			{	/* Have a label name match. Update line pointer for this entry */
 				olbt_ent->lnr_adr = lbt_ent->lnr_adr;
