@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2014 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2021 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -17,24 +18,27 @@
 #include "gtm_unistd.h"
 #include "gtm_stat.h"
 #include "gtm_stdio.h"
+#include "gtm_string.h"
 
 #include "io.h"
 #include "iormdef.h"
 #include "io_params.h"
-#include "eintr_wrappers.h"
+#include "gtm_signal.h"
 #include "gtmio.h"
 #include "iosp.h"
-#include "string.h"
 #include "stringpool.h"
 #include "iotimer.h"
 #include "gt_timer.h"
 #include "wake_alarm.h"
 #include "copy.h"
+#include "error.h"
+#include "is_file_identical.h"
 
 GBLREF io_pair		io_curr_device;
 GBLREF io_pair		io_std_device;
-GBLREF	boolean_t	gtm_pipe_child;
-GBLREF	volatile bool	out_of_time;
+GBLREF boolean_t	gtm_pipe_child;
+GBLREF volatile bool	out_of_time;
+GBLREF int		process_exiting;
 
 error_def(ERR_SYSCALL);
 error_def(ERR_DEVPARMTOOSMALL);
@@ -50,6 +54,7 @@ void iorm_close(io_desc *iod, mval *pp)
 	unsigned char	c;
 	char		*path, *path2;
 	int		stat_res;
+	int		save_errno;
 	int		fstat_res;
 	struct stat	statbuf, fstatbuf;
 	int		p_offset;
@@ -60,16 +65,20 @@ void iorm_close(io_desc *iod, mval *pp)
         int4            wait_status;
 #endif
 
-	int  		status;
+	int  		rc, status;
 	unsigned int	*dollarx_ptr;
 	unsigned int	*dollary_ptr;
 	char 		*savepath2 = 0;
 	int		path2len;
 	boolean_t	rm_destroy = TRUE;
 	boolean_t	rm_rundown = FALSE;
+	boolean_t	rn_overwrite = FALSE;
 	TID		timer_id;
 	int4		pipe_timeout = 2;	/* default timeout in sec waiting for waitpid */
 	off_t		cur_position;
+	boolean_t	ch_set;
+	sigset_t	empty_set, old_set;
+	boolean_t	use_timer;
 
 	DCL_THREADGBL_ACCESS;
 
@@ -82,6 +91,7 @@ void iorm_close(io_desc *iod, mval *pp)
 		return;
 	}
 
+	ESTABLISH_GTMIO_CH(&iod->pair, ch_set);
 	rm_ptr = (d_rm_struct *)iod->dev_sp;
 
 #ifdef __MVS__
@@ -100,7 +110,7 @@ void iorm_close(io_desc *iod, mval *pp)
 	iorm_use(iod,pp);
 	/* We do not want a NEWLINE to be issued by the middle process. */
 	if (!gtm_pipe_child)
-		iorm_flush(iod);
+		iorm_cond_wteol(iod);
 
 	p_offset = 0;
 	while (*(pp->str.addr + p_offset) != iop_eol)
@@ -111,23 +121,51 @@ void iorm_close(io_desc *iod, mval *pp)
 				path = iod->trans_name->dollar_io;
 				FSTAT_FILE(rm_ptr->fildes, &fstatbuf, fstat_res);
 				if (-1 == fstat_res)
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) errno);
+				{
+					save_errno = errno;
+					SET_DOLLARDEVICE_ONECOMMA_STRERROR(iod, save_errno);
+					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) save_errno);
+				}
 				STAT_FILE(path, &statbuf, stat_res);
 				if (-1 == stat_res)
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) errno);
+				{
+					save_errno = errno;
+					SET_DOLLARDEVICE_ONECOMMA_STRERROR(iod, save_errno);
+					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) save_errno);
+				}
 				if (CYGWIN_ONLY(rm_ptr->fifo ||) fstatbuf.st_ino == statbuf.st_ino)
 					if (UNLINK(path) == -1)
-						rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) errno);
+					{
+						save_errno = errno;
+						SET_DOLLARDEVICE_ONECOMMA_STRERROR(iod, save_errno);
+						rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) save_errno);
+					}
 				break;
+			case iop_noinsert:
+				rn_overwrite = TRUE;
+				break;
+			case iop_replace:
+				/* iop_rename is intended to protect against accidental replacement.
+				 * iop_replace(REPLACE on CLOSE) provides an overwrite functionality.
+				 */
+				rn_overwrite = TRUE;				/* WARNING fallthrough */
 			case iop_rename:
 				path = iod->trans_name->dollar_io;
 				path2 = (char*)(pp->str.addr + p_offset + 1);
 				FSTAT_FILE(rm_ptr->fildes, &fstatbuf, fstat_res);
 				if (-1 == fstat_res)
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) errno);
+				{
+					save_errno = errno;
+					SET_DOLLARDEVICE_ONECOMMA_STRERROR(iod, save_errno);
+					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) save_errno);
+				}
 				STAT_FILE(path, &statbuf, stat_res);
 				if (-1 == stat_res)
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) errno);
+				{
+					save_errno = errno;
+					SET_DOLLARDEVICE_ONECOMMA_STRERROR(iod, save_errno);
+					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) save_errno);
+				}
 				if (CYGWIN_ONLY(rm_ptr->fifo ||) fstatbuf.st_ino == statbuf.st_ino)
 				{
 					/* make a copy of path2 so we can null terminate it */
@@ -140,10 +178,23 @@ void iorm_close(io_desc *iod, mval *pp)
 					stringpool.free += path2len + 1;
 					assert(stringpool.free >= stringpool.base);
 					assert(stringpool.free <= stringpool.top);
-					if (LINK(path, savepath2) == -1)
-						rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) errno);
-					if (UNLINK(path) == -1)
-						rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) errno);
+					if (is_file_identical(path, savepath2))
+						break;					/* rename or replace to itself a noop */
+					if (rn_overwrite)
+						UNLINK(savepath2); /* overwrite deletes now, but rename alone requires delete 1st */
+test_rename_gap:
+					if (-1 == LINK(path, savepath2))
+					{
+						save_errno = errno;
+						SET_DOLLARDEVICE_ONECOMMA_STRERROR(iod, save_errno);
+						rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) save_errno);
+					}
+					if (-1 == UNLINK(path))
+					{
+						save_errno = errno;
+						SET_DOLLARDEVICE_ONECOMMA_STRERROR(iod, save_errno);
+						rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) save_errno);
+					}
 				}
 				break;
 			case iop_destroy:
@@ -158,7 +209,7 @@ void iorm_close(io_desc *iod, mval *pp)
 			case iop_timeout:
 				GET_LONG(pipe_timeout, (pp->str.addr + p_offset));
 				if (1 > pipe_timeout)
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_DEVPARMTOOSMALL);
+					RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_DEVPARMTOOSMALL);
 				break;
 			default:
 				break;
@@ -174,7 +225,7 @@ void iorm_close(io_desc *iod, mval *pp)
 
 	iod->state = dev_closed;
 	/* save no_destroy for disk device */
-	if ((FALSE == rm_destroy) && !rm_ptr->fifo && !rm_ptr->pipe && (2 < rm_ptr->fildes))
+	if ((FALSE == rm_destroy) && !rm_ptr->fifo && !rm_ptr->is_pipe && (2 < rm_ptr->fildes))
 	{
 		rm_ptr->no_destroy = TRUE;
 		/* We can write anywhere in the file so need to save current file pointer for re-open
@@ -186,9 +237,11 @@ void iorm_close(io_desc *iod, mval *pp)
 			cur_position = lseek(rm_ptr->fildes, (off_t)0, SEEK_CUR);
 			if ((off_t)-1 == cur_position)
 			{
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(9) ERR_IOERROR, 7,
-					      RTS_ERROR_LITERAL("lseek"),
-					      RTS_ERROR_LITERAL("iorm_close()"), CALLFROM, errno);
+				save_errno = errno;
+				SET_DOLLARDEVICE_ONECOMMA_STRERROR(iod, save_errno);
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(9) ERR_IOERROR, 7,
+					RTS_ERROR_LITERAL("lseek"),
+					RTS_ERROR_LITERAL("iorm_close()"), CALLFROM, save_errno);
 			} else
 				rm_ptr->file_pos = cur_position;
 		}
@@ -209,6 +262,16 @@ void iorm_close(io_desc *iod, mval *pp)
 			free(rm_ptr->outbuf);
 			rm_ptr->outbuf = NULL;
 		}
+		if (rm_ptr->tmp_buffer)
+		{
+			free(rm_ptr->tmp_buffer);
+			rm_ptr->tmp_buffer = NULL;
+		}
+		if (iod->dollar.devicebuffer)
+		{
+			free(iod->dollar.devicebuffer);
+			iod->dollar.devicebuffer = NULL;
+		}
 	}
 
 	/* Do the close first. If the fclose is done first and we are being called from io_rundown just prior to the execv
@@ -231,53 +294,71 @@ void iorm_close(io_desc *iod, mval *pp)
 		assert(FD_INVALID == rm_ptr->read_fildes);
 		assert(NULL == rm_ptr->read_filstr);
 	}
-	/* reap the forked shell process if a pipe - it will be a zombie, otherwise*/
+	/* Reap the forked shell process if a pipe - it will be a zombie otherwise. */
 	if (rm_ptr->pipe_pid > 0)
-	{
-		/* Don't reap if in a child process creating a new pipe or if in parent and independent is set */
+	{	/* Do not reap if in a child process creating a new pipe or if in parent and independent is set. */
 		if (FALSE == gtm_pipe_child)
 		{
 			if (!rm_ptr->independent)
 			{
-				/* start timer for pipe_timeout sec to wait for reap of close via waitpid.  If this
-				   times out, set dollar_zclose = -99 */
+				if (!process_exiting)
+				{	/* Find out whether timers are available. If not, instead of scheduling one to interrupt
+					 * waitpid, simply invoke it with WNOHANG flag instead. Note that SIGPROCMASK below is
+					 * operating on an empty signal set and so is not blocking any signals.
+					 */
+					sigemptyset(&empty_set);
+					SIGPROCMASK(SIG_BLOCK, &empty_set, &old_set, rc);
+					use_timer = !sigismember(&old_set, SIGALRM);
+				} else
+					use_timer = FALSE;
 				out_of_time = FALSE;
-				timer_id = (TID)iorm_close;
-				start_timer(timer_id, pipe_timeout * 1000, wake_alarm, 0, NULL);
+				if (use_timer)
+				{	/* Start timer for pipe_timeout seconds to wait for reap of close via waitpid. If this times
+					 * out, set dollar_zclose = -99.
+					 */
+					timer_id = (TID)iorm_close;
+					start_timer(timer_id, pipe_timeout * 1000, wake_alarm, 0, NULL);
+				}
 				do
 				{
-					done_pid = waitpid(rm_ptr->pipe_pid, &status, 0); /* BYPASSOK */
-				} while((pid_t)-1 == done_pid && EINTR == errno && !out_of_time);
-
-				if ((pid_t)-1 == done_pid && !out_of_time)
+					status = 0;
+					done_pid = waitpid(rm_ptr->pipe_pid, &status, use_timer ? 0 : WNOHANG); /* BYPASSOK */
+				} while (((pid_t)-1 == done_pid) && (EINTR == errno) && (!out_of_time));
+				if (((pid_t)-1 == done_pid) && (!out_of_time))
 				{
-					cancel_timer(timer_id);
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_SYSCALL, 5,
-						      RTS_ERROR_LITERAL("waitpid"), CALLFROM, errno);
+					save_errno = errno;
+					if (use_timer)
+						cancel_timer(timer_id);
+					SET_DOLLARDEVICE_ONECOMMA_STRERROR(iod, save_errno);
+					RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_SYSCALL, 5,
+						RTS_ERROR_LITERAL("waitpid"), CALLFROM, save_errno);
 				}
-				if (out_of_time)
+				if (out_of_time || (!use_timer && (0 == done_pid)))
 				{
 					TREF(dollar_zclose) = -99;
 				} else
-				{
-					/* not an error and didn't timeout */
-					assert(done_pid == rm_ptr->pipe_pid);
-					/* cancel timer since it didn't timeout */
-					cancel_timer(timer_id);
-#ifdef _BSD
+				{	/* Not an error and did not timeout. */
+					assert(!use_timer || (done_pid == rm_ptr->pipe_pid));
+					/* Cancel timer since it did not timeout. */
+					if (use_timer)
+						cancel_timer(timer_id);
+#					ifdef _BSD
 					assert(SIZEOF(wait_status) == SIZEOF(int4));
+					/* The WIF* macros expect a union wait_stat as an argument on BSD. */
 					wait_status.w_status = status;
-					/* the WIF* macros expect a union wait_stat as an argument on BSD */
-#else
+#					else
 					wait_status = status;
-#endif
+#					endif
 					if (WIFEXITED(wait_status))
-						TREF(dollar_zclose) = WEXITSTATUS(wait_status); /* normal exit */
-					else if (WIFSIGNALED(wait_status))
-						/* change the signal to a negative for distinguishing from normal exit codes */
+					{	/* Normal exit. */
+						TREF(dollar_zclose) = WEXITSTATUS(wait_status);
+					} else if (WIFSIGNALED(wait_status))
+					{	/* Change signal to negative for distinguishing from normal exit codes. */
 						TREF(dollar_zclose) = -WTERMSIG(wait_status);
-					else /* set any other non-normal status to -98 */
+					} else
+					{	/* Set any other non-normal status to -98. */
 						TREF(dollar_zclose) = -98;
+					}
 				}
 			}
 		}
@@ -299,7 +380,12 @@ void iorm_close(io_desc *iod, mval *pp)
 			iorm_close(rm_ptr->stderr_child,pp);
 		}
 	}
-	if ((rm_destroy || rm_ptr->pipe || rm_ptr->fifo) && !rm_rundown)
+	if ((rm_destroy || rm_ptr->is_pipe || rm_ptr->fifo) && !rm_rundown)
 	        remove_rms (iod);
+	REVERT_GTMIO_CH(&iod->pair, ch_set);
 	return;
+	assertpro(FALSE);	/* ensure we never reach the goto below */
+	goto test_rename_gap;	/* Dummy goto makes the compiler satisfied the label used as a breakpoint in a test is legit.
+				 * The test_rename_gap label is used by gde/gdeput test.
+				 */
 }
