@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2022 Fidelity National Information	*
+ * Copyright (c) 2001-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -345,24 +345,26 @@ static readonly CLI_PARM mup_integ_map_parm[] = {
 };
 
 static	CLI_ENTRY	mup_integ_qual[] = {		/* INTEG */
-{ "ADJACENCY",   mupip_integ, 0, 0,                  0, 0, 0, VAL_REQ,        1, NON_NEG, VAL_NUM, 0       },
-{ "ANALYZE",     mupip_integ, 0, 0,                  0, 0, 0, VAL_REQ,        1, NON_NEG, VAL_STR, 0       },
-{ "BLOCK",       mupip_integ, 0, 0,                  0, 0, 0, VAL_REQ,        1, NON_NEG, VAL_NUM, VAL_HEX },
-{ "BRIEF",       mupip_integ, 0, 0,                  0, 0, 0, VAL_DISALLOWED, 1, NON_NEG, VAL_N_A, 0       },
-{ "DBG",         mupip_integ, 0, 0,                  0, 0, 0, VAL_DISALLOWED, 2, NON_NEG, VAL_N_A, 0       },
-{ "FAST",        mupip_integ, 0, 0,                  0, 0, 0, VAL_DISALLOWED, 1, NON_NEG, VAL_N_A, 0       },
-{ "FILE",        mupip_integ, 0, 0,                  0, 0, 0, VAL_DISALLOWED, 1, NON_NEG, VAL_N_A, 0       },
-{ "FULL",        mupip_integ, 0, 0,                  0, 0, 0, VAL_DISALLOWED, 1, NON_NEG, VAL_N_A, 0       },
-{ "KEYRANGES",   mupip_integ, 0, 0,                  0, 0, 0, VAL_DISALLOWED, 1, NEG,     VAL_N_A, 0       },
-{ "MAP",         mupip_integ, 0, mup_integ_map_parm, 0, 0, 0, VAL_NOT_REQ,    1, NEG,     VAL_NUM, 0       },
-{ "MAXKEYSIZE",  mupip_integ, 0, mup_integ_map_parm, 0, 0, 0, VAL_NOT_REQ,    1, NEG,     VAL_NUM, 0       },
-{ "ONLINE",      mupip_integ, 0, 0,                  0, 0, 0, VAL_DISALLOWED, 2, NEG,     VAL_N_A, 0       },
-{ "PRESERVE",    mupip_integ, 0, 0,                  0, 0, 0, VAL_NOT_REQ,    1, NON_NEG, VAL_N_A, 0       },
-{ "REGION",      mupip_integ, 0, 0,                  0, 0, 0, VAL_DISALLOWED, 1, NON_NEG, VAL_N_A, 0       },
-{ "STATS",       mupip_integ, 0, 0,                  0, 0, 0, VAL_DISALLOWED, 1, NEG,     VAL_N_A, 0       },
-{ "SUBSCRIPT",   mupip_integ, 0, 0,                  0, 0, 0, VAL_REQ,        1, NON_NEG, VAL_STR, 0       },
-{ "TN_RESET",    mupip_integ, 0, 0,                  0, 0, 0, VAL_DISALLOWED, 1, NON_NEG, VAL_N_A, 0       },
-{ "TRANSACTION", mupip_integ, 0, mup_integ_map_parm, 0, 0, 0, VAL_NOT_REQ,    1, NEG,     VAL_NUM, 0       },
+{ "ADJACENCY",     mupip_integ, 0, 0,                  0, 0, 0, VAL_REQ,        1, NON_NEG, VAL_NUM, 0       },
+{ "ANALYZE",       mupip_integ, 0, 0,                  0, 0, 0, VAL_REQ,        1, NON_NEG, VAL_STR, 0       },
+{ "BLOCK",         mupip_integ, 0, 0,                  0, 0, 0, VAL_REQ,        1, NON_NEG, VAL_NUM, VAL_HEX },
+{ "BRIEF",         mupip_integ, 0, 0,                  0, 0, 0, VAL_DISALLOWED, 1, NON_NEG, VAL_N_A, 0       },
+{ "DBG",           mupip_integ, 0, 0,                  0, 0, 0, VAL_DISALLOWED, 2, NON_NEG, VAL_N_A, 0       },
+{ "DMAXBLOCKSIZE", mupip_integ, 0, 0,                  0, 0, 0, VAL_REQ,        1, NON_NEG, VAL_NUM, 0       },
+{ "FAST",          mupip_integ, 0, 0,                  0, 0, 0, VAL_DISALLOWED, 1, NON_NEG, VAL_N_A, 0       },
+{ "FILE",          mupip_integ, 0, 0,                  0, 0, 0, VAL_DISALLOWED, 1, NON_NEG, VAL_N_A, 0       },
+{ "FULL",          mupip_integ, 0, 0,                  0, 0, 0, VAL_DISALLOWED, 1, NON_NEG, VAL_N_A, 0       },
+{ "IMAXBLOCKSIZE", mupip_integ, 0, 0,                  0, 0, 0, VAL_REQ,        1, NON_NEG, VAL_NUM, 0       },
+{ "KEYRANGES",     mupip_integ, 0, 0,                  0, 0, 0, VAL_DISALLOWED, 1, NEG,     VAL_N_A, 0       },
+{ "MAP",           mupip_integ, 0, mup_integ_map_parm, 0, 0, 0, VAL_NOT_REQ,    1, NEG,     VAL_NUM, 0       },
+{ "MAXKEYSIZE",    mupip_integ, 0, mup_integ_map_parm, 0, 0, 0, VAL_NOT_REQ,    1, NEG,     VAL_NUM, 0       },
+{ "ONLINE",        mupip_integ, 0, 0,                  0, 0, 0, VAL_DISALLOWED, 2, NEG,     VAL_N_A, 0       },
+{ "PRESERVE",      mupip_integ, 0, 0,                  0, 0, 0, VAL_NOT_REQ,    1, NON_NEG, VAL_N_A, 0       },
+{ "REGION",        mupip_integ, 0, 0,                  0, 0, 0, VAL_DISALLOWED, 1, NON_NEG, VAL_N_A, 0       },
+{ "STATS",         mupip_integ, 0, 0,                  0, 0, 0, VAL_DISALLOWED, 1, NEG,     VAL_N_A, 0       },
+{ "SUBSCRIPT",     mupip_integ, 0, 0,                  0, 0, 0, VAL_REQ,        1, NON_NEG, VAL_STR, 0       },
+{ "TN_RESET",      mupip_integ, 0, 0,                  0, 0, 0, VAL_DISALLOWED, 1, NON_NEG, VAL_N_A, 0       },
+{ "TRANSACTION",   mupip_integ, 0, mup_integ_map_parm, 0, 0, 0, VAL_NOT_REQ,    1, NEG,     VAL_NUM, 0       },
 { "" }
 };
 
@@ -493,12 +495,15 @@ static readonly CLI_PARM mup_reorg_ff_parm[] = {
 
 /* USER_DEFINED_REORG is currently undocumented */
 static  CLI_ENTRY       mup_reorg_qual[] = {		/* REORG */
+{ "DBG",                mupip_reorg, 0, 0,                 0, 0, 0, VAL_DISALLOWED, 2, NON_NEG, VAL_N_A, 0       },
 { "DOWNGRADE",          mupip_reorg, 0, 0,                 0, 0, 0, VAL_DISALLOWED, 1, NON_NEG, VAL_N_A, 0       },
 { "ENCRYPT",            mupip_reorg, 0, 0,                 0, 0, 0, VAL_REQ,        1, NON_NEG, VAL_STR, 0       },
 { "EXCLUDE",            mupip_reorg, 0, 0,                 0, 0, 0, VAL_REQ,        1, NON_NEG, VAL_STR, 0       },
+{ "FILE",               mupip_reorg, 0, 0,                 0, 0, 0, VAL_REQ,        1, NON_NEG, VAL_N_A, 0       },
 { "FILL_FACTOR",        mupip_reorg, 0, mup_reorg_ff_parm, 0, 0, 0, VAL_REQ,        1, NON_NEG, VAL_NUM, 0       },
 { "INDEX_FILL_FACTOR",  mupip_reorg, 0, mup_reorg_ff_parm, 0, 0, 0, VAL_REQ,        1, NON_NEG, VAL_NUM, 0       },
 { "KEEP",               mupip_reorg, 0, 0,                 0, 0, 0, VAL_NOT_REQ,    1, NON_NEG, VAL_STR, 0       },
+{ "MIN_LEVEL",          mupip_reorg, 0, 0,                 0, 0, 0, VAL_REQ,        1, NON_NEG, VAL_NUM, 0       },
 { "NOCOALESCE",         mupip_reorg, 0, 0,                 0, 0, 0, VAL_DISALLOWED, 1, NON_NEG, VAL_N_A, 0       },
 { "NOSPLIT",            mupip_reorg, 0, 0,                 0, 0, 0, VAL_DISALLOWED, 1, NON_NEG, VAL_N_A, 0       },
 { "NOSWAP",             mupip_reorg, 0, 0,                 0, 0, 0, VAL_DISALLOWED, 1, NON_NEG, VAL_N_A, 0       },
@@ -585,9 +590,11 @@ static CLI_ENTRY	gtmsource_qual[] = {	/* REPLIC -SOURCE */
 {"PASSIVE",              0, 0,                0,                      0, 0,                                  0, VAL_DISALLOWED, 0, NON_NEG, VAL_N_A, 0 },
 {"PLAINTEXTFALLBACK",    0, 0,                0,                      0, 0,                                  0, VAL_DISALLOWED, 1, NEG,     VAL_N_A, 0 },
 {"PROPAGATEPRIMARY",     0, 0,                0,                      0, 0,                                  0, VAL_DISALLOWED, 0, NON_NEG, VAL_N_A, 0 },
+{"RECVBUFFSIZE",         0, 0,                0,                      0, 0,                                  0, VAL_NOT_REQ,    1, NEG,     VAL_NUM, 0 },
 {"RENEGOTIATE_INTERVAL", 0, 0,                0,                      0, 0,                                  0, VAL_REQ,        0, NON_NEG, VAL_STR, 0 },
 {"ROOTPRIMARY",          0, 0,                0,                      0, 0,                                  0, VAL_DISALLOWED, 0, NON_NEG, VAL_N_A, 0 },
 {"SECONDARY",            0, 0,                0,                      0, 0,                                  0, VAL_REQ,        0, NON_NEG, VAL_STR, 0 },
+{"SENDBUFFSIZE",         0, 0,                0,                      0, 0,                                  0, VAL_NOT_REQ,    1, NEG,     VAL_NUM, 0 },
 {"SHOWBACKLOG",          0, 0,                0,                      0, 0,                                  0, VAL_DISALLOWED, 0, NON_NEG, VAL_N_A, 0 },
 {"SHUTDOWN",             0, 0,                0,                      0, 0,                                  0, VAL_DISALLOWED, 0, NON_NEG, VAL_N_A, 0 },
 {"START",                0, 0,                0,                      0, 0,                                  0, VAL_DISALLOWED, 0, NON_NEG, VAL_N_A, 0 },
@@ -625,8 +632,10 @@ static CLI_ENTRY	gtmrecv_qual[] = {	/* REPLIC -RECEIVE */
 {"LOG_INTERVAL",       0, 0, 0,                    0,                     0, 0, VAL_REQ,        0, NON_NEG, VAL_STR, 0 },
 {"NORESYNC",           0, 0, 0,                    0,                     0, 0, VAL_DISALLOWED, 0, NON_NEG, VAL_N_A, 0 },
 {"PLAINTEXTFALLBACK",  0, 0, 0,                    0,                     0, 0, VAL_DISALLOWED, 1, NEG,     VAL_N_A, 0 },
+{"RECVBUFFSIZE",       0, 0, 0,                    0,                     0, 0, VAL_NOT_REQ,    1, NEG,     VAL_NUM, 0 },
 {"RESUME",             0, 0, 0,                    0,                     0, 0, VAL_REQ,        0, NON_NEG, VAL_NUM, 0 },
 {"REUSE",              0, 0, 0,                    0,                     0, 0, VAL_REQ,        0, NON_NEG, VAL_STR, 0 },
+{"SENDBUFFSIZE",       0, 0, 0,                    0,                     0, 0, VAL_NOT_REQ,    1, NEG,     VAL_NUM, 0 },
 {"SHOWBACKLOG",        0, 0, 0,                    0,                     0, 0, VAL_DISALLOWED, 0, NON_NEG, VAL_N_A, 0 },
 {"SHUTDOWN",           0, 0, 0,                    0,                     0, 0, VAL_DISALLOWED, 0, NON_NEG, VAL_N_A, 0 },
 {"START",              0, 0, 0,                    0,                     0, 0, VAL_DISALLOWED, 0, NON_NEG, VAL_N_A, 0 },
@@ -692,7 +701,8 @@ static	CLI_ENTRY	mup_restore_qual[] = {	/* RESTORE */
 };
 
 static	CLI_PARM	mup_rundown_parm[] = {
-{ "WHAT", "File or Region: ", PARM_REQ},
+{ "R_OR_F_NAME", "File or Region: ", PARM_REQ},
+{ "WHAT",   "Dir: ",            PARM_REQ}, /* required when rundown and relinkctl called together*/
 { "", "", PARM_REQ}
 };
 
@@ -728,6 +738,7 @@ static	CLI_ENTRY	mup_set_qual[] = {		/* SET */
 { "ACCESS_METHOD",        mupip_set, 0, 0,                  mup_set_acc_qual,     0, 0, VAL_REQ,        1, NON_NEG, VAL_STR,  0       },
 { "ASYNCIO",              mupip_set, 0, 0,                  0,                    0, 0, VAL_DISALLOWED, 1, NEG,     VAL_N_A,  0       },
 { "BYPASS",               mupip_set, 0, 0,                  0,                    0, 0, VAL_DISALLOWED, 1, NON_NEG, VAL_N_A,  0       },
+{ "DATA_RESERVED_BYTES",  mupip_set, 0, 0,                  0,                    0, 0, VAL_REQ,        1, NON_NEG, VAL_NUM,  0       },
 { "DBFILENAME",           mupip_set, 0, 0,                  0,                    0, 0, VAL_REQ,        1, NEG,     VAL_STR,  0       },
 { "DEFER_ALLOCATE",       mupip_set, 0, 0,                  0,                    0, 0, VAL_DISALLOWED, 1, NEG,     VAL_N_A,  0       },
 { "DEFER_TIME",           mupip_set, 0, 0,                  0,                    0, 0, VAL_REQ,        1, NEG,     VAL_STR,  0       },
@@ -740,6 +751,7 @@ static	CLI_ENTRY	mup_set_qual[] = {		/* SET */
 { "FULLBLKWRT",           mupip_set, 0, 0,                  0,                    0, 0, VAL_REQ,        1, NON_NEG, VAL_NUM,  0       },
 { "GLOBAL_BUFFERS",       mupip_set, 0, 0,                  0,                    0, 0, VAL_REQ,        1, NON_NEG, VAL_NUM,  0       },
 { "HARD_SPIN_COUNT",      mupip_set, 0, 0,                  0,                    0, 0, VAL_REQ,        1, NON_NEG, VAL_NUM,  VAL_DCM },
+{ "INDEX_RESERVED_BYTES", mupip_set, 0, 0,                  0,                    0, 0, VAL_REQ,        1, NON_NEG, VAL_NUM,  0	      },
 { "INST_FREEZE_ON_ERROR", mupip_set, 0, 0,                  0,                    0, 0, VAL_DISALLOWED, 1, NEG,     VAL_N_A,  0       },
 { "JNLFILE",              mupip_set, 0, 0,                  0,                    0, 0, VAL_DISALLOWED, 1, NON_NEG, VAL_N_A,  0       },
 { "JOURNAL",              mupip_set, 0, 0,                  mup_set_journal_qual, 0, 0, VAL_NOT_REQ,    1, NEG,     VAL_STR,  0       },
@@ -806,14 +818,15 @@ static	CLI_PARM	mup_stop_parm[] = {
 };
 
 static	CLI_ENTRY	mup_upgrade_qual[] = {		/* UPGRADE */
-{ "MASTERMAP", mupip_upgrade, 0, 0, 0, 0, 0, VAL_DISALLOWED, 1, NON_NEG, VAL_N_A, 0},
-{ "REGION",    mupip_upgrade, 0, 0, 0, 0, 0, VAL_DISALLOWED, 1, NON_NEG, VAL_N_A, 0},
+{ "DBG",    mupip_upgrade, 0, 0, 0, 0, 0, VAL_DISALLOWED, 2, NON_NEG, VAL_N_A, 0 },
+{ "FILE",   mupip_upgrade, 0, 0, 0, 0, 0, VAL_DISALLOWED, 1, NON_NEG, VAL_N_A, 0 },
+{ "REGION", mupip_upgrade, 0, 0, 0, 0, 0, VAL_DISALLOWED, 1, NON_NEG, VAL_N_A, 0 },
 { "" }
-};	/* TODO: revisit file vs region vs choice? 1 function or two? names? */
+};
 
 static	CLI_PARM	mup_upgrade_parm[] = {
-{ "REGION", "Region: ", PARM_REQ},
-{ "", "",           PARM_REQ}
+{"WHAT", "File or Region: ", PARM_REQ},
+{ "",	"",		     PARM_REQ}
 };
 
 static	CLI_PARM	mup_downgrade_parm[] = {
@@ -855,7 +868,7 @@ GBLDEF	CLI_ENTRY	mupip_cmd_ary[] = {
 { "REORG",     mupip_reorg,      mup_reorg_qual,     mup_reorg_parm,     0, cli_disallow_mupip_reorg,     0, VAL_DISALLOWED, 1,         0, 0, 0 },
 { "REPLICATE", 0,                mup_replicate_qual, mup_replicate_parm, 0, cli_disallow_mupip_replicate, 0, VAL_DISALLOWED, 1,         0, 0, 0 },
 { "RESTORE",   mupip_restore,    mup_restore_qual,   mup_restore_parm,   0, 0,                            0, VAL_DISALLOWED, 2,         0, 0, 0 },
-{ "RUNDOWN",   mupip_rundown,    mup_rundown_qual,   mup_rundown_parm,   0, cli_disallow_mupip_rundown,   0, VAL_DISALLOWED, 1,         0, 0, 0 },
+{ "RUNDOWN",   mupip_rundown,    mup_rundown_qual,   mup_rundown_parm,   0, cli_disallow_mupip_rundown,   0, VAL_DISALLOWED, 2,         0, 0, 0 },
 { "SEMAPHORE", mupip_sems,       0,                  mup_sems_parm,      0, 0,                            0, VAL_DISALLOWED, MAX_PARMS, 0, 0, 0 },
 { "SET",       mupip_set,        mup_set_qual,       mup_set_parm,       0, cli_disallow_mupip_set,       0, VAL_DISALLOWED, 1,         0, 0, 0 },
 { "SIZE",      mupip_size,       mup_size_qual,      0,                  0, cli_disallow_mupip_size,      0, VAL_DISALLOWED, 1,         0, 0, 0 },
