@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2021 Fidelity National Information	*
+ * Copyright (c) 2001-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -19,7 +19,7 @@
 #include "gtm_inet.h"
 #include "gtm_stdio.h"
 #include "gtm_string.h"
-#include "gtm_select.h"
+#include "gtm_poll.h"
 
 #include <sys/wait.h>
 #include <stddef.h>
@@ -64,7 +64,6 @@
 #include "gtmmsg.h"
 #include "wcs_sleep.h"
 #include "db_ipcs_reset.h"
-#include "gds_blk_downgrade.h"
 #include "shmpool.h"
 #include "min_max.h"
 #include "gtmxc_types.h"
@@ -138,7 +137,7 @@ void mupip_restore(void)
 	backup_type		type;
 	unsigned short		port;
 	int4			timeout, cut, match;
-	void			(*common_read)();
+	void			(*common_read)(BFILE *, char *, int4);
 	char			*errptr;
 	pid_t			waitpid_res;
 	muinc_blk_hdr_ptr_t	sblkh_p;
@@ -147,7 +146,7 @@ void mupip_restore(void)
 	int			in_len, gtmcrypt_errno;
 	boolean_t		same_encr_settings;
 	boolean_t		check_mdb_ver, bad_mdb_ver;
-	boolean_t		in_is_encrypted, in_to_be_encrypted;
+	boolean_t		in_is_encrypted = FALSE, in_to_be_encrypted = FALSE;
 	boolean_t		old_is_encrypted, old_to_be_encrypted;
 	boolean_t		in_use_new_key;
 	enc_handles		in_encr_handles, old_encr_handles;
@@ -316,7 +315,7 @@ void mupip_restore(void)
 				util_out_print("Unrecognized input format !AD", TRUE, ptr->input_file.len, ptr->input_file.addr);
 				CLNUP_AND_EXIT(ERR_MUPRESTERR, inbuf);
 		}
-		COMMON_READ(in, &inhead, SIZEOF(inc_header), inbuf);
+		COMMON_READ(in, (char *)&inhead, SIZEOF(inc_header), inbuf);
 		check_mdb_ver = FALSE;
 		if (!memcmp(inhead.label, INC_HEADER_LABEL_V5_NOENCR, INC_HDR_LABEL_SZ))
 			assert(!IS_ENCRYPTED(inhead.is_encrypted));
@@ -389,7 +388,10 @@ void mupip_restore(void)
 			old_is_encrypted = IS_ENCRYPTED(old_data.is_encrypted);
 			old_to_be_encrypted = USES_NEW_KEY(&old_data);
 		} else
+		{
 			same_encr_settings = TRUE;
+			old_is_encrypted = old_to_be_encrypted = FALSE;
+		}
 		if (curr_tn != inhead.start_tn)
 		{
 			util_out_print("Transaction in input file !AD does not align with database TN.!/DB: !16@XQ!_"
@@ -514,7 +516,7 @@ void mupip_restore(void)
 			COMMON_READ(in, inbuf, rsize, inbuf);	/* Note rsize == sblkh_p */
 			if (0 == sblkh_p->blkid && FALSE == sblkh_p->valid_data)
 			{	/* This is supposed to be the end of list marker (null entry */
-				COMMON_READ(in, &rsize, SIZEOF(rsize), inbuf);
+				COMMON_READ(in, (char *)&rsize, SIZEOF(rsize), inbuf);
 				if (SIZEOF(END_MSG) + SIZEOF(int4) == rsize)
 				{	/* the length of our secondary check is correct .. now check substance */
 					COMMON_READ(in, inbuf, rsize - SIZEOF(int4), inbuf);
@@ -547,19 +549,9 @@ void mupip_restore(void)
 			*/
 			blk_ptr = inbuf + SIZEOF(muinc_blk_hdr);
 			size = old_blk_size;
-			if (GDSNOVER != sblkh_p->use.bkup.ondsk_blkver)
-			{	/* Specifically versioned blocks - Put them back in the version they were originally */
-				if (GDSV4 == sblkh_p->use.bkup.ondsk_blkver)
-				{
-					gds_blk_downgrade((v15_blk_hdr_ptr_t)blk_ptr, (blk_hdr_ptr_t)blk_ptr);
-					size = (((v15_blk_hdr_ptr_t)blk_ptr)->bsiz + 1) & ~1;
-					assert((size <= old_blk_size) && (size >= SIZEOF(v15_blk_hdr)));
-				} else
-				{
-					size = (((blk_hdr_ptr_t)blk_ptr)->bsiz + 1) & ~1;
-					assert((size <= old_blk_size) && (size >= SIZEOF(blk_hdr)));
-				}
-			}
+			/* Previously, blocks could be downgraded here as needed */
+			size = (((blk_hdr_ptr_t)blk_ptr)->bsiz + 1) & ~1;
+			assert((size <= old_blk_size) && (size >= SIZEOF(blk_hdr)));
 			in_len = MIN(old_blk_size, size) - SIZEOF(blk_hdr);
 			if (!same_encr_settings && IS_BLK_ENCRYPTED(((blk_hdr_ptr_t)blk_ptr)->levl, in_len))
 			{
@@ -624,7 +616,7 @@ void mupip_restore(void)
 			}
 		}
 		/* Next section is the file header which we need to restore */
-		COMMON_READ(in, &rsize, SIZEOF(rsize), inbuf);
+		COMMON_READ(in, (char *)&rsize, SIZEOF(rsize), inbuf);
 		assert((SGMNT_HDR_LEN + SIZEOF(int4)) == rsize);
 		COMMON_READ(in, inbuf, rsize, inbuf);
 		if (0 == memcmp(inbuf, V6_GDS_LABEL, GDS_LABEL_SZ - 1))
@@ -769,26 +761,22 @@ STATICFNDEF void tcp_read(BFILE *bf, char *buf, int nbytes)
 {
 	int     	needed, status;
 	char		*curr;
-	fd_set          fs;
-	struct timeval	save_nap, nap;
 	int		rc;
+	int		poll_timeout, save_poll_timeout;
+	nfds_t		poll_nfds;
+	struct pollfd	poll_fdlist[1];
 
 	needed = nbytes;
 	curr = buf;
-	nap.tv_sec = 1;
-	nap.tv_usec = 0;
+	poll_timeout = MILLISECS_IN_SEC;
 	while (1)
 	{
-		assertpro(FD_SETSIZE > bf->fd);
-		FD_ZERO(&fs);
-		FD_SET(bf->fd, &fs);
-		assert(0 != FD_ISSET(bf->fd, &fs));
-		/* Note: the check for EINTR from the select below should remain, as aa_select is a
-		 * function, and not all callers of aa_select behave the same when EINTR is returned.
-		 */
-		save_nap = nap;
-		status = select(bf->fd + 1, (void *)(&fs), (void *)0, (void *)0, &nap);
-		nap = save_nap;
+		poll_fdlist[0].fd = bf->fd;
+		poll_fdlist[0].events = POLLIN;
+		poll_nfds = 1;
+		save_poll_timeout = poll_timeout;
+		status = poll(&poll_fdlist[0], poll_nfds, poll_timeout);
+		poll_timeout = save_poll_timeout;
 		if (status > 0)
 		{
 			RECV(bf->fd, curr, needed, 0, status);
