@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2012-2022 Fidelity National Information	*
+ * Copyright (c) 2012-2023 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -381,7 +381,11 @@ enum cdb_sc mu_size_rand_traverse(double *r, double *a)
 				valBlk[musz_rec] = nBlkId;
 				musz_rec++;
 			} else
-				valBlk[rCnt] = nBlkId;
+			{
+				assert(MAX_RECS_PER_BLK > rCnt);
+				if (MAX_RECS_PER_BLK > rCnt)  /* Our loop macro guarantes rCnt is OK, but SCA doesn't know that */
+					valBlk[rCnt] = nBlkId;
+			}
 		}
 		(mu_subsc) ? (r[nLevl] = musz_rec) : (r[nLevl] = rCnt);
 		/* randomly select next block */
