@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2013-2021 Fidelity National Information	*
+ * Copyright (c) 2013-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -438,16 +438,16 @@ void	op_fnzpeek(mval *structid, int offset, int len, mval *format, mval *ret)
 	struct sigaction	new_action, prev_action_bus, prev_action_segv;
 	sigset_t		savemask;
 	int			errtoraise, rc, rslt;
-	char			fmtcode;
+	unsigned char		fmtcode;
 	boolean_t		arg_supplied, attach_success;
 	unsigned char		mnemonic[NAME_ENTRY_SZ], *nptr, *cptr, *cptrend, *argptr;
-	int			mnemonic_len, mnemonic_index, mnemonic_opcode, arglen, arryidx;
-	gd_region		*r_top, *r_ptr;
+	int			mnemonic_len, mnemonic_index, mnemonic_opcode, arglen, arryidx = -1;
+	gd_region		*r_top, *r_ptr = NULL;
 	replpool_identifier	replpool_id;
 	unsigned int		full_len;
 	unsigned char		argument_uc_buf[ARGUMENT_MAX_LEN];
 	sgmnt_addrs		*csa;
-	int			pass;
+	int			iter;
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
@@ -495,10 +495,10 @@ void	op_fnzpeek(mval *structid, int offset, int len, mval *format, mval *ret)
 			 * so first check if region as is does exist. If so use that. If not, do uppercase.
 			 */
 			assert(arglen);
-			pass = (ISLOWER_ASCII(argptr[0]) ? PASS1 : PASS2);
+			iter = (ISLOWER_ASCII(argptr[0]) ? PASS1 : PASS2);
 			for ( ; ; )
 			{
-				if (PASS1 != pass)
+				if (PASS1 != iter)
 				{
 					lower_to_upper(argument_uc_buf, argptr, arglen);
 					argptr = argument_uc_buf;	/* Reset now to point to upper case version */
@@ -517,7 +517,7 @@ void	op_fnzpeek(mval *structid, int offset, int len, mval *format, mval *ret)
 				{
 					if (r_ptr >= r_top)
 					{
-						if (PASS1 == pass)
+						if (PASS1 == iter)
 							break;
 						RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(4) ERR_BADZPEEKARG, 2,
 							RTS_ERROR_LITERAL("mnemonic argument (region name)"));
@@ -533,8 +533,8 @@ void	op_fnzpeek(mval *structid, int offset, int len, mval *format, mval *ret)
 					 * region name. In that case, it is possible we find the region in the gld only when we
 					 * convert the entire region name into upper case.
 					 */
-					assert(PASS1 == pass);
-					pass = PASS2;
+					assert(PASS1 == iter);
+					iter = PASS2;
 					continue;
 				}
 				/* Cache new region access for followup references */
@@ -547,6 +547,7 @@ void	op_fnzpeek(mval *structid, int offset, int len, mval *format, mval *ret)
 			 * All the rest need it to be open. If there are any errors in the open (e.g. statsdb specified
 			 * and gtm_statsdir env var is too long etc.) then handle it by issuing an error.
 			 */
+			assert((PO_GDRREG == mnemonic_opcode) || r_ptr);
 			if ((PO_GDRREG != mnemonic_opcode) && !r_ptr->open)
 			{
 				gv_init_reg(r_ptr, NULL);
@@ -563,7 +564,7 @@ void	op_fnzpeek(mval *structid, int offset, int len, mval *format, mval *ret)
 					RTS_ERROR_LITERAL("mnemonic argument (array index)"));
 			break;
 		case PO_PEEK:			/* Argument is address of form 0Xhhhhhhhh[hhhhhhhh] */
-			if (('0' != *cptr++) || ('x' != *cptr) && ('X' != *cptr))
+			if (('0' != *cptr++) || (('x' != *cptr) && ('X' != *cptr)))
 				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(4) ERR_BADZPEEKARG, 2,
 					RTS_ERROR_LITERAL("mnemonic argument (peek base address)"));
 			cptr++;			/* Bump past 'x' or 'X' - rest of arg should be hex value */
@@ -583,20 +584,25 @@ void	op_fnzpeek(mval *structid, int offset, int len, mval *format, mval *ret)
 	switch(mnemonic_opcode)
 	{
 		case PO_CSAREG:		/* r_ptr set from option processing */
+			assert(r_ptr);
 			zpeekadr = &FILE_INFO(r_ptr)->s_addrs;
 			break;
 		case PO_FHREG:		/* r_ptr set from option processing */
+			assert(r_ptr);
 			zpeekadr = (&FILE_INFO(r_ptr)->s_addrs)->hdr;
 			break;
 		case PO_GDRREG:		/* r_ptr set from option processing */
 			assert(arg_supplied);	/* 4SCA: Assigned value is garbage or undefined, even though args are required */
+			assert(r_ptr);
 			zpeekadr = r_ptr;
 			break;
 		case PO_NLREG:		/* r_ptr set from option processing */
+			assert(r_ptr);
 			zpeekadr = (&FILE_INFO(r_ptr)->s_addrs)->nl;
 			break;
 		case PO_JNLREG:		/* r_ptr set from option processing */
 		case PO_JBFREG:
+			assert(r_ptr);
 			csa = &FILE_INFO(r_ptr)->s_addrs;
 			if (NULL == csa->jnl)
 				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(4) ERR_ZPEEKNOJNLINFO, 2, REG_LEN_STR(r_ptr));
@@ -608,8 +614,11 @@ void	op_fnzpeek(mval *structid, int offset, int len, mval *format, mval *ret)
 		case PO_NLREPL:
 		case PO_RIHREPL:
 			/* Make sure jnlpool_addrs are availble */
-			if (!REPL_INST_AVAILABLE(NULL))
+			if (!REPL_INST_AVAILABLE(jnlpool && jnlpool->pool_init ? jnlpool->gd_ptr : NULL))
+			{
+				assert(gd_header);
 				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_ZPEEKNORPLINFO);
+			}
 			if (!pool_init)
 			{
 				attach_success = op_fnzpeek_attach_jnlpool();
@@ -619,9 +628,11 @@ void	op_fnzpeek(mval *structid, int offset, int len, mval *format, mval *ret)
 			switch(mnemonic_opcode)
 			{
 				case PO_GLFREPL:	/* arryidx set by option processing */
+					assert(-1 != arryidx);
 					zpeekadr = (jnlpool->gtmsrc_lcl_array + arryidx);
 					break;
 				case PO_GSLREPL:	/* arryidx set by option processing */
+					assert(-1 != arryidx);
 					zpeekadr = (jnlpool->gtmsource_local_array + arryidx);
 					break;
 				case PO_NLREPL:
@@ -635,6 +646,7 @@ void	op_fnzpeek(mval *structid, int offset, int len, mval *format, mval *ret)
 					break;
 				default:
 					assert(FALSE);
+					GTM_UNREACHABLE();
 			}
 			break;
 		case PO_RPCREPL:	/* This set of opcodes all require the receive pool to be initialized. Verify it */
@@ -642,7 +654,7 @@ void	op_fnzpeek(mval *structid, int offset, int len, mval *format, mval *ret)
 		case PO_UPLREPL:
 		case PO_UHCREPL:
 			/* Make sure recvpool_addrs are available */
-			if (!REPL_INST_AVAILABLE(NULL))
+			if (!REPL_INST_AVAILABLE(jnlpool && jnlpool->pool_init ? jnlpool->gd_ptr : NULL))
 				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_ZPEEKNORPLINFO);
 			if (NULL == recvpool.recvpool_ctl)
 			{
@@ -666,6 +678,7 @@ void	op_fnzpeek(mval *structid, int offset, int len, mval *format, mval *ret)
 					break;
 				default:
 					assert(FALSE);
+					GTM_UNREACHABLE();
 			}
 			break;
 		case PO_PEEK:		/* prmpeekadr set up in argument processing */
@@ -674,6 +687,7 @@ void	op_fnzpeek(mval *structid, int offset, int len, mval *format, mval *ret)
 			break;
 		default:
 			assert(FALSE);
+			GTM_UNREACHABLE();
 	}
 	assert(NULL != zpeekadr);
 	/* Check the rest of the args */
@@ -687,7 +701,7 @@ void	op_fnzpeek(mval *structid, int offset, int len, mval *format, mval *ret)
 	else if (1 == format->str.len)
 	{	/* Validate format option */
 		fmtcode = *format->str.addr;
-		fmtcode = lower_to_upper_table[fmtcode];
+		fmtcode = TOUPPER(fmtcode);
 		switch(fmtcode)
 		{
 			case 'C':	/* Character data - returned as is */
