@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2021 Fidelity National Information	*
+ * Copyright (c) 2001-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -30,29 +30,11 @@
 #include "deferred_events_queue.h"
 #include "op.h"
 #include "io.h"
+#include "iott_setterm.h"
 #include "mvalconv.h"
-
-#include "gdsroot.h"
-#include "gdskill.h"
-#include "gdsbt.h"
-#include "gtm_facility.h"
-#include "fileinfo.h"
-#include "gdsfhead.h"
-#include "gdscc.h"
-#include "filestruct.h"
-#include "buddy_list.h"		/* needed for tp.h */
-#include "jnl.h"
-#include "tp.h"
-#include "send_msg.h"
-#include "gtmmsg.h"		/* for gtm_putmsg() prototype */
-#include "change_reg.h"
-#include "setterm.h"
-#include "getzposition.h"
+#include "tpnotacid_chk_inline.h"
 #include "iosocketdef.h"
 #include "min_max.h"
-#ifdef DEBUG
-#include "have_crit.h"		/* for the TPNOTACID_CHECK macro */
-#endif
 
 void	job_timer_handler(void);
 
@@ -162,7 +144,7 @@ int	op_job(int4 argcnt, ...)
 	}
 	/* Start the timer */
 	ojtimeout = timed = FALSE;
-	MV_FORCE_MSTIMEOUT(timeout, msec_timeout, JOBTIMESTR);
+	MV_FORCE_MSTIMEOUT(timeout, &msec_timeout, JOBTIMESTR);
 	if ((0 < msec_timeout) && (NO_M_TIMEOUT != msec_timeout))
 	{
 		timed = TRUE;
@@ -199,7 +181,7 @@ int	op_job(int4 argcnt, ...)
 		cancel_timer((TID)&tid);
 	if (argcnt)
 		free(job_params.parms);
-	if (job_params.passcurlvn)
+	if (job_params.xfercurlvn)
 		system_free(job_params.curlvn_buffer_ptr);		/* Space allocated by open_memstream in ojparams */
 	if (!non_exit_return)
 	{
@@ -235,7 +217,8 @@ int	op_job(int4 argcnt, ...)
 						errno);
 			}
 		}
-	}
+	} else
+		exit_stat = 0;
 	assert(SIZEOF(pid_t) == SIZEOF(zjob_pid));
 	DOREADRC(pipe_fds[0], &zjob_pid, SIZEOF(zjob_pid), pipe_status);
 	/* empty pipe (pipe_status == -1) is ignored and not reported as error */
