@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2014-2022 Fidelity National Information	*
+ * Copyright (c) 2014-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -55,9 +55,9 @@
 /* Constants defining how many times to retry the loop in relinkctl_open() based on the specific error conditions encountered. */
 #define MAX_RCTL_INIT_WAIT_RETRIES	1000	/* # of sleeps to allow while waiting for the shared memory to be initialized. */
 #define MAX_RCTL_DELETED_RETRIES	16	/* # of times to allow an existing relinkctl file to be deleted before open(). */
-#define MAX_RCTL_RUNDOWN_RETRIES	16	/* # of times to allow a mapped relinkctl file to get run down before shmat(). */
 
 DEBUG_ONLY(GBLDEF int	saved_errno;)
+GBLDEF	int		umask_orig;
 GBLREF	uint4		process_id;
 GBLREF	rtn_tabent	*rtn_names, *rtn_names_end;
 GBLREF	stack_frame	*frame_pointer;
@@ -77,11 +77,19 @@ error_def(ERR_FILEPARSE);
 error_def(ERR_RELINKCTLERR);
 error_def(ERR_RELINKCTLFULL);
 error_def(ERR_REQRLNKCTLRNDWN);
+error_def(ERR_RLNKCTLOPENDEL);
 error_def(ERR_RLNKCTLRNDWNFL);
 error_def(ERR_RLNKCTLRNDWNSUC);
 error_def(ERR_SYSCALL);
 error_def(ERR_TEXT);
 
+CONDITION_HANDLER(relinkctl_handler)
+{
+	START_CH(TRUE);
+	(void)umask(umask_orig);	/* reset umask */
+	NEXTCH;
+}
+
 /* Routine called to see if a relinkctl structure already exists for the given zroutines element.
  *
  * Parameters:
@@ -145,9 +153,9 @@ open_relinkctl_sgm *relinkctl_attach(mstr *obj_container_name, mstr *objpath, in
 		 */
 		if (!obj_dir_found)
 			obj_container_name->len = 0;
-		else if (objdir.len <= objpath_alloc_len)
-		{
-			memcpy(obj_container_name->addr, objdir.addr, objdir.len);
+		else if (objdir.len < objpath_alloc_len)
+		{	/* len+1 because of an assert null check in mu_rndwn_rlnkctl() */
+			memcpy(obj_container_name->addr, objdir.addr, (objdir.len + 1));
 			obj_container_name->len = objdir.len;
 		}
 	}
@@ -184,12 +192,18 @@ open_relinkctl_sgm *relinkctl_attach(mstr *obj_container_name, mstr *objpath, in
 	 * It is also possible to get a non-zero return status if the caller is MUPIP RUNDOWN -RELINKCTL and the relinkctl file does
 	 * not exist and therefore does not need to be run down.
 	 */
+	umask_orig = umask(000);	/* clear umask */
+	ESTABLISH_RET(relinkctl_handler,0);
 	if (0 != relinkctl_open(&new_link, !obj_dir_found))
 	{
 		if (!obj_dir_found)
 			errno = save_errno;
+		REVERT;
+		(void)umask(umask_orig);	/* reset umask */
 		return NULL;
 	}
+	REVERT;
+	(void)umask(umask_orig);	/* reset umask */
 	/* No errors were raised so far, so copy the segment information into a malloced space. */
 	new_link_ptr = malloc(SIZEOF(open_relinkctl_sgm) + objdir.len + 1 + len); /* + 1 for trailing null in zro_entry_name */
 	memcpy(new_link_ptr, &new_link, SIZEOF(open_relinkctl_sgm));
@@ -258,6 +272,7 @@ int relinkctl_open(open_relinkctl_sgm *linkctl, boolean_t object_dir_missing)
 			 	 */
 				SNPRINTF(errstr, SIZEOF(errstr), "stat() of file %s failed", linkctl->relinkctl_path);
 				ISSUE_RELINKCTLERR_SYSCALL(&linkctl->zro_entry_name, errstr, errno);
+				rctl_create_attempted = FALSE;
 			} else
 			{	/* If the relinkctl file is missing, then we are not going to create one if either the object
 				 * directory does not exist or we are a MUPIP RUNDOWN -RELINKCTL process.
@@ -354,7 +369,8 @@ int relinkctl_open(open_relinkctl_sgm *linkctl, boolean_t object_dir_missing)
 			relinkctl_unlock_exclu(linkctl);
 			relinkctl_unmap(linkctl);
 			assert(NULL == linkctl->hdr);
-			assertpro(MAX_RCTL_RUNDOWN_RETRIES > rctl_rundown_count++); /* Too many loops should not be possible. */
+			send_msg_csa(NULL, VARLSTCNT(6) ERR_RLNKCTLOPENDEL, 4,linkctl->relinkctl_path,
+					RTS_ERROR_MSTR(&linkctl->zro_entry_name), ++rctl_rundown_count);
 			continue;
 		}
 		if (0 == hdr->relinkctl_max_rtn_entries)
@@ -418,7 +434,7 @@ int relinkctl_open(open_relinkctl_sgm *linkctl, boolean_t object_dir_missing)
 			DBGARLNK((stderr, "relinkctl_open: file first open\n"));
 			hdr->n_records = 0;
 			/* Create shared memory to store hash buckets of routine names for faster search in relinkctl file */
-			shmid = gtm_shmget(IPC_PRIVATE, shm_size, RWDALL | IPC_CREAT, TRUE);
+			shmid = gtm_shmget(IPC_PRIVATE, shm_size, RWDALL | IPC_CREAT, TRUE, RELINK, linkctl->relinkctl_path);
 			if (-1 == shmid)
 			{
 				save_errno = errno;
@@ -827,6 +843,7 @@ relinkrec_t *relinkctl_insert_record(open_relinkctl_sgm *linkctl, mstr *rtnname)
 			/* assert(valid_mname(rtnname)) is not needed here because it was already done in relinkctl_find_record */
 			memcpy(&rec->rtnname_fixed.c[0], rtnname->addr, rtnname->len);
 			rec->cycle = 0;	/* Note incr_link() will bump this to 1 when routine is linked */
+			rec->rtnobj_supersede = 0;
 			rec->hashindex_fl = 0;
 			rec->objhash = 0;
 			rec->rtnobj_shm_offset = (rtnobj_sm_off_t)NULL_RTNOBJ_SM_OFF_T;
@@ -863,7 +880,7 @@ void relinkctl_rundown(boolean_t decr_attached, boolean_t do_rtnobj_shm_free)
 	rhdtyp			*rtnhdr;
 	relinkctl_data		*hdr;
 	int			shmid, i, nattached;
-	relinkshm_hdr_t		*shm_hdr;
+	relinkshm_hdr_t		*shm_hdr = NULL ;
 	rtnobjshm_hdr_t		*rtnobj_shm_hdr;
 	stack_frame		*fp;
 	boolean_t		is_mu_rndwn_rlnkctl, remove_shm, remove_rctl;
@@ -961,6 +978,7 @@ void relinkctl_rundown(boolean_t decr_attached, boolean_t do_rtnobj_shm_free)
 						nattached = shm_buf.shm_nattch - 1; /* remove self since we will do a SHMDT soon */
 						if (hdr->nattached != nattached)
 						{
+							assert(shm_hdr);
 							hdr->nattached = nattached;	/* fix hdr->nattached while at this */
 							shm_hdr->rndwn_adjusted_nattch = TRUE;
 						}
@@ -985,6 +1003,7 @@ void relinkctl_rundown(boolean_t decr_attached, boolean_t do_rtnobj_shm_free)
 			}
 			if (remove_shm)
 			{
+				assert(shm_hdr);
 #				ifdef DEBUG
 				clean_shutdown = (!shm_hdr->rndwn_adjusted_nattch && !shm_hdr->skip_rundown_check);
 				if (clean_shutdown)
