@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2006-2017 Fidelity National Information	*
+ * Copyright (c) 2006-2023 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -116,7 +116,8 @@ GBLREF	uint4		section_offset;		/* Used by PRINT_OFFSET_PREFIX macro in repl_inst
 
 void	repl_inst_dump_filehdr(repl_inst_hdr_ptr_t repl_instance)
 {
-	char		dststr[MAX_DIGITS_IN_INT], dstlen;
+	char		dststr[MAX_DIGITS_IN_INT];
+	unsigned char	dstlen;
 	char		*string;
 	int4		minorver, nodename_len, last_histinfo_num;
 	int		idx, strm_idx, offset;
@@ -443,8 +444,8 @@ void	repl_inst_dump_history_records(char *inst_fn, int4 num_histinfo)
 			curhistinfo.history_type, curhistinfo.history_type);
 
 		/* Assert that lms_group is filled in for non-zero stream #s and not for zero stream #s */
-		assert(!curhistinfo.strm_index && IS_REPL_INST_UUID_NULL(curhistinfo.lms_group)
-			|| curhistinfo.strm_index && IS_REPL_INST_UUID_NON_NULL(curhistinfo.lms_group));
+		assert((!curhistinfo.strm_index && IS_REPL_INST_UUID_NULL(curhistinfo.lms_group))
+			|| (curhistinfo.strm_index && IS_REPL_INST_UUID_NON_NULL(curhistinfo.lms_group)));
 		/* Assert that UPDATERESYNC type of history record is possible only in non-zero stream #s */
 		assert((HISTINFO_TYPE_UPDRESYNC != curhistinfo.history_type) || curhistinfo.strm_index);
 		/* Do not print "lms_group" info for all history records as it will clutter the output.
@@ -510,8 +511,8 @@ void	repl_dump_histinfo(FILE *log_fp, boolean_t stamptime, boolean_t flush, char
 		cur_histinfo->created_time, cur_histinfo->created_time, cur_histinfo->histinfo_num,
 		cur_histinfo->prev_histinfo_num, cur_histinfo->strm_index, cur_histinfo->history_type);
 	/* Assert that lms_group is filled in for non-zero stream #s and not for zero stream #s */
-	assert(!cur_histinfo->strm_index && IS_REPL_INST_UUID_NULL(cur_histinfo->lms_group)
-		|| cur_histinfo->strm_index && IS_REPL_INST_UUID_NON_NULL(cur_histinfo->lms_group));
+	assert((!cur_histinfo->strm_index && IS_REPL_INST_UUID_NULL(cur_histinfo->lms_group))
+		|| (cur_histinfo->strm_index && IS_REPL_INST_UUID_NON_NULL(cur_histinfo->lms_group)));
 	/* Assert that UPDATERESYNC type of history record is possible only in non-zero stream #s */
 	assert((HISTINFO_TYPE_UPDRESYNC != cur_histinfo->history_type) || cur_histinfo->strm_index);
 	/* Do not print "lms_group" info for all history records as it will clutter the output.
@@ -849,9 +850,9 @@ void	repl_inst_dump_gtmsourcelocal(gtmsource_local_ptr_t gtmsourcelocal_ptr)
 				string, gtmsourcelocal_ptr->read_state);
 		}
 
-		PRINT_OFFSET_PREFIX(offsetof(gtmsource_local_struct, jnlfileonly), SIZEOF(gtmsourcelocal_ptr->jnlfileonly));
+		PRINT_OFFSET_PREFIX(offsetof(gtmsource_local_struct, read_algo), SIZEOF(gtmsourcelocal_ptr->read_algo));
 		PRINT_BOOLEAN(PREFIX_SOURCELOCAL "Journal File Only                   !R12AZ",
-				gtmsourcelocal_ptr->jnlfileonly, idx);
+				gtmsourcelocal_ptr->read_algo, idx);
 
 		PRINT_OFFSET_PREFIX(offsetof(gtmsource_local_struct, read), SIZEOF(gtmsourcelocal_ptr->read));
 		util_out_print( PREFIX_SOURCELOCAL "Relative Read Offset                  !10UL [0x!XL]", TRUE, idx,
