@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2013 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2024 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -12,6 +13,7 @@
 #include "mdef.h"
 
 #include "gtm_inet.h"
+#include "gtm_ipc.h"
 
 #include <sys/shm.h>
 #include <errno.h>
@@ -38,16 +40,18 @@
 #include "repl_instance.h"
 #include "wbox_test_init.h"
 #include "have_crit.h"
-#include "gtm_ipc.h"
+#include "anticipatory_freeze.h"
+#include "jnl.h"
+#include "wcs_sleep.h"
 
-GBLREF	jnlpool_addrs		jnlpool;
-GBLREF	boolean_t		pool_init;
-GBLREF	jnlpool_ctl_ptr_t	jnlpool_ctl;
-GBLREF	int			gtmsource_srv_count;
+GBLREF	jnlpool_addrs_ptr_t	jnlpool;
+GBLREF	int			pool_init;
 GBLREF	int			gtmrecv_srv_count;
-
 GBLREF	recvpool_addrs		recvpool;
 GBLREF	int			recvpool_shmid;
+#ifdef DEBUG
+GBLREF	bool			only_usr_jnlpool_flush;
+#endif
 
 int	gtmsource_ipc_cleanup(boolean_t auto_shutdown, int *exit_status, int4 *num_src_servers_running)
 {
@@ -55,7 +59,11 @@ int	gtmsource_ipc_cleanup(boolean_t auto_shutdown, int *exit_status, int4 *num_s
 	int		status, detach_status, remove_status, semval, save_errno;
 	unix_db_info	*udi;
 	struct shmid_ds	shm_buf;
+	int sleep_counter;
+	int n_attached = 0;
+	DCL_THREADGBL_ACCESS;
 
+	SETUP_THREADGBL_ACCESS;
 	/* Attempt cleaning up the IPCs */
 	attempt_ipc_cleanup = TRUE;
 	*num_src_servers_running = 0;
@@ -79,45 +87,73 @@ int	gtmsource_ipc_cleanup(boolean_t auto_shutdown, int *exit_status, int4 *num_s
 			*exit_status = ABNORMAL_SHUTDOWN;
 		}
 	}
-	udi = (unix_db_info *)FILE_INFO(jnlpool.jnlpool_dummy_reg);
+	/* If the instance is frozen, there is no point checking if we are the last user, since we need to keep the journal pool
+	 * around, and we will need the journal pool attached so that we can check for instance freeze in database rundown.
+	 * In that case, the detach will happen automatically when the process terminates.
+	 */
+	assert(NULL != jnlpool);
+	if (IS_REPL_INST_FROZEN(RECOGNIZE_FREEZES))
+		return FALSE;
+	udi = (unix_db_info *)FILE_INFO(jnlpool->jnlpool_dummy_reg);
 	assert(INVALID_SHMID != udi->shmid);
 	if (attempt_ipc_cleanup)
 	{
-		i_am_the_last_user = (((status = shmctl(udi->shmid, IPC_STAT, &shm_buf)) == 0) && (1 == shm_buf.shm_nattch));
+		/* From time to time we see processes that don't terminate quickly enough (or don't get waited on
+		 * in the <defunct> state gumming up our cleanup attempt.  If that happens (ie: !i_am_the_last_user)
+		 * we wait a little while (2 sec max) to allow everyone to get out of our way.  If that still doesn't
+		 * clear up, we note the fact and move on */
+		for (sleep_counter = SLEEP_TWO_SEC;; sleep_counter--)
+		{
+			i_am_the_last_user = (((status = shmctl(udi->shmid, IPC_STAT, &shm_buf)) == 0) &&
+				(1 == (n_attached = shm_buf.shm_nattch))); /* WARNING: assignment */
+			if (i_am_the_last_user || (0 == sleep_counter))
+				break;
+			if (SLEEP_TWO_SEC == sleep_counter) /* Just note once */
+				repl_log(stderr, TRUE, TRUE,
+					"Waiting for sole jnlpool access (%d other processes still attached).\n",
+					n_attached - 1);
+
+			wcs_sleep(1);
+		}
 		if (!i_am_the_last_user)
 		{
 			if (status < 0)
 				repl_log(stderr, TRUE, TRUE, "Error in jnlpool shmctl : %s\n", STRERROR(ERRNO));
 			else
-				repl_log(stderr, TRUE, TRUE, "Not deleting jnlpool ipcs. %d processes still attached to jnlpool\n",
-					 shm_buf.shm_nattch - 1);
+			{
+				/* If we still can't delete, say how many others are accessing, the shm create pid &
+				 * and the shm last access pid */
+				repl_log(stderr, TRUE, TRUE,
+					"Not deleting jnlpool ipcs. %d processes still attached to "
+					"jnlpool:  last pid %d\n",
+					 shm_buf.shm_nattch - 1, shm_buf.shm_lpid);
+			}
 			attempt_ipc_cleanup = FALSE;
 			*exit_status = ABNORMAL_SHUTDOWN;
 		} else if (INVALID_SHMID != udi->shmid)
 		{
-			if (INVALID_SEMID != jnlpool.repl_inst_filehdr->recvpool_semid DEBUG_ONLY(&&
+			if (INVALID_SEMID != jnlpool->repl_inst_filehdr->recvpool_semid DEBUG_ONLY(&&
 						!(gtm_white_box_test_case_enabled &&
 							(WBTEST_UPD_PROCESS_ERROR == gtm_white_box_test_case_number))))
 				repl_log(stderr, TRUE, TRUE, "Receiver pool semaphore IDs were not removed\n");
-			if ((INVALID_SHMID != jnlpool.repl_inst_filehdr->recvpool_shmid) DEBUG_ONLY(&&
+			if ((INVALID_SHMID != jnlpool->repl_inst_filehdr->recvpool_shmid) DEBUG_ONLY(&&
 						!(gtm_white_box_test_case_enabled &&
 							 (WBTEST_UPD_PROCESS_ERROR == gtm_white_box_test_case_number))))
 							 repl_log(stderr, TRUE, TRUE, "Receiver pool shared memory not removed\n");
+			DEBUG_ONLY(only_usr_jnlpool_flush = true;)
 			repl_inst_flush_jnlpool(TRUE, TRUE);
+			DEBUG_ONLY(only_usr_jnlpool_flush = false;)
 		}
 	}
+	/* We need to keep the journal pool attached when IFOE is configured so that we can check for instance freeze
+	 * in database rundown.
+	 * However, if we are about to cleanup ipcs, it doesn't matter, so we are better off cleaning up.
+	 */
+	if (INST_FREEZE_ON_ERROR_POLICY && !(attempt_ipc_cleanup && (INVALID_SHMID != udi->shmid)))
+		return FALSE;
 	/* detach from shared memory irrespective of whether we need to cleanup ipcs or not */
-	JNLPOOL_SHMDT(detach_status, save_errno);
-	if (0 == detach_status)
-	{
-		jnlpool.jnlpool_ctl = NULL; /* Detached successfully */
-		jnlpool_ctl = NULL;
-		jnlpool.repl_inst_filehdr = NULL;
-		jnlpool.gtmsrc_lcl_array = NULL;
-		jnlpool.gtmsource_local_array = NULL;
-		jnlpool.jnldata_base = NULL;
-		pool_init = FALSE;
-	} else
+	JNLPOOL_SHMDT(jnlpool, detach_status, save_errno);
+	if (0 != detach_status)
 	{
 		repl_log(stderr, TRUE, TRUE, "Error detaching from Journal Pool : %s\n", STRERROR(save_errno));
 		attempt_ipc_cleanup = FALSE;
@@ -182,7 +218,7 @@ int	gtmrecv_ipc_cleanup(boolean_t auto_shutdown, int *exit_status)
 	if (!i_am_the_last_user)
 	{
 		if (status < 0)
-			repl_log(stderr, TRUE, TRUE, "Error in jnlpool shmctl : %s\n", STRERROR(errno));
+			repl_log(stderr, TRUE, TRUE, "Error in recvpool shmctl : %s\n", STRERROR(errno));
 		else
 			repl_log(stderr, TRUE, TRUE,
 				"Not deleting receive pool ipcs. %d processes still attached to receive pool\n",
