@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2006, 2013 Fidelity Information Services, Inc	*
+ * Copyright (c) 2006-2018 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -9,8 +10,8 @@
  *								*
  ****************************************************************/
 
-#ifndef _REPL_MSG_H
-#define _REPL_MSG_H
+#ifndef REPL_MSG_H_INCLUDED
+#define REPL_MSG_H_INCLUDED
 
 enum
 {
@@ -66,10 +67,11 @@ enum
 #define	REPL_PROTO_VER_MULTISITE_CMP	(char)0x2	/* Versions V5.3-003 and above that support multisite replication with
 							 * the ability to compress the logical records in the replication pipe.
 							 */
-#define	REPL_PROTO_VER_SUPPLEMENTARY	(char)0x3	/* Versions V5.5-000 and above that support supplementary instances */
-#define	REPL_PROTO_VER_REMOTE_LOGPATH	(char)0x4	/* Versions V6.0-003 and above that send remote $CWD as part of handshake */
-#define REPL_PROTO_VER_TLS_SUPPORT	(char)0x5	/* Versions V6.1-000 and above that supports SSL/TLS communication. */
-#define	REPL_PROTO_VER_THIS		REPL_PROTO_VER_TLS_SUPPORT
+#define	REPL_PROTO_VER_SUPPLEMENTARY	(char)0x3	/* Versions >= V5.5-000 that support supplementary instances */
+#define	REPL_PROTO_VER_REMOTE_LOGPATH	(char)0x4	/* Versions >= V6.0-003 that send remote $CWD as part of handshake */
+#define REPL_PROTO_VER_TLS_SUPPORT	(char)0x5	/* Versions >= V6.1-000 that supports SSL/TLS communication. */
+#define REPL_PROTO_VER_XENDIANFIXES	(char)0x6	/* Versions >= V6.2-001 support cross-endian replication (GTM-8205) */
+#define	REPL_PROTO_VER_THIS		REPL_PROTO_VER_XENDIANFIXES
 							/* The current/latest version of the communication protocol between the
 							 * primary (source server) and secondary (receiver server or rollback)
 							 */
@@ -115,6 +117,8 @@ enum
 #define	REPL_MSG_LEN	(uint4)(SIZEOF(int4))
 #define	REPL_MSG_HDRLEN	(uint4)(REPL_MSG_TYPE + REPL_MSG_LEN) /* For type and len fields */
 
+#define REPL_MSG_SIZE	1024 + 1
+
 typedef struct	/* Used also to send a message of type REPL_INST_NOHIST */
 {
 	int4		type;
@@ -162,6 +166,7 @@ typedef struct	/* Used also to send a message of type REPL_TR_CMP_JNL_RECS2 */
  */
 #define	MAX_REPL_MSGLEN	(2 * 1024 * 1024)
 #define MAX_TR_BUFFSIZE	(MAX_REPL_MSGLEN - REPL_MSG_HDRLEN2) /* allow for biggest replication message header */
+#define	MAX_REPL_OPMSG_LEN 256 + 1
 
 typedef struct	/* used to send a message of type REPL_START_JNL_SEQNO */
 {
@@ -225,7 +230,8 @@ typedef struct	/* used to send a message of type REPL_NEED_INSTINFO */
 					 * the macros REPL_PROTO_VER_DUALSITE (0) and REPL_PROTO_VER_UNINITIALIZED (-1) */
 	char		is_rootprimary;	/* Whether the source server that is sending this message is a root primary or not. */
 	char		is_supplementary;	/* TRUE for a supplementary instance; FALSE otherwise */
-	char		filler_32[5];
+	char		jnl_ver;	/* jnl format of this side */
+	char		filler_32[4];
 } repl_needinst_msg_t; /* The first two fields should be as in repl_msg_t */
 
 typedef struct	/* used to send a message of type REPL_OLD_INSTANCE_INFO */
