@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2014-2022 Fidelity National Information	*
+ * Copyright (c) 2014-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -458,7 +458,7 @@ void	remq_rtnobj_specific(que_ent_ptr_t que_base, sm_uc_ptr_t shm_base, rtnobj_h
 sm_uc_ptr_t rtnobj_shm_malloc(zro_hist *zhist, int fd, off_t objSize, gtm_uint64_t objhash)
 {
 	boolean_t		has_relinkctl_lock, has_rtnobj_lock;
-	boolean_t		initialized, return_NULL, need_shmctl;
+	boolean_t		initialized, need_shmctl, return_NULL = FALSE;
 	char			errstr[256];
 	gtm_uint64_t		src_cksum_8byte;
 	int			min_index, max_index, min_free_index, max_free_index, shm_index;
@@ -489,7 +489,7 @@ sm_uc_ptr_t rtnobj_shm_malloc(zro_hist *zhist, int fd, off_t objSize, gtm_uint64
 	struct perm_diag_data	pdd;
 #	ifdef DEBUG
 	boolean_t		rtnobj_integ_done = FALSE;
-	rtnobj_sm_off_t		shm_index_off_mid;
+	rtnobj_sm_off_t		shm_index_off_mid = NULL_RTNOBJ_SM_OFF_T;
 	int			dbg_shm_cnt;
 	DCL_THREADGBL_ACCESS;
 
@@ -568,9 +568,10 @@ sm_uc_ptr_t rtnobj_shm_malloc(zro_hist *zhist, int fd, off_t objSize, gtm_uint64
 					shm_index_off = rtnobj->next_rtnobj_shm_offset;
 				}
 				if (MAXVERS_TEST < maxvers)
-					// non-head node as the loop point when there is more than 2 elements
+				{	/* non-head node as the loop point when there is more than 2 elements */
+					assert(NULL_RTNOBJ_SM_OFF_T != shm_index_off_mid);
 					rtnobj->next_rtnobj_shm_offset = shm_index_off_mid;
-				else
+				} else
 					rtnobj->next_rtnobj_shm_offset = relinkrec->rtnobj_shm_offset;
 				maxvers = -1;
 				rtnobj_integ_done = TRUE;
@@ -691,7 +692,7 @@ sm_uc_ptr_t rtnobj_shm_malloc(zro_hist *zhist, int fd, off_t objSize, gtm_uint64
 			assert(((size_t)1 << shmSizeIndex) == shm_size);
 			assert(shmSizeIndex >= (shm_index + MIN_RTNOBJ_SHM_INDEX));
 			shm_index = shmSizeIndex - MIN_RTNOBJ_SHM_INDEX;
-			shmid = gtm_shmget(IPC_PRIVATE, shm_size, RWDALL | IPC_CREAT, TRUE);
+			shmid = gtm_shmget(IPC_PRIVATE, shm_size, RWDALL | IPC_CREAT, TRUE, RELINK, linkctl->relinkctl_path);
 			if (-1 == shmid)
 			{
 				save_errno = errno;
@@ -926,13 +927,9 @@ sm_uc_ptr_t rtnobj_shm_malloc(zro_hist *zhist, int fd, off_t objSize, gtm_uint64
 		relinkrec->numvers++;
 		/* Fill the shared memory buffer with the object code */
 		objBuff = (sm_uc_ptr_t)&rtnobj->userStorage.userStart;
-		if (-1 == lseek(fd, NATIVE_HDR_LEN, SEEK_SET))
+		DOPREADRL(fd, objBuff, objSize, NATIVE_HDR_LEN, actualSize);
+		if ((-1 == actualSize) || (actualSize != objSize))
 			return_NULL = TRUE;
-		else
-		{
-			DOREADRL(fd, objBuff, objSize, actualSize);
-			return_NULL = (actualSize != objSize);
-		}
 		if (return_NULL)
 		{
 			/* Free allocated object in shared memory using rtnobj_shm_free. Set up a dummy rtnhdr
@@ -961,7 +958,11 @@ sm_uc_ptr_t rtnobj_shm_malloc(zro_hist *zhist, int fd, off_t objSize, gtm_uint64
 			relinkrec->rtnname_fixed.c, (UINTPTR_T)objhash,				\
 			(UINTPTR_T)linkctl, rtnobj->refcnt,					\
 			(UINTPTR_T)objBuff, relinkrec->numvers);)
-		relinkrec->objhash = objhash;
+		if (relinkrec->objhash != objhash)
+		{
+			++relinkrec->rtnobj_supersede;
+			relinkrec->objhash = objhash;
+		}
 		++relinkrec->cycle;	/* cycle bump is because of the implicit ZRUPDATE that linking a routine does */
 		if (0 == relinkrec->cycle)
 			relinkrec->cycle = 1;	/* cycle value of 0 has special meaning, so bump to 1 in case we roll over */
@@ -983,7 +984,7 @@ void	rtnobj_shm_free(rhdtyp *rhead, boolean_t latch_grabbed)
 	open_relinkctl_sgm	*linkctl;
 	relinkshm_hdr_t		*shm_hdr;
 	int			maxObjIndex, sizeIndex;
-	int			min_index, max_index, min_free_index, max_free_index, shm_index;
+	int			min_index, max_index, min_free_index, max_free_index, shm_index = -1;
 	gtm_uint64_t		elemSize, origElemSize, objLen;
 	rtnobj_hdr_t		*rtnobj, *prev_rtnobj, *rtnobj2;
 	rtnobj_sm_off_t		shm_index_off;
