@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2010-2022 Fidelity National Information	*
+ * Copyright (c) 2010-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -405,7 +405,7 @@ STATICFNDEF int4 update_trigger_name(char *trigvn, int trigvn_len, int trigger_i
 
 	SETUP_THREADGBL_ACCESS;
 	retval = NO_NAME_CHANGE;
-	if (tf_trig_name_len && (tf_trig_name_len != STRLEN(db_trig_name) - 1)
+	if ((tf_trig_name_len && (tf_trig_name_len != STRLEN(db_trig_name) - 1))
 		|| memcmp(tf_trig_name, db_trig_name, tf_trig_name_len))
 	{
 		if (!validate_label(trigvn, trigvn_len))
@@ -1143,7 +1143,7 @@ boolean_t trigger_update_rec(mval *trigger_rec, boolean_t noprompt, uint4 *trig_
 	mname_entry		gvname;
 	int4			max_len;
 	boolean_t 		multi_line, multi_line_xecute;
-	int4			rec_len;
+	int4			rec_len = -1;
 	int4			rec_num;
 	boolean_t		status;
 	char			tfile_rec_val[MAX_BUFF_SIZE];
@@ -1161,7 +1161,7 @@ boolean_t trigger_update_rec(mval *trigger_rec, boolean_t noprompt, uint4 *trig_
 	char			xecute_buffer[MAX_BUFF_SIZE + MAX_XECUTE_LEN];
 	unsigned char		*dispbuff;
 	mval			multi_jrec, *trigjrec;
-	char			*trigjrecptr;
+	char			*trigjrecptr = NULL;
 	int			trigjreclen;
 	io_pair			io_save_device;
 	int4			max_xecute_size;
@@ -1411,8 +1411,8 @@ boolean_t trigger_update_rec(mval *trigger_rec, boolean_t noprompt, uint4 *trig_
 	{	/* If this is a not a multi-line xecute string, we dont expect newlines. The only exception is if it is
 		 * the last byte in the string.
 		 */
-		 assert(memchr(trigjrec->str.addr, '\n', trigjrec->str.len) == trigjrecptr);
-		 *trigjrecptr++;
+		assert(memchr(trigjrec->str.addr, '\n', trigjrec->str.len) == trigjrecptr);
+		trigjrecptr++;
 		if (trigjrecptr != (trigjrec->str.addr + trigjrec->str.len))
 		{
 			util_out_print_gtmio("Error : Newline allowed only inside multi-line xecute in $ztrigger ITEM", FLUSH);
@@ -1453,6 +1453,9 @@ boolean_t trigger_update_rec(mval *trigger_rec, boolean_t noprompt, uint4 *trig_
 		memcpy(disp_trigvn, trigvn, trigvn_len);
 		disp_trigvn_len = trigvn_len;
 		disp_trigvn[disp_trigvn_len] = '\0';	/* null terminate just in case */
+		reg_index = -1;
+		min_reg_index = -1;
+		max_reg_index = -1;
 	}
 	jnl_format_done = FALSE;
 	new_name_check_done = FALSE;
@@ -1482,6 +1485,9 @@ boolean_t trigger_update_rec(mval *trigger_rec, boolean_t noprompt, uint4 *trig_
 		/* else if (STATS_ERROR_TRIGFILE == overall_trig_status)   : it is already what it should be */
 		if (NULL == gvspan)
 			break;
+		assert(-1 != reg_index);
+		assert(-1 != max_reg_index);
+		assert(-1 != min_reg_index);
 		if (reg_index >= max_reg_index)
 			break;
 		do
