@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2010 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2017 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -22,7 +23,7 @@ static char rcsid[] = "$Header:$";
 
 #include "mdef.h"
 
-#include "gtm_stdlib.h"		/* for exit() */
+#include "gtm_stdlib.h"		/* for EXIT() */
 
 #include "error.h"
 #include "gtcm.h"
@@ -33,12 +34,12 @@ static char rcsid[] = "$Header:$";
 
 #include "gv_rundown.h"		/* for gv_rundown() prototype */
 #include "op.h"			/* for op_unlock() and op_lkinit() prototype */
+#include "gtmcrypt.h"
 
 GBLREF int4	 gtcm_exi_condition;
 
 void gtcm_exit()
 {
-	op_lkinit();
 	op_unlock();
 	SET_PROCESS_EXITING_TRUE;
 	gv_rundown();
@@ -46,5 +47,6 @@ void gtcm_exit()
 	rc_delete_cpt();
 	rc_rundown();
 #endif
-	exit(gtcm_exi_condition);
+	GTMCRYPT_CLOSE;
+	EXIT(gtcm_exi_condition);
 }
