@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2009 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2024 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -27,7 +28,7 @@ static char rcsid[] = "$Header:$";
 
 
 rc_oflow *
-rc_oflow_alc()
+rc_oflow_alc(void)
 {
     rc_oflow	*rv;
 
@@ -42,8 +43,7 @@ rc_oflow_alc()
 
 
 void
-rc_oflow_fin(fop)
-    rc_oflow *fop;
+rc_oflow_fin(rc_oflow *fop)
 {
 
     if (!fop)
