@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2015 Fidelity National Information	*
+ * Copyright (c) 2001-2023 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -42,8 +42,9 @@ struct CLB *cmu_getclb(cmi_descriptor *node, cmi_descriptor *task)
 				qp = RELQUE2PTR(p->cqe.fl))
 		{
 			p = QUEENT2CLB(qp, cqe);
-			if (0 == memcpy(ai_ptr->ai_addr, (sockaddr_ptr)(&p->peer_sas), ai_ptr->ai_addrlen))
+			if (0 == memcmp(ai_ptr->ai_addr, (sockaddr_ptr)(&p->peer_sas), ai_ptr->ai_addrlen))
 			{
+				freeaddrinfo(ai_ptr);
 				SIGPROCMASK(SIG_SETMASK, &oset, NULL, rc);
 				return p;
 			}
