@@ -1,6 +1,6 @@
 /****************************************************************
  *                                                              *
- * Copyright (c) 2007-2016 Fidelity National Information	*
+ * Copyright (c) 2007-2023 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *                                                              *
  *      This source code contains the intellectual property     *
@@ -129,7 +129,7 @@ GBLREF emit_base_info_struct	emit_base_info;
 GBLREF int 			call_4lcldo_variant;
 GBLREF int			jmp_offset;	/* Offset to jump target */
 GBLREF char			cg_phase;	/* Code generation phase */
-GBLREF char 			code_buf[];
+GBLREF unsigned char		code_buf[];
 GBLREF int			code_idx;
 GBLREF unsigned char		*obpt;	 	/* Output buffer index */
 GBLREF unsigned char		outbuf[];
@@ -469,12 +469,14 @@ void set_memory_reg()
 		instruction.destination_operand_reg = (char *)register_list[modrm_byte.modrm.r_m + 8 * rex_prefix.Base];
 	/* Printing of RIP has to be handled differently	*/
  	if (instruction.reg_rip)
+ 	{
 		if (instruction.source_operand_class == memory_class)
 			instruction.source_operand_reg = (char *)register_list[REG_RIP];
 		else if (instruction.destination_operand_class == memory_class)
 			instruction.destination_operand_reg = (char *)register_list[REG_RIP];
 		else
 			assertpro(FALSE);
+ 	}
 }
 
 void set_register_reg()
