@@ -100,6 +100,7 @@ LITDEF	err_msg gdeerrors[] = {
 	{ "GDEASYNCIONOMM", "!AD segment has ASYNCIO turned on. Cannot support MM access method.", 2, 0 },
 	{ "NOPERCENTY", "^%Y* is a reserved global name in GT.M", 0, 0 },
 	{ "GDELOGFAIL", "GDE failed to log command. Check operator log for more information", 0, 0 },
+	{ "INVALIDAUTODELDB", "Autodelete DB must be specified as non-journaled autodb", 0, 0 },
 };
 
 LITDEF	int GDE_BLKSIZ512 = 150503435;
@@ -189,6 +190,7 @@ LITDEF	int GDE_GBLNAMCOLLVER = 150504098;
 LITDEF	int GDE_GDEASYNCIONOMM = 150504106;
 LITDEF	int GDE_NOPERCENTY = 150504114;
 LITDEF	int GDE_GDELOGFAIL = 150504122;
+LITDEF	int GDE_INVALIDAUTODELDB = 150504130;
 
 
 LITDEF	int gdeerrors_undocarr[] = {
@@ -200,7 +202,7 @@ GBLDEF	err_ctl gdeerrors_ctl = {
 	248,
 	"GDE",
 	&gdeerrors[0],
-	87,
+	88,
 	&gdeerrors_undocarr[0],
 	0
 };
